import unittest

from pyantv import options as opts
from pyantv.charts import GeoPath
from pyantv.commons.utils import JsCode
from pyantv.globals import ChartType

from test import chart_base_test

TEST_GEO_PATH_JOIN_DATA = [
    {"id": 1001, "state": "Alabama", "county": "Autauga County", "rate": 5.1},
    {"id": 1003, "state": "Alabama", "county": "Baldwin County", "rate": 4.9},
    {"id": 1005, "state": "Alabama", "county": "Barbour County", "rate": 8.6},
    {"id": 1007, "state": "Alabama", "county": "Bibb County", "rate": 6.2},
    {"id": 1009, "state": "Alabama", "county": "Blount County", "rate": 5.1},
    {"id": 1011, "state": "Alabama", "county": "Bullock County", "rate": 7.1},
    {"id": 1013, "state": "Alabama", "county": "Butler County", "rate": 6.7},
    {"id": 1015, "state": "Alabama", "county": "Calhoun County", "rate": 6.1},
    {"id": 1017, "state": "Alabama", "county": "Chambers County", "rate": 5},
    {"id": 1019, "state": "Alabama", "county": "Cherokee County", "rate": 5},
    {"id": 1021, "state": "Alabama", "county": "Chilton County", "rate": 5.2},
    {"id": 1023, "state": "Alabama", "county": "Choctaw County", "rate": 7.9},
    {"id": 1025, "state": "Alabama", "county": "Clarke County", "rate": 11.1},
    {"id": 1027, "state": "Alabama", "county": "Clay County", "rate": 5.9},
    {"id": 1029, "state": "Alabama", "county": "Cleburne County", "rate": 5.5},
    {"id": 1031, "state": "Alabama", "county": "Coffee County", "rate": 5.6},
    {"id": 1033, "state": "Alabama", "county": "Colbert County", "rate": 6.5},
    {"id": 1035, "state": "Alabama", "county": "Conecuh County", "rate": 7.7},
    {"id": 1037, "state": "Alabama", "county": "Coosa County", "rate": 5.7},
    {"id": 1039, "state": "Alabama", "county": "Covington County", "rate": 6.7},
    {"id": 1041, "state": "Alabama", "county": "Crenshaw County", "rate": 5.7},
    {"id": 1043, "state": "Alabama", "county": "Cullman County", "rate": 4.8},
    {"id": 1045, "state": "Alabama", "county": "Dale County", "rate": 5.6},
    {"id": 1047, "state": "Alabama", "county": "Dallas County", "rate": 9.5},
    {"id": 1049, "state": "Alabama", "county": "DeKalb County", "rate": 5.7},
    {"id": 1051, "state": "Alabama", "county": "Elmore County", "rate": 4.7},
    {"id": 1053, "state": "Alabama", "county": "Escambia County", "rate": 6.3},
    {"id": 1055, "state": "Alabama", "county": "Etowah County", "rate": 5.7},
    {"id": 1057, "state": "Alabama", "county": "Fayette County", "rate": 6.6},
    {"id": 1059, "state": "Alabama", "county": "Franklin County", "rate": 5.5},
    {"id": 1061, "state": "Alabama", "county": "Geneva County", "rate": 5.4},
    {"id": 1063, "state": "Alabama", "county": "Greene County", "rate": 9.3},
    {"id": 1065, "state": "Alabama", "county": "Hale County", "rate": 7.6},
    {"id": 1067, "state": "Alabama", "county": "Henry County", "rate": 6.3},
    {"id": 1069, "state": "Alabama", "county": "Houston County", "rate": 5.6},
    {"id": 1071, "state": "Alabama", "county": "Jackson County", "rate": 5.9},
    {"id": 1073, "state": "Alabama", "county": "Jefferson County", "rate": 5.5},
    {"id": 1075, "state": "Alabama", "county": "Lamar County", "rate": 5.2},
    {"id": 1077, "state": "Alabama", "county": "Lauderdale County", "rate": 6},
    {"id": 1079, "state": "Alabama", "county": "Lawrence County", "rate": 6.4},
    {"id": 1081, "state": "Alabama", "county": "Lee County", "rate": 4.9},
    {"id": 1083, "state": "Alabama", "county": "Limestone County", "rate": 5},
    {"id": 1085, "state": "Alabama", "county": "Lowndes County", "rate": 10.3},
    {"id": 1087, "state": "Alabama", "county": "Macon County", "rate": 7.2},
    {"id": 1089, "state": "Alabama", "county": "Madison County", "rate": 4.9},
    {"id": 1091, "state": "Alabama", "county": "Marengo County", "rate": 6.9},
    {"id": 1093, "state": "Alabama", "county": "Marion County", "rate": 6.1},
    {"id": 1095, "state": "Alabama", "county": "Marshall County", "rate": 5.1},
    {"id": 1097, "state": "Alabama", "county": "Mobile County", "rate": 6.5},
    {"id": 1099, "state": "Alabama", "county": "Monroe County", "rate": 8.6},
    {
        "id": 1101,
        "state": "Alabama",
        "county": "Montgomery County",
        "rate": 5.6,
    },
    {"id": 1103, "state": "Alabama", "county": "Morgan County", "rate": 5.2},
    {"id": 1105, "state": "Alabama", "county": "Perry County", "rate": 10.9},
    {"id": 1107, "state": "Alabama", "county": "Pickens County", "rate": 6.7},
    {"id": 1109, "state": "Alabama", "county": "Pike County", "rate": 6.4},
    {"id": 1111, "state": "Alabama", "county": "Randolph County", "rate": 5.7},
    {"id": 1113, "state": "Alabama", "county": "Russell County", "rate": 5.3},
    {"id": 1115, "state": "Alabama", "county": "St. Clair County", "rate": 5},
    {"id": 1117, "state": "Alabama", "county": "Shelby County", "rate": 4.2},
    {"id": 1119, "state": "Alabama", "county": "Sumter County", "rate": 7.3},
    {"id": 1121, "state": "Alabama", "county": "Talladega County", "rate": 6.4},
    {
        "id": 1123,
        "state": "Alabama",
        "county": "Tallapoosa County",
        "rate": 5.1,
    },
    {
        "id": 1125,
        "state": "Alabama",
        "county": "Tuscaloosa County",
        "rate": 5.5,
    },
    {"id": 1127, "state": "Alabama", "county": "Walker County", "rate": 7.2},
    {
        "id": 1129,
        "state": "Alabama",
        "county": "Washington County",
        "rate": 8.3,
    },
    {"id": 1131, "state": "Alabama", "county": "Wilcox County", "rate": 13.9},
    {"id": 1133, "state": "Alabama", "county": "Winston County", "rate": 7},
    {
        "id": 2013,
        "state": "Alaska",
        "county": "Aleutians East Borough",
        "rate": 1.9,
    },
    {
        "id": 2016,
        "state": "Alaska",
        "county": "Aleutians West Census Area",
        "rate": 2.2,
    },
    {
        "id": 2020,
        "state": "Alaska",
        "county": "Anchorage Municipality",
        "rate": 5,
    },
    {
        "id": 2050,
        "state": "Alaska",
        "county": "Bethel Census Area",
        "rate": 14.1,
    },
    {
        "id": 2060,
        "state": "Alaska",
        "county": "Bristol Bay Borough",
        "rate": 5.6,
    },
    {"id": 2068, "state": "Alaska", "county": "Denali Borough", "rate": 3.2},
    {
        "id": 2070,
        "state": "Alaska",
        "county": "Dillingham Census Area",
        "rate": 8.1,
    },
    {
        "id": 2090,
        "state": "Alaska",
        "county": "Fairbanks North Star Borough",
        "rate": 4.9,
    },
    {"id": 2100, "state": "Alaska", "county": "Haines Borough", "rate": 5.9},
    {
        "id": 2105,
        "state": "Alaska",
        "county": "Hoonah-Angoon Census Area",
        "rate": 7.3,
    },
    {
        "id": 2110,
        "state": "Alaska",
        "county": "Juneau City and Borough",
        "rate": 3.6,
    },
    {
        "id": 2122,
        "state": "Alaska",
        "county": "Kenai Peninsula Borough",
        "rate": 6.9,
    },
    {
        "id": 2130,
        "state": "Alaska",
        "county": "Ketchikan Gateway Borough",
        "rate": 4.4,
    },
    {
        "id": 2150,
        "state": "Alaska",
        "county": "Kodiak Island Borough",
        "rate": 4.8,
    },
    {
        "id": 2158,
        "state": "Alaska",
        "county": "Kusilvak Census Area",
        "rate": 21.7,
    },
    {
        "id": 2164,
        "state": "Alaska",
        "county": "Lake and Peninsula Borough",
        "rate": 9.2,
    },
    {
        "id": 2170,
        "state": "Alaska",
        "county": "Matanuska-Susitna Borough",
        "rate": 7.4,
    },
    {"id": 2180, "state": "Alaska", "county": "Nome Census Area", "rate": 13.6},
    {
        "id": 2185,
        "state": "Alaska",
        "county": "North Slope Borough",
        "rate": 6.9,
    },
    {
        "id": 2188,
        "state": "Alaska",
        "county": "Northwest Arctic Borough",
        "rate": 16.5,
    },
    {
        "id": 2195,
        "state": "Alaska",
        "county": "Petersburg Census Area",
        "rate": 5.7,
    },
    {
        "id": 2198,
        "state": "Alaska",
        "county": "Prince of Wales-Hyder Census Area",
        "rate": 9.4,
    },
    {
        "id": 2220,
        "state": "Alaska",
        "county": "Sitka City and Borough",
        "rate": 3,
    },
    {
        "id": 2230,
        "state": "Alaska",
        "county": "Skagway Municipality",
        "rate": 3,
    },
    {
        "id": 2240,
        "state": "Alaska",
        "county": "Southeast Fairbanks Census Area",
        "rate": 8.7,
    },
    {
        "id": 2261,
        "state": "Alaska",
        "county": "Valdez-Cordova Census Area",
        "rate": 5.4,
    },
    {
        "id": 2275,
        "state": "Alaska",
        "county": "Wrangell City and Borough",
        "rate": 5.4,
    },
    {
        "id": 2282,
        "state": "Alaska",
        "county": "Yakutat City and Borough",
        "rate": 5.8,
    },
    {
        "id": 2290,
        "state": "Alaska",
        "county": "Yukon-Koyukuk Census Area",
        "rate": 16.4,
    },
    {"id": 4001, "state": "Arizona", "county": "Apache County", "rate": 11.8},
    {"id": 4003, "state": "Arizona", "county": "Cochise County", "rate": 6.5},
    {"id": 4005, "state": "Arizona", "county": "Coconino County", "rate": 6.4},
    {"id": 4007, "state": "Arizona", "county": "Gila County", "rate": 7.6},
    {"id": 4009, "state": "Arizona", "county": "Graham County", "rate": 7.3},
    {"id": 4011, "state": "Arizona", "county": "Greenlee County", "rate": 7.7},
    {"id": 4012, "state": "Arizona", "county": "La Paz County", "rate": 6.6},
    {"id": 4013, "state": "Arizona", "county": "Maricopa County", "rate": 4.9},
    {"id": 4015, "state": "Arizona", "county": "Mohave County", "rate": 7},
    {"id": 4017, "state": "Arizona", "county": "Navajo County", "rate": 8.5},
    {"id": 4019, "state": "Arizona", "county": "Pima County", "rate": 5.4},
    {"id": 4021, "state": "Arizona", "county": "Pinal County", "rate": 5.9},
    {"id": 4023, "state": "Arizona", "county": "Santa Cruz County", "rate": 13},
    {"id": 4025, "state": "Arizona", "county": "Yavapai County", "rate": 5},
    {"id": 4027, "state": "Arizona", "county": "Yuma County", "rate": 24.4},
    {"id": 5001, "state": "Arkansas", "county": "Arkansas County", "rate": 3.2},
    {"id": 5003, "state": "Arkansas", "county": "Ashley County", "rate": 6.3},
    {"id": 5005, "state": "Arkansas", "county": "Baxter County", "rate": 4.1},
    {"id": 5007, "state": "Arkansas", "county": "Benton County", "rate": 2.7},
    {"id": 5009, "state": "Arkansas", "county": "Boone County", "rate": 3.3},
    {"id": 5011, "state": "Arkansas", "county": "Bradley County", "rate": 5.9},
    {"id": 5013, "state": "Arkansas", "county": "Calhoun County", "rate": 5.1},
    {"id": 5015, "state": "Arkansas", "county": "Carroll County", "rate": 2.8},
    {"id": 5017, "state": "Arkansas", "county": "Chicot County", "rate": 6},
    {"id": 5019, "state": "Arkansas", "county": "Clark County", "rate": 4.5},
    {"id": 5021, "state": "Arkansas", "county": "Clay County", "rate": 5},
    {"id": 5023, "state": "Arkansas", "county": "Cleburne County", "rate": 5.7},
    {
        "id": 5025,
        "state": "Arkansas",
        "county": "Cleveland County",
        "rate": 4.4,
    },
    {"id": 5027, "state": "Arkansas", "county": "Columbia County", "rate": 5.7},
    {"id": 5029, "state": "Arkansas", "county": "Conway County", "rate": 5.9},
    {
        "id": 5031,
        "state": "Arkansas",
        "county": "Craighead County",
        "rate": 3.1,
    },
    {"id": 5033, "state": "Arkansas", "county": "Crawford County", "rate": 3.7},
    {
        "id": 5035,
        "state": "Arkansas",
        "county": "Crittenden County",
        "rate": 4.4,
    },
    {"id": 5037, "state": "Arkansas", "county": "Cross County", "rate": 3.9},
    {"id": 5039, "state": "Arkansas", "county": "Dallas County", "rate": 5},
    {"id": 5041, "state": "Arkansas", "county": "Desha County", "rate": 5.1},
    {"id": 5043, "state": "Arkansas", "county": "Drew County", "rate": 5.7},
    {"id": 5045, "state": "Arkansas", "county": "Faulkner County", "rate": 3.7},
    {"id": 5047, "state": "Arkansas", "county": "Franklin County", "rate": 3.9},
    {"id": 5049, "state": "Arkansas", "county": "Fulton County", "rate": 3.9},
    {"id": 5051, "state": "Arkansas", "county": "Garland County", "rate": 4},
    {"id": 5053, "state": "Arkansas", "county": "Grant County", "rate": 3.2},
    {"id": 5055, "state": "Arkansas", "county": "Greene County", "rate": 4},
    {
        "id": 5057,
        "state": "Arkansas",
        "county": "Hempstead County",
        "rate": 3.7,
    },
    {
        "id": 5059,
        "state": "Arkansas",
        "county": "Hot Spring County",
        "rate": 3.6,
    },
    {"id": 5061, "state": "Arkansas", "county": "Howard County", "rate": 3.4},
    {
        "id": 5063,
        "state": "Arkansas",
        "county": "Independence County",
        "rate": 5,
    },
    {"id": 5065, "state": "Arkansas", "county": "Izard County", "rate": 5.3},
    {"id": 5067, "state": "Arkansas", "county": "Jackson County", "rate": 5.9},
    {
        "id": 5069,
        "state": "Arkansas",
        "county": "Jefferson County",
        "rate": 5.9,
    },
    {"id": 5071, "state": "Arkansas", "county": "Johnson County", "rate": 4.8},
    {
        "id": 5073,
        "state": "Arkansas",
        "county": "Lafayette County",
        "rate": 6.2,
    },
    {"id": 5075, "state": "Arkansas", "county": "Lawrence County", "rate": 4.5},
    {"id": 5077, "state": "Arkansas", "county": "Lee County", "rate": 4.5},
    {"id": 5079, "state": "Arkansas", "county": "Lincoln County", "rate": 4.9},
    {
        "id": 5081,
        "state": "Arkansas",
        "county": "Little River County",
        "rate": 5.1,
    },
    {"id": 5083, "state": "Arkansas", "county": "Logan County", "rate": 4.4},
    {"id": 5085, "state": "Arkansas", "county": "Lonoke County", "rate": 3.2},
    {"id": 5087, "state": "Arkansas", "county": "Madison County", "rate": 3},
    {"id": 5089, "state": "Arkansas", "county": "Marion County", "rate": 3.8},
    {"id": 5091, "state": "Arkansas", "county": "Miller County", "rate": 4.3},
    {
        "id": 5093,
        "state": "Arkansas",
        "county": "Mississippi County",
        "rate": 7.1,
    },
    {"id": 5095, "state": "Arkansas", "county": "Monroe County", "rate": 4.5},
    {
        "id": 5097,
        "state": "Arkansas",
        "county": "Montgomery County",
        "rate": 4.6,
    },
    {"id": 5099, "state": "Arkansas", "county": "Nevada County", "rate": 3.7},
    {"id": 5101, "state": "Arkansas", "county": "Newton County", "rate": 3.6},
    {"id": 5103, "state": "Arkansas", "county": "Ouachita County", "rate": 5.6},
    {"id": 5105, "state": "Arkansas", "county": "Perry County", "rate": 4.4},
    {"id": 5107, "state": "Arkansas", "county": "Phillips County", "rate": 5.5},
    {"id": 5109, "state": "Arkansas", "county": "Pike County", "rate": 3.8},
    {"id": 5111, "state": "Arkansas", "county": "Poinsett County", "rate": 4.1},
    {"id": 5113, "state": "Arkansas", "county": "Polk County", "rate": 4.7},
    {"id": 5115, "state": "Arkansas", "county": "Pope County", "rate": 4.7},
    {"id": 5117, "state": "Arkansas", "county": "Prairie County", "rate": 3.7},
    {"id": 5119, "state": "Arkansas", "county": "Pulaski County", "rate": 3.5},
    {"id": 5121, "state": "Arkansas", "county": "Randolph County", "rate": 4.6},
    {
        "id": 5123,
        "state": "Arkansas",
        "county": "St. Francis County",
        "rate": 5,
    },
    {"id": 5125, "state": "Arkansas", "county": "Saline County", "rate": 3},
    {"id": 5127, "state": "Arkansas", "county": "Scott County", "rate": 3.7},
    {"id": 5129, "state": "Arkansas", "county": "Searcy County", "rate": 4.3},
    {
        "id": 5131,
        "state": "Arkansas",
        "county": "Sebastian County",
        "rate": 3.5,
    },
    {"id": 5133, "state": "Arkansas", "county": "Sevier County", "rate": 4.8},
    {"id": 5135, "state": "Arkansas", "county": "Sharp County", "rate": 5.2},
    {"id": 5137, "state": "Arkansas", "county": "Stone County", "rate": 4.8},
    {"id": 5139, "state": "Arkansas", "county": "Union County", "rate": 5.6},
    {
        "id": 5141,
        "state": "Arkansas",
        "county": "Van Buren County",
        "rate": 6.4,
    },
    {
        "id": 5143,
        "state": "Arkansas",
        "county": "Washington County",
        "rate": 2.6,
    },
    {"id": 5145, "state": "Arkansas", "county": "White County", "rate": 4.8},
    {"id": 5147, "state": "Arkansas", "county": "Woodruff County", "rate": 5.3},
    {"id": 5149, "state": "Arkansas", "county": "Yell County", "rate": 4.4},
    {
        "id": 6001,
        "state": "California",
        "county": "Alameda County",
        "rate": 4.6,
    },
    {"id": 6003, "state": "California", "county": "Alpine County", "rate": 7.3},
    {"id": 6005, "state": "California", "county": "Amador County", "rate": 5.7},
    {"id": 6007, "state": "California", "county": "Butte County", "rate": 6.6},
    {
        "id": 6009,
        "state": "California",
        "county": "Calaveras County",
        "rate": 5.7,
    },
    {"id": 6011, "state": "California", "county": "Colusa County", "rate": 10},
    {
        "id": 6013,
        "state": "California",
        "county": "Contra Costa County",
        "rate": 4.7,
    },
    {
        "id": 6015,
        "state": "California",
        "county": "Del Norte County",
        "rate": 7.3,
    },
    {
        "id": 6017,
        "state": "California",
        "county": "El Dorado County",
        "rate": 5.1,
    },
    {"id": 6019, "state": "California", "county": "Fresno County", "rate": 8.7},
    {"id": 6021, "state": "California", "county": "Glenn County", "rate": 7.9},
    {
        "id": 6023,
        "state": "California",
        "county": "Humboldt County",
        "rate": 5.1,
    },
    {
        "id": 6025,
        "state": "California",
        "county": "Imperial County",
        "rate": 26.4,
    },
    {"id": 6027, "state": "California", "county": "Inyo County", "rate": 4.9},
    {"id": 6029, "state": "California", "county": "Kern County", "rate": 9.7},
    {"id": 6031, "state": "California", "county": "Kings County", "rate": 8.9},
    {"id": 6033, "state": "California", "county": "Lake County", "rate": 6.3},
    {"id": 6035, "state": "California", "county": "Lassen County", "rate": 6.1},
    {
        "id": 6037,
        "state": "California",
        "county": "Los Angeles County",
        "rate": 5.3,
    },
    {"id": 6039, "state": "California", "county": "Madera County", "rate": 8.2},
    {"id": 6041, "state": "California", "county": "Marin County", "rate": 3.5},
    {
        "id": 6043,
        "state": "California",
        "county": "Mariposa County",
        "rate": 5.4,
    },
    {
        "id": 6045,
        "state": "California",
        "county": "Mendocino County",
        "rate": 5,
    },
    {"id": 6047, "state": "California", "county": "Merced County", "rate": 9.2},
    {"id": 6049, "state": "California", "county": "Modoc County", "rate": 6.6},
    {"id": 6051, "state": "California", "county": "Mono County", "rate": 5.2},
    {
        "id": 6053,
        "state": "California",
        "county": "Monterey County",
        "rate": 5.6,
    },
    {"id": 6055, "state": "California", "county": "Napa County", "rate": 4},
    {"id": 6057, "state": "California", "county": "Nevada County", "rate": 4.8},
    {"id": 6059, "state": "California", "county": "Orange County", "rate": 4.3},
    {"id": 6061, "state": "California", "county": "Placer County", "rate": 4.7},
    {"id": 6063, "state": "California", "county": "Plumas County", "rate": 7.1},
    {
        "id": 6065,
        "state": "California",
        "county": "Riverside County",
        "rate": 6.9,
    },
    {
        "id": 6067,
        "state": "California",
        "county": "Sacramento County",
        "rate": 5.7,
    },
    {
        "id": 6069,
        "state": "California",
        "county": "San Benito County",
        "rate": 6.2,
    },
    {
        "id": 6071,
        "state": "California",
        "county": "San Bernardino County",
        "rate": 6.2,
    },
    {
        "id": 6073,
        "state": "California",
        "county": "San Diego County",
        "rate": 5,
    },
    {
        "id": 6075,
        "state": "California",
        "county": "San Francisco County",
        "rate": 3.5,
    },
    {
        "id": 6077,
        "state": "California",
        "county": "San Joaquin County",
        "rate": 7.7,
    },
    {
        "id": 6079,
        "state": "California",
        "county": "San Luis Obispo County",
        "rate": 4.5,
    },
    {
        "id": 6081,
        "state": "California",
        "county": "San Mateo County",
        "rate": 3.2,
    },
    {
        "id": 6083,
        "state": "California",
        "county": "Santa Barbara County",
        "rate": 4.8,
    },
    {
        "id": 6085,
        "state": "California",
        "county": "Santa Clara County",
        "rate": 4,
    },
    {
        "id": 6087,
        "state": "California",
        "county": "Santa Cruz County",
        "rate": 5.7,
    },
    {"id": 6089, "state": "California", "county": "Shasta County", "rate": 6.8},
    {"id": 6091, "state": "California", "county": "Sierra County", "rate": 6.6},
    {
        "id": 6093,
        "state": "California",
        "county": "Siskiyou County",
        "rate": 7.1,
    },
    {"id": 6095, "state": "California", "county": "Solano County", "rate": 5.7},
    {"id": 6097, "state": "California", "county": "Sonoma County", "rate": 4.1},
    {
        "id": 6099,
        "state": "California",
        "county": "Stanislaus County",
        "rate": 7.9,
    },
    {"id": 6101, "state": "California", "county": "Sutter County", "rate": 7.9},
    {"id": 6103, "state": "California", "county": "Tehama County", "rate": 7.4},
    {
        "id": 6105,
        "state": "California",
        "county": "Trinity County",
        "rate": 5.9,
    },
    {
        "id": 6107,
        "state": "California",
        "county": "Tulare County",
        "rate": 10.6,
    },
    {
        "id": 6109,
        "state": "California",
        "county": "Tuolumne County",
        "rate": 6.1,
    },
    {
        "id": 6111,
        "state": "California",
        "county": "Ventura County",
        "rate": 5.7,
    },
    {"id": 6113, "state": "California", "county": "Yolo County", "rate": 5.5},
    {"id": 6115, "state": "California", "county": "Yuba County", "rate": 8},
    {"id": 8001, "state": "Colorado", "county": "Adams County", "rate": 3.6},
    {"id": 8003, "state": "Colorado", "county": "Alamosa County", "rate": 4.3},
    {"id": 8005, "state": "Colorado", "county": "Arapahoe County", "rate": 3.2},
    {
        "id": 8007,
        "state": "Colorado",
        "county": "Archuleta County",
        "rate": 3.1,
    },
    {"id": 8009, "state": "Colorado", "county": "Baca County", "rate": 1.8},
    {"id": 8011, "state": "Colorado", "county": "Bent County", "rate": 3.1},
    {"id": 8013, "state": "Colorado", "county": "Boulder County", "rate": 2.9},
    {
        "id": 8014,
        "state": "Colorado",
        "county": "Broomfield County",
        "rate": 3.1,
    },
    {"id": 8015, "state": "Colorado", "county": "Chaffee County", "rate": 2.4},
    {"id": 8017, "state": "Colorado", "county": "Cheyenne County", "rate": 2.7},
    {
        "id": 8019,
        "state": "Colorado",
        "county": "Clear Creek County",
        "rate": 3.3,
    },
    {"id": 8021, "state": "Colorado", "county": "Conejos County", "rate": 4.9},
    {"id": 8023, "state": "Colorado", "county": "Costilla County", "rate": 4.8},
    {"id": 8025, "state": "Colorado", "county": "Crowley County", "rate": 3.6},
    {"id": 8027, "state": "Colorado", "county": "Custer County", "rate": 3},
    {"id": 8029, "state": "Colorado", "county": "Delta County", "rate": 4.7},
    {"id": 8031, "state": "Colorado", "county": "Denver County", "rate": 3.2},
    {"id": 8033, "state": "Colorado", "county": "Dolores County", "rate": 4.1},
    {"id": 8035, "state": "Colorado", "county": "Douglas County", "rate": 2.8},
    {"id": 8037, "state": "Colorado", "county": "Eagle County", "rate": 2.5},
    {"id": 8039, "state": "Colorado", "county": "Elbert County", "rate": 2.6},
    {"id": 8041, "state": "Colorado", "county": "El Paso County", "rate": 3.9},
    {"id": 8043, "state": "Colorado", "county": "Fremont County", "rate": 5.1},
    {"id": 8045, "state": "Colorado", "county": "Garfield County", "rate": 3.3},
    {"id": 8047, "state": "Colorado", "county": "Gilpin County", "rate": 2.6},
    {"id": 8049, "state": "Colorado", "county": "Grand County", "rate": 2.3},
    {"id": 8051, "state": "Colorado", "county": "Gunnison County", "rate": 2.1},
    {"id": 8053, "state": "Colorado", "county": "Hinsdale County", "rate": 2},
    {"id": 8055, "state": "Colorado", "county": "Huerfano County", "rate": 6.2},
    {"id": 8057, "state": "Colorado", "county": "Jackson County", "rate": 2.3},
    {
        "id": 8059,
        "state": "Colorado",
        "county": "Jefferson County",
        "rate": 3.1,
    },
    {"id": 8061, "state": "Colorado", "county": "Kiowa County", "rate": 2.1},
    {
        "id": 8063,
        "state": "Colorado",
        "county": "Kit Carson County",
        "rate": 1.8,
    },
    {"id": 8065, "state": "Colorado", "county": "Lake County", "rate": 2.6},
    {"id": 8067, "state": "Colorado", "county": "La Plata County", "rate": 2.9},
    {"id": 8069, "state": "Colorado", "county": "Larimer County", "rate": 2.9},
    {"id": 8071, "state": "Colorado", "county": "Las Animas County", "rate": 5},
    {"id": 8073, "state": "Colorado", "county": "Lincoln County", "rate": 2.5},
    {"id": 8075, "state": "Colorado", "county": "Logan County", "rate": 3.3},
    {"id": 8077, "state": "Colorado", "county": "Mesa County", "rate": 5.3},
    {"id": 8079, "state": "Colorado", "county": "Mineral County", "rate": 1.7},
    {"id": 8081, "state": "Colorado", "county": "Moffat County", "rate": 3.6},
    {
        "id": 8083,
        "state": "Colorado",
        "county": "Montezuma County",
        "rate": 4.5,
    },
    {"id": 8085, "state": "Colorado", "county": "Montrose County", "rate": 3.9},
    {"id": 8087, "state": "Colorado", "county": "Morgan County", "rate": 3.5},
    {"id": 8089, "state": "Colorado", "county": "Otero County", "rate": 4.6},
    {"id": 8091, "state": "Colorado", "county": "Ouray County", "rate": 3.2},
    {"id": 8093, "state": "Colorado", "county": "Park County", "rate": 2.7},
    {"id": 8095, "state": "Colorado", "county": "Phillips County", "rate": 2.3},
    {"id": 8097, "state": "Colorado", "county": "Pitkin County", "rate": 2.7},
    {"id": 8099, "state": "Colorado", "county": "Prowers County", "rate": 3.5},
    {"id": 8101, "state": "Colorado", "county": "Pueblo County", "rate": 4.9},
    {
        "id": 8103,
        "state": "Colorado",
        "county": "Rio Blanco County",
        "rate": 5.1,
    },
    {
        "id": 8105,
        "state": "Colorado",
        "county": "Rio Grande County",
        "rate": 5.1,
    },
    {"id": 8107, "state": "Colorado", "county": "Routt County", "rate": 2.6},
    {"id": 8109, "state": "Colorado", "county": "Saguache County", "rate": 5.9},
    {"id": 8111, "state": "Colorado", "county": "San Juan County", "rate": 2.4},
    {
        "id": 8113,
        "state": "Colorado",
        "county": "San Miguel County",
        "rate": 2.4,
    },
    {"id": 8115, "state": "Colorado", "county": "Sedgwick County", "rate": 2.2},
    {"id": 8117, "state": "Colorado", "county": "Summit County", "rate": 2.1},
    {"id": 8119, "state": "Colorado", "county": "Teller County", "rate": 3.7},
    {
        "id": 8121,
        "state": "Colorado",
        "county": "Washington County",
        "rate": 2.1,
    },
    {"id": 8123, "state": "Colorado", "county": "Weld County", "rate": 3.5},
    {"id": 8125, "state": "Colorado", "county": "Yuma County", "rate": 2.2},
    {
        "id": 9001,
        "state": "Connecticut",
        "county": "Fairfield County",
        "rate": 5.2,
    },
    {
        "id": 9003,
        "state": "Connecticut",
        "county": "Hartford County",
        "rate": 6,
    },
    {
        "id": 9005,
        "state": "Connecticut",
        "county": "Litchfield County",
        "rate": 4.8,
    },
    {
        "id": 9007,
        "state": "Connecticut",
        "county": "Middlesex County",
        "rate": 4.8,
    },
    {
        "id": 9009,
        "state": "Connecticut",
        "county": "New Haven County",
        "rate": 6.1,
    },
    {
        "id": 9011,
        "state": "Connecticut",
        "county": "New London County",
        "rate": 5.4,
    },
    {
        "id": 9013,
        "state": "Connecticut",
        "county": "Tolland County",
        "rate": 4.8,
    },
    {
        "id": 9015,
        "state": "Connecticut",
        "county": "Windham County",
        "rate": 5.8,
    },
    {"id": 10001, "state": "Delaware", "county": "Kent County", "rate": 5.1},
    {
        "id": 10003,
        "state": "Delaware",
        "county": "New Castle County",
        "rate": 4.5,
    },
    {"id": 10005, "state": "Delaware", "county": "Sussex County", "rate": 3.7},
    {
        "id": 11001,
        "state": "District of Columbia",
        "county": "District of Columbia",
        "rate": 6.5,
    },
    {"id": 12001, "state": "Florida", "county": "Alachua County", "rate": 4.4},
    {"id": 12003, "state": "Florida", "county": "Baker County", "rate": 4.7},
    {"id": 12005, "state": "Florida", "county": "Bay County", "rate": 4.6},
    {"id": 12007, "state": "Florida", "county": "Bradford County", "rate": 4.2},
    {"id": 12009, "state": "Florida", "county": "Brevard County", "rate": 5.2},
    {"id": 12011, "state": "Florida", "county": "Broward County", "rate": 4.6},
    {"id": 12013, "state": "Florida", "county": "Calhoun County", "rate": 5.9},
    {
        "id": 12015,
        "state": "Florida",
        "county": "Charlotte County",
        "rate": 5.3,
    },
    {"id": 12017, "state": "Florida", "county": "Citrus County", "rate": 6.8},
    {"id": 12019, "state": "Florida", "county": "Clay County", "rate": 4.4},
    {"id": 12021, "state": "Florida", "county": "Collier County", "rate": 5.3},
    {"id": 12023, "state": "Florida", "county": "Columbia County", "rate": 4.8},
    {"id": 12027, "state": "Florida", "county": "DeSoto County", "rate": 6.1},
    {"id": 12029, "state": "Florida", "county": "Dixie County", "rate": 5.3},
    {"id": 12031, "state": "Florida", "county": "Duval County", "rate": 5.1},
    {"id": 12033, "state": "Florida", "county": "Escambia County", "rate": 5},
    {"id": 12035, "state": "Florida", "county": "Flagler County", "rate": 5.4},
    {"id": 12037, "state": "Florida", "county": "Franklin County", "rate": 4.1},
    {"id": 12039, "state": "Florida", "county": "Gadsden County", "rate": 6.3},
    {
        "id": 12041,
        "state": "Florida",
        "county": "Gilchrist County",
        "rate": 5.1,
    },
    {"id": 12043, "state": "Florida", "county": "Glades County", "rate": 6.9},
    {"id": 12045, "state": "Florida", "county": "Gulf County", "rate": 4.3},
    {"id": 12047, "state": "Florida", "county": "Hamilton County", "rate": 4.4},
    {"id": 12049, "state": "Florida", "county": "Hardee County", "rate": 7.2},
    {"id": 12051, "state": "Florida", "county": "Hendry County", "rate": 11.5},
    {"id": 12053, "state": "Florida", "county": "Hernando County", "rate": 6},
    {
        "id": 12055,
        "state": "Florida",
        "county": "Highlands County",
        "rate": 6.9,
    },
    {
        "id": 12057,
        "state": "Florida",
        "county": "Hillsborough County",
        "rate": 4.5,
    },
    {"id": 12059, "state": "Florida", "county": "Holmes County", "rate": 5.6},
    {
        "id": 12061,
        "state": "Florida",
        "county": "Indian River County",
        "rate": 6.6,
    },
    {"id": 12063, "state": "Florida", "county": "Jackson County", "rate": 5.4},
    {
        "id": 12065,
        "state": "Florida",
        "county": "Jefferson County",
        "rate": 5.3,
    },
    {
        "id": 12067,
        "state": "Florida",
        "county": "Lafayette County",
        "rate": 3.8,
    },
    {"id": 12069, "state": "Florida", "county": "Lake County", "rate": 4.7},
    {"id": 12071, "state": "Florida", "county": "Lee County", "rate": 4.7},
    {"id": 12073, "state": "Florida", "county": "Leon County", "rate": 4.7},
    {"id": 12075, "state": "Florida", "county": "Levy County", "rate": 5.2},
    {"id": 12077, "state": "Florida", "county": "Liberty County", "rate": 5.4},
    {"id": 12079, "state": "Florida", "county": "Madison County", "rate": 5.3},
    {"id": 12081, "state": "Florida", "county": "Manatee County", "rate": 4.6},
    {"id": 12083, "state": "Florida", "county": "Marion County", "rate": 5.9},
    {"id": 12085, "state": "Florida", "county": "Martin County", "rate": 5},
    {
        "id": 12086,
        "state": "Florida",
        "county": "Miami-Dade County",
        "rate": 5.6,
    },
    {"id": 12087, "state": "Florida", "county": "Monroe County", "rate": 3.1},
    {"id": 12089, "state": "Florida", "county": "Nassau County", "rate": 4.6},
    {"id": 12091, "state": "Florida", "county": "Okaloosa County", "rate": 3.9},
    {
        "id": 12093,
        "state": "Florida",
        "county": "Okeechobee County",
        "rate": 5.8,
    },
    {"id": 12095, "state": "Florida", "county": "Orange County", "rate": 4.3},
    {"id": 12097, "state": "Florida", "county": "Osceola County", "rate": 5},
    {
        "id": 12099,
        "state": "Florida",
        "county": "Palm Beach County",
        "rate": 5.1,
    },
    {"id": 12101, "state": "Florida", "county": "Pasco County", "rate": 5.1},
    {"id": 12103, "state": "Florida", "county": "Pinellas County", "rate": 4.3},
    {"id": 12105, "state": "Florida", "county": "Polk County", "rate": 5.8},
    {"id": 12107, "state": "Florida", "county": "Putnam County", "rate": 6},
    {
        "id": 12109,
        "state": "Florida",
        "county": "St. Johns County",
        "rate": 3.6,
    },
    {"id": 12111, "state": "Florida", "county": "St. Lucie County", "rate": 6},
    {
        "id": 12113,
        "state": "Florida",
        "county": "Santa Rosa County",
        "rate": 4.6,
    },
    {"id": 12115, "state": "Florida", "county": "Sarasota County", "rate": 4.5},
    {"id": 12117, "state": "Florida", "county": "Seminole County", "rate": 4.2},
    {"id": 12119, "state": "Florida", "county": "Sumter County", "rate": 6.7},
    {"id": 12121, "state": "Florida", "county": "Suwannee County", "rate": 5.1},
    {"id": 12123, "state": "Florida", "county": "Taylor County", "rate": 5.5},
    {"id": 12125, "state": "Florida", "county": "Union County", "rate": 4.6},
    {"id": 12127, "state": "Florida", "county": "Volusia County", "rate": 5},
    {"id": 12129, "state": "Florida", "county": "Wakulla County", "rate": 4.1},
    {"id": 12131, "state": "Florida", "county": "Walton County", "rate": 4.1},
    {"id": 12133, "state": "Florida", "county": "Washington County", "rate": 5},
    {"id": 13001, "state": "Georgia", "county": "Appling County", "rate": 6.9},
    {"id": 13003, "state": "Georgia", "county": "Atkinson County", "rate": 4.6},
    {"id": 13005, "state": "Georgia", "county": "Bacon County", "rate": 5.2},
    {"id": 13007, "state": "Georgia", "county": "Baker County", "rate": 8.3},
    {"id": 13009, "state": "Georgia", "county": "Baldwin County", "rate": 7.2},
    {"id": 13011, "state": "Georgia", "county": "Banks County", "rate": 4.8},
    {"id": 13013, "state": "Georgia", "county": "Barrow County", "rate": 4.6},
    {"id": 13015, "state": "Georgia", "county": "Bartow County", "rate": 5},
    {"id": 13017, "state": "Georgia", "county": "Ben Hill County", "rate": 8.1},
    {"id": 13019, "state": "Georgia", "county": "Berrien County", "rate": 6.1},
    {"id": 13021, "state": "Georgia", "county": "Bibb County", "rate": 5.7},
    {"id": 13023, "state": "Georgia", "county": "Bleckley County", "rate": 7.6},
    {"id": 13025, "state": "Georgia", "county": "Brantley County", "rate": 6.6},
    {"id": 13027, "state": "Georgia", "county": "Brooks County", "rate": 4.8},
    {"id": 13029, "state": "Georgia", "county": "Bryan County", "rate": 4.8},
    {"id": 13031, "state": "Georgia", "county": "Bulloch County", "rate": 5.6},
    {"id": 13033, "state": "Georgia", "county": "Burke County", "rate": 7.5},
    {"id": 13035, "state": "Georgia", "county": "Butts County", "rate": 5.2},
    {"id": 13037, "state": "Georgia", "county": "Calhoun County", "rate": 6},
    {"id": 13039, "state": "Georgia", "county": "Camden County", "rate": 5.7},
    {"id": 13043, "state": "Georgia", "county": "Candler County", "rate": 4.7},
    {"id": 13045, "state": "Georgia", "county": "Carroll County", "rate": 5.9},
    {"id": 13047, "state": "Georgia", "county": "Catoosa County", "rate": 4.8},
    {"id": 13049, "state": "Georgia", "county": "Charlton County", "rate": 5.8},
    {"id": 13051, "state": "Georgia", "county": "Chatham County", "rate": 5},
    {
        "id": 13053,
        "state": "Georgia",
        "county": "Chattahoochee County",
        "rate": 8.6,
    },
    {
        "id": 13055,
        "state": "Georgia",
        "county": "Chattooga County",
        "rate": 6.1,
    },
    {"id": 13057, "state": "Georgia", "county": "Cherokee County", "rate": 4.1},
    {"id": 13059, "state": "Georgia", "county": "Clarke County", "rate": 5.4},
    {"id": 13061, "state": "Georgia", "county": "Clay County", "rate": 9.6},
    {"id": 13063, "state": "Georgia", "county": "Clayton County", "rate": 6.4},
    {"id": 13065, "state": "Georgia", "county": "Clinch County", "rate": 6},
    {"id": 13067, "state": "Georgia", "county": "Cobb County", "rate": 4.4},
    {"id": 13069, "state": "Georgia", "county": "Coffee County", "rate": 5.6},
    {"id": 13071, "state": "Georgia", "county": "Colquitt County", "rate": 5.3},
    {"id": 13073, "state": "Georgia", "county": "Columbia County", "rate": 4.8},
    {"id": 13075, "state": "Georgia", "county": "Cook County", "rate": 5.2},
    {"id": 13077, "state": "Georgia", "county": "Coweta County", "rate": 4.9},
    {"id": 13079, "state": "Georgia", "county": "Crawford County", "rate": 5.5},
    {"id": 13081, "state": "Georgia", "county": "Crisp County", "rate": 5.9},
    {"id": 13083, "state": "Georgia", "county": "Dade County", "rate": 5.1},
    {"id": 13085, "state": "Georgia", "county": "Dawson County", "rate": 4.5},
    {"id": 13087, "state": "Georgia", "county": "Decatur County", "rate": 6.5},
    {"id": 13089, "state": "Georgia", "county": "DeKalb County", "rate": 5.2},
    {"id": 13091, "state": "Georgia", "county": "Dodge County", "rate": 7},
    {"id": 13093, "state": "Georgia", "county": "Dooly County", "rate": 5.5},
    {
        "id": 13095,
        "state": "Georgia",
        "county": "Dougherty County",
        "rate": 6.8,
    },
    {"id": 13097, "state": "Georgia", "county": "Douglas County", "rate": 5.3},
    {"id": 13099, "state": "Georgia", "county": "Early County", "rate": 6.7},
    {"id": 13101, "state": "Georgia", "county": "Echols County", "rate": 4.5},
    {
        "id": 13103,
        "state": "Georgia",
        "county": "Effingham County",
        "rate": 4.7,
    },
    {"id": 13105, "state": "Georgia", "county": "Elbert County", "rate": 6.2},
    {"id": 13107, "state": "Georgia", "county": "Emanuel County", "rate": 7.4},
    {"id": 13109, "state": "Georgia", "county": "Evans County", "rate": 4.8},
    {"id": 13111, "state": "Georgia", "county": "Fannin County", "rate": 5},
    {"id": 13113, "state": "Georgia", "county": "Fayette County", "rate": 4.7},
    {"id": 13115, "state": "Georgia", "county": "Floyd County", "rate": 5.9},
    {"id": 13117, "state": "Georgia", "county": "Forsyth County", "rate": 4.1},
    {"id": 13119, "state": "Georgia", "county": "Franklin County", "rate": 5.5},
    {"id": 13121, "state": "Georgia", "county": "Fulton County", "rate": 5.2},
    {"id": 13123, "state": "Georgia", "county": "Gilmer County", "rate": 5.4},
    {"id": 13125, "state": "Georgia", "county": "Glascock County", "rate": 6.3},
    {"id": 13127, "state": "Georgia", "county": "Glynn County", "rate": 5},
    {"id": 13129, "state": "Georgia", "county": "Gordon County", "rate": 5.3},
    {"id": 13131, "state": "Georgia", "county": "Grady County", "rate": 5.4},
    {"id": 13133, "state": "Georgia", "county": "Greene County", "rate": 5.9},
    {"id": 13135, "state": "Georgia", "county": "Gwinnett County", "rate": 4.6},
    {"id": 13137, "state": "Georgia", "county": "Habersham County", "rate": 5},
    {"id": 13139, "state": "Georgia", "county": "Hall County", "rate": 4.4},
    {"id": 13141, "state": "Georgia", "county": "Hancock County", "rate": 8.9},
    {"id": 13143, "state": "Georgia", "county": "Haralson County", "rate": 5.6},
    {"id": 13145, "state": "Georgia", "county": "Harris County", "rate": 4.7},
    {"id": 13147, "state": "Georgia", "county": "Hart County", "rate": 5.3},
    {"id": 13149, "state": "Georgia", "county": "Heard County", "rate": 5.8},
    {"id": 13151, "state": "Georgia", "county": "Henry County", "rate": 5.4},
    {"id": 13153, "state": "Georgia", "county": "Houston County", "rate": 5.3},
    {"id": 13155, "state": "Georgia", "county": "Irwin County", "rate": 7.3},
    {"id": 13157, "state": "Georgia", "county": "Jackson County", "rate": 4.1},
    {"id": 13159, "state": "Georgia", "county": "Jasper County", "rate": 4.7},
    {
        "id": 13161,
        "state": "Georgia",
        "county": "Jeff Davis County",
        "rate": 6.3,
    },
    {"id": 13163, "state": "Georgia", "county": "Jefferson County", "rate": 7},
    {"id": 13165, "state": "Georgia", "county": "Jenkins County", "rate": 7.1},
    {"id": 13167, "state": "Georgia", "county": "Johnson County", "rate": 6},
    {"id": 13169, "state": "Georgia", "county": "Jones County", "rate": 4.6},
    {"id": 13171, "state": "Georgia", "county": "Lamar County", "rate": 6.4},
    {"id": 13173, "state": "Georgia", "county": "Lanier County", "rate": 5.3},
    {"id": 13175, "state": "Georgia", "county": "Laurens County", "rate": 6.4},
    {"id": 13177, "state": "Georgia", "county": "Lee County", "rate": 4.8},
    {"id": 13179, "state": "Georgia", "county": "Liberty County", "rate": 5.7},
    {"id": 13181, "state": "Georgia", "county": "Lincoln County", "rate": 5.7},
    {"id": 13183, "state": "Georgia", "county": "Long County", "rate": 5.6},
    {"id": 13185, "state": "Georgia", "county": "Lowndes County", "rate": 5.2},
    {"id": 13187, "state": "Georgia", "county": "Lumpkin County", "rate": 5},
    {"id": 13189, "state": "Georgia", "county": "McDuffie County", "rate": 7.2},
    {"id": 13191, "state": "Georgia", "county": "McIntosh County", "rate": 5.5},
    {"id": 13193, "state": "Georgia", "county": "Macon County", "rate": 7.6},
    {"id": 13195, "state": "Georgia", "county": "Madison County", "rate": 4.7},
    {"id": 13197, "state": "Georgia", "county": "Marion County", "rate": 6.9},
    {
        "id": 13199,
        "state": "Georgia",
        "county": "Meriwether County",
        "rate": 6.8,
    },
    {"id": 13201, "state": "Georgia", "county": "Miller County", "rate": 5.5},
    {"id": 13205, "state": "Georgia", "county": "Mitchell County", "rate": 6.3},
    {"id": 13207, "state": "Georgia", "county": "Monroe County", "rate": 4.9},
    {
        "id": 13209,
        "state": "Georgia",
        "county": "Montgomery County",
        "rate": 7.1,
    },
    {"id": 13211, "state": "Georgia", "county": "Morgan County", "rate": 4.8},
    {"id": 13213, "state": "Georgia", "county": "Murray County", "rate": 6.6},
    {"id": 13215, "state": "Georgia", "county": "Muscogee County", "rate": 6.6},
    {"id": 13217, "state": "Georgia", "county": "Newton County", "rate": 5.8},
    {"id": 13219, "state": "Georgia", "county": "Oconee County", "rate": 4},
    {
        "id": 13221,
        "state": "Georgia",
        "county": "Oglethorpe County",
        "rate": 4.5,
    },
    {"id": 13223, "state": "Georgia", "county": "Paulding County", "rate": 4.5},
    {"id": 13225, "state": "Georgia", "county": "Peach County", "rate": 6.4},
    {"id": 13227, "state": "Georgia", "county": "Pickens County", "rate": 4.8},
    {"id": 13229, "state": "Georgia", "county": "Pierce County", "rate": 5.3},
    {"id": 13231, "state": "Georgia", "county": "Pike County", "rate": 5.1},
    {"id": 13233, "state": "Georgia", "county": "Polk County", "rate": 5.7},
    {"id": 13235, "state": "Georgia", "county": "Pulaski County", "rate": 5.6},
    {"id": 13237, "state": "Georgia", "county": "Putnam County", "rate": 6.8},
    {"id": 13239, "state": "Georgia", "county": "Quitman County", "rate": 7.7},
    {"id": 13241, "state": "Georgia", "county": "Rabun County", "rate": 5.4},
    {"id": 13243, "state": "Georgia", "county": "Randolph County", "rate": 8.6},
    {"id": 13245, "state": "Georgia", "county": "Richmond County", "rate": 6.5},
    {"id": 13247, "state": "Georgia", "county": "Rockdale County", "rate": 5.5},
    {"id": 13249, "state": "Georgia", "county": "Schley County", "rate": 6.4},
    {"id": 13251, "state": "Georgia", "county": "Screven County", "rate": 6.9},
    {"id": 13253, "state": "Georgia", "county": "Seminole County", "rate": 7.5},
    {"id": 13255, "state": "Georgia", "county": "Spalding County", "rate": 6.5},
    {"id": 13257, "state": "Georgia", "county": "Stephens County", "rate": 5.9},
    {"id": 13259, "state": "Georgia", "county": "Stewart County", "rate": 5.8},
    {"id": 13261, "state": "Georgia", "county": "Sumter County", "rate": 7.4},
    {"id": 13263, "state": "Georgia", "county": "Talbot County", "rate": 6.6},
    {
        "id": 13265,
        "state": "Georgia",
        "county": "Taliaferro County",
        "rate": 6.6,
    },
    {"id": 13267, "state": "Georgia", "county": "Tattnall County", "rate": 5.4},
    {"id": 13269, "state": "Georgia", "county": "Taylor County", "rate": 7},
    {"id": 13271, "state": "Georgia", "county": "Telfair County", "rate": 7.7},
    {"id": 13273, "state": "Georgia", "county": "Terrell County", "rate": 6},
    {"id": 13275, "state": "Georgia", "county": "Thomas County", "rate": 6.3},
    {"id": 13277, "state": "Georgia", "county": "Tift County", "rate": 5.6},
    {"id": 13279, "state": "Georgia", "county": "Toombs County", "rate": 7.3},
    {"id": 13281, "state": "Georgia", "county": "Towns County", "rate": 6.3},
    {"id": 13283, "state": "Georgia", "county": "Treutlen County", "rate": 7.3},
    {"id": 13285, "state": "Georgia", "county": "Troup County", "rate": 5},
    {"id": 13287, "state": "Georgia", "county": "Turner County", "rate": 6.4},
    {"id": 13289, "state": "Georgia", "county": "Twiggs County", "rate": 8.2},
    {"id": 13291, "state": "Georgia", "county": "Union County", "rate": 4.5},
    {"id": 13293, "state": "Georgia", "county": "Upson County", "rate": 6.2},
    {"id": 13295, "state": "Georgia", "county": "Walker County", "rate": 5.3},
    {"id": 13297, "state": "Georgia", "county": "Walton County", "rate": 4.7},
    {"id": 13299, "state": "Georgia", "county": "Ware County", "rate": 5.4},
    {"id": 13301, "state": "Georgia", "county": "Warren County", "rate": 7},
    {
        "id": 13303,
        "state": "Georgia",
        "county": "Washington County",
        "rate": 6.3,
    },
    {"id": 13305, "state": "Georgia", "county": "Wayne County", "rate": 6.6},
    {"id": 13307, "state": "Georgia", "county": "Webster County", "rate": 8.6},
    {"id": 13309, "state": "Georgia", "county": "Wheeler County", "rate": 8.5},
    {"id": 13311, "state": "Georgia", "county": "White County", "rate": 4.4},
    {
        "id": 13313,
        "state": "Georgia",
        "county": "Whitfield County",
        "rate": 5.7,
    },
    {"id": 13315, "state": "Georgia", "county": "Wilcox County", "rate": 6.8},
    {"id": 13317, "state": "Georgia", "county": "Wilkes County", "rate": 6.7},
    {"id": 13319, "state": "Georgia", "county": "Wilkinson County", "rate": 6},
    {"id": 13321, "state": "Georgia", "county": "Worth County", "rate": 5.8},
    {"id": 15001, "state": "Hawaii", "county": "Hawaii County", "rate": 4.1},
    {"id": 15003, "state": "Hawaii", "county": "Honolulu County", "rate": 3},
    {"id": 15007, "state": "Hawaii", "county": "Kauai County", "rate": 3.4},
    {"id": 15009, "state": "Hawaii", "county": "Maui County", "rate": 3.4},
    {"id": 16001, "state": "Idaho", "county": "Ada County", "rate": 3.5},
    {"id": 16003, "state": "Idaho", "county": "Adams County", "rate": 4.9},
    {"id": 16005, "state": "Idaho", "county": "Bannock County", "rate": 3.6},
    {"id": 16007, "state": "Idaho", "county": "Bear Lake County", "rate": 3.4},
    {"id": 16009, "state": "Idaho", "county": "Benewah County", "rate": 4.6},
    {"id": 16011, "state": "Idaho", "county": "Bingham County", "rate": 3.7},
    {"id": 16013, "state": "Idaho", "county": "Blaine County", "rate": 2.8},
    {"id": 16015, "state": "Idaho", "county": "Boise County", "rate": 4.9},
    {"id": 16017, "state": "Idaho", "county": "Bonner County", "rate": 4.3},
    {"id": 16019, "state": "Idaho", "county": "Bonneville County", "rate": 3.2},
    {"id": 16021, "state": "Idaho", "county": "Boundary County", "rate": 4.1},
    {"id": 16023, "state": "Idaho", "county": "Butte County", "rate": 3.7},
    {"id": 16025, "state": "Idaho", "county": "Camas County", "rate": 2.6},
    {"id": 16027, "state": "Idaho", "county": "Canyon County", "rate": 4.4},
    {"id": 16029, "state": "Idaho", "county": "Caribou County", "rate": 3.2},
    {"id": 16031, "state": "Idaho", "county": "Cassia County", "rate": 2.9},
    {"id": 16033, "state": "Idaho", "county": "Clark County", "rate": 3.3},
    {"id": 16035, "state": "Idaho", "county": "Clearwater County", "rate": 5.9},
    {"id": 16037, "state": "Idaho", "county": "Custer County", "rate": 4.1},
    {"id": 16039, "state": "Idaho", "county": "Elmore County", "rate": 4.2},
    {"id": 16041, "state": "Idaho", "county": "Franklin County", "rate": 2.9},
    {"id": 16043, "state": "Idaho", "county": "Fremont County", "rate": 3.2},
    {"id": 16045, "state": "Idaho", "county": "Gem County", "rate": 4.3},
    {"id": 16047, "state": "Idaho", "county": "Gooding County", "rate": 3},
    {"id": 16049, "state": "Idaho", "county": "Idaho County", "rate": 4.8},
    {"id": 16051, "state": "Idaho", "county": "Jefferson County", "rate": 3.1},
    {"id": 16053, "state": "Idaho", "county": "Jerome County", "rate": 2.8},
    {"id": 16055, "state": "Idaho", "county": "Kootenai County", "rate": 4},
    {"id": 16057, "state": "Idaho", "county": "Latah County", "rate": 3.2},
    {"id": 16059, "state": "Idaho", "county": "Lemhi County", "rate": 4.8},
    {"id": 16061, "state": "Idaho", "county": "Lewis County", "rate": 6.9},
    {"id": 16063, "state": "Idaho", "county": "Lincoln County", "rate": 3.6},
    {"id": 16065, "state": "Idaho", "county": "Madison County", "rate": 2.9},
    {"id": 16067, "state": "Idaho", "county": "Minidoka County", "rate": 3.1},
    {"id": 16069, "state": "Idaho", "county": "Nez Perce County", "rate": 3.1},
    {"id": 16071, "state": "Idaho", "county": "Oneida County", "rate": 3.1},
    {"id": 16073, "state": "Idaho", "county": "Owyhee County", "rate": 3.7},
    {"id": 16075, "state": "Idaho", "county": "Payette County", "rate": 4.1},
    {"id": 16077, "state": "Idaho", "county": "Power County", "rate": 4.2},
    {"id": 16079, "state": "Idaho", "county": "Shoshone County", "rate": 5.9},
    {"id": 16081, "state": "Idaho", "county": "Teton County", "rate": 2},
    {"id": 16083, "state": "Idaho", "county": "Twin Falls County", "rate": 3.4},
    {"id": 16085, "state": "Idaho", "county": "Valley County", "rate": 4.2},
    {"id": 16087, "state": "Idaho", "county": "Washington County", "rate": 5.3},
    {"id": 17001, "state": "Illinois", "county": "Adams County", "rate": 4.7},
    {
        "id": 17003,
        "state": "Illinois",
        "county": "Alexander County",
        "rate": 9.5,
    },
    {"id": 17005, "state": "Illinois", "county": "Bond County", "rate": 4.7},
    {"id": 17007, "state": "Illinois", "county": "Boone County", "rate": 5.8},
    {"id": 17009, "state": "Illinois", "county": "Brown County", "rate": 3.2},
    {"id": 17011, "state": "Illinois", "county": "Bureau County", "rate": 5.8},
    {"id": 17013, "state": "Illinois", "county": "Calhoun County", "rate": 5.7},
    {"id": 17015, "state": "Illinois", "county": "Carroll County", "rate": 4.8},
    {"id": 17017, "state": "Illinois", "county": "Cass County", "rate": 4.8},
    {"id": 17019, "state": "Illinois", "county": "Champaign County", "rate": 5},
    {
        "id": 17021,
        "state": "Illinois",
        "county": "Christian County",
        "rate": 5.8,
    },
    {"id": 17023, "state": "Illinois", "county": "Clark County", "rate": 5.5},
    {"id": 17025, "state": "Illinois", "county": "Clay County", "rate": 6.4},
    {"id": 17027, "state": "Illinois", "county": "Clinton County", "rate": 4.1},
    {"id": 17029, "state": "Illinois", "county": "Coles County", "rate": 6},
    {"id": 17031, "state": "Illinois", "county": "Cook County", "rate": 5.7},
    {
        "id": 17033,
        "state": "Illinois",
        "county": "Crawford County",
        "rate": 5.6,
    },
    {
        "id": 17035,
        "state": "Illinois",
        "county": "Cumberland County",
        "rate": 5,
    },
    {"id": 17037, "state": "Illinois", "county": "DeKalb County", "rate": 5},
    {"id": 17039, "state": "Illinois", "county": "De Witt County", "rate": 5.6},
    {"id": 17041, "state": "Illinois", "county": "Douglas County", "rate": 4.6},
    {"id": 17043, "state": "Illinois", "county": "DuPage County", "rate": 4.4},
    {"id": 17045, "state": "Illinois", "county": "Edgar County", "rate": 6.1},
    {"id": 17047, "state": "Illinois", "county": "Edwards County", "rate": 5.2},
    {
        "id": 17049,
        "state": "Illinois",
        "county": "Effingham County",
        "rate": 4.5,
    },
    {"id": 17051, "state": "Illinois", "county": "Fayette County", "rate": 6},
    {"id": 17053, "state": "Illinois", "county": "Ford County", "rate": 5.3},
    {"id": 17055, "state": "Illinois", "county": "Franklin County", "rate": 8},
    {"id": 17057, "state": "Illinois", "county": "Fulton County", "rate": 6.9},
    {
        "id": 17059,
        "state": "Illinois",
        "county": "Gallatin County",
        "rate": 7.6,
    },
    {"id": 17061, "state": "Illinois", "county": "Greene County", "rate": 5.3},
    {"id": 17063, "state": "Illinois", "county": "Grundy County", "rate": 5.9},
    {
        "id": 17065,
        "state": "Illinois",
        "county": "Hamilton County",
        "rate": 6.1,
    },
    {"id": 17067, "state": "Illinois", "county": "Hancock County", "rate": 6.9},
    {"id": 17069, "state": "Illinois", "county": "Hardin County", "rate": 8.1},
    {
        "id": 17071,
        "state": "Illinois",
        "county": "Henderson County",
        "rate": 5.8,
    },
    {"id": 17073, "state": "Illinois", "county": "Henry County", "rate": 5.4},
    {
        "id": 17075,
        "state": "Illinois",
        "county": "Iroquois County",
        "rate": 4.9,
    },
    {"id": 17077, "state": "Illinois", "county": "Jackson County", "rate": 5.5},
    {"id": 17079, "state": "Illinois", "county": "Jasper County", "rate": 6.2},
    {
        "id": 17081,
        "state": "Illinois",
        "county": "Jefferson County",
        "rate": 6.5,
    },
    {"id": 17083, "state": "Illinois", "county": "Jersey County", "rate": 5.5},
    {
        "id": 17085,
        "state": "Illinois",
        "county": "Jo Daviess County",
        "rate": 4.4,
    },
    {"id": 17087, "state": "Illinois", "county": "Johnson County", "rate": 8.3},
    {"id": 17089, "state": "Illinois", "county": "Kane County", "rate": 5.1},
    {
        "id": 17091,
        "state": "Illinois",
        "county": "Kankakee County",
        "rate": 6.1,
    },
    {"id": 17093, "state": "Illinois", "county": "Kendall County", "rate": 4.7},
    {"id": 17095, "state": "Illinois", "county": "Knox County", "rate": 5.6},
    {"id": 17097, "state": "Illinois", "county": "Lake County", "rate": 4.8},
    {"id": 17099, "state": "Illinois", "county": "LaSalle County", "rate": 6.2},
    {
        "id": 17101,
        "state": "Illinois",
        "county": "Lawrence County",
        "rate": 7.6,
    },
    {"id": 17103, "state": "Illinois", "county": "Lee County", "rate": 4.8},
    {
        "id": 17105,
        "state": "Illinois",
        "county": "Livingston County",
        "rate": 5.2,
    },
    {"id": 17107, "state": "Illinois", "county": "Logan County", "rate": 5.1},
    {
        "id": 17109,
        "state": "Illinois",
        "county": "McDonough County",
        "rate": 6.8,
    },
    {"id": 17111, "state": "Illinois", "county": "McHenry County", "rate": 4.6},
    {"id": 17113, "state": "Illinois", "county": "McLean County", "rate": 5.1},
    {"id": 17115, "state": "Illinois", "county": "Macon County", "rate": 6.6},
    {
        "id": 17117,
        "state": "Illinois",
        "county": "Macoupin County",
        "rate": 5.5,
    },
    {"id": 17119, "state": "Illinois", "county": "Madison County", "rate": 6},
    {"id": 17121, "state": "Illinois", "county": "Marion County", "rate": 6.2},
    {
        "id": 17123,
        "state": "Illinois",
        "county": "Marshall County",
        "rate": 5.9,
    },
    {"id": 17125, "state": "Illinois", "county": "Mason County", "rate": 7.1},
    {"id": 17127, "state": "Illinois", "county": "Massac County", "rate": 7.4},
    {"id": 17129, "state": "Illinois", "county": "Menard County", "rate": 3.9},
    {"id": 17131, "state": "Illinois", "county": "Mercer County", "rate": 5.3},
    {"id": 17133, "state": "Illinois", "county": "Monroe County", "rate": 3.9},
    {
        "id": 17135,
        "state": "Illinois",
        "county": "Montgomery County",
        "rate": 6.3,
    },
    {"id": 17137, "state": "Illinois", "county": "Morgan County", "rate": 4.6},
    {
        "id": 17139,
        "state": "Illinois",
        "county": "Moultrie County",
        "rate": 4.5,
    },
    {"id": 17141, "state": "Illinois", "county": "Ogle County", "rate": 5.3},
    {"id": 17143, "state": "Illinois", "county": "Peoria County", "rate": 6.6},
    {"id": 17145, "state": "Illinois", "county": "Perry County", "rate": 6.9},
    {"id": 17147, "state": "Illinois", "county": "Piatt County", "rate": 4.8},
    {"id": 17149, "state": "Illinois", "county": "Pike County", "rate": 4.4},
    {"id": 17151, "state": "Illinois", "county": "Pope County", "rate": 6.8},
    {"id": 17153, "state": "Illinois", "county": "Pulaski County", "rate": 8.6},
    {"id": 17155, "state": "Illinois", "county": "Putnam County", "rate": 5.2},
    {
        "id": 17157,
        "state": "Illinois",
        "county": "Randolph County",
        "rate": 4.8,
    },
    {
        "id": 17159,
        "state": "Illinois",
        "county": "Richland County",
        "rate": 5.8,
    },
    {
        "id": 17161,
        "state": "Illinois",
        "county": "Rock Island County",
        "rate": 6,
    },
    {
        "id": 17163,
        "state": "Illinois",
        "county": "St. Clair County",
        "rate": 6.2,
    },
    {"id": 17165, "state": "Illinois", "county": "Saline County", "rate": 8.1},
    {
        "id": 17167,
        "state": "Illinois",
        "county": "Sangamon County",
        "rate": 4.6,
    },
    {
        "id": 17169,
        "state": "Illinois",
        "county": "Schuyler County",
        "rate": 4.9,
    },
    {"id": 17171, "state": "Illinois", "county": "Scott County", "rate": 4.3},
    {"id": 17173, "state": "Illinois", "county": "Shelby County", "rate": 5.6},
    {"id": 17175, "state": "Illinois", "county": "Stark County", "rate": 6.7},
    {
        "id": 17177,
        "state": "Illinois",
        "county": "Stephenson County",
        "rate": 5.6,
    },
    {"id": 17179, "state": "Illinois", "county": "Tazewell County", "rate": 6},
    {"id": 17181, "state": "Illinois", "county": "Union County", "rate": 7.1},
    {
        "id": 17183,
        "state": "Illinois",
        "county": "Vermilion County",
        "rate": 7.1,
    },
    {"id": 17185, "state": "Illinois", "county": "Wabash County", "rate": 6.5},
    {"id": 17187, "state": "Illinois", "county": "Warren County", "rate": 5.2},
    {
        "id": 17189,
        "state": "Illinois",
        "county": "Washington County",
        "rate": 3.8,
    },
    {"id": 17191, "state": "Illinois", "county": "Wayne County", "rate": 8.4},
    {"id": 17193, "state": "Illinois", "county": "White County", "rate": 6.2},
    {
        "id": 17195,
        "state": "Illinois",
        "county": "Whiteside County",
        "rate": 5.8,
    },
    {"id": 17197, "state": "Illinois", "county": "Will County", "rate": 5.4},
    {
        "id": 17199,
        "state": "Illinois",
        "county": "Williamson County",
        "rate": 6.2,
    },
    {
        "id": 17201,
        "state": "Illinois",
        "county": "Winnebago County",
        "rate": 6.4,
    },
    {
        "id": 17203,
        "state": "Illinois",
        "county": "Woodford County",
        "rate": 4.9,
    },
    {"id": 18001, "state": "Indiana", "county": "Adams County", "rate": 3.8},
    {"id": 18003, "state": "Indiana", "county": "Allen County", "rate": 4.3},
    {
        "id": 18005,
        "state": "Indiana",
        "county": "Bartholomew County",
        "rate": 3.6,
    },
    {"id": 18007, "state": "Indiana", "county": "Benton County", "rate": 4.3},
    {
        "id": 18009,
        "state": "Indiana",
        "county": "Blackford County",
        "rate": 5.7,
    },
    {"id": 18011, "state": "Indiana", "county": "Boone County", "rate": 3.6},
    {"id": 18013, "state": "Indiana", "county": "Brown County", "rate": 3.8},
    {"id": 18015, "state": "Indiana", "county": "Carroll County", "rate": 4.4},
    {"id": 18017, "state": "Indiana", "county": "Cass County", "rate": 4.7},
    {"id": 18019, "state": "Indiana", "county": "Clark County", "rate": 4.4},
    {"id": 18021, "state": "Indiana", "county": "Clay County", "rate": 5},
    {"id": 18023, "state": "Indiana", "county": "Clinton County", "rate": 4},
    {"id": 18025, "state": "Indiana", "county": "Crawford County", "rate": 5.6},
    {"id": 18027, "state": "Indiana", "county": "Daviess County", "rate": 3.6},
    {"id": 18029, "state": "Indiana", "county": "Dearborn County", "rate": 4.7},
    {"id": 18031, "state": "Indiana", "county": "Decatur County", "rate": 3.6},
    {"id": 18033, "state": "Indiana", "county": "DeKalb County", "rate": 4.2},
    {"id": 18035, "state": "Indiana", "county": "Delaware County", "rate": 5.5},
    {"id": 18037, "state": "Indiana", "county": "Dubois County", "rate": 3.3},
    {"id": 18039, "state": "Indiana", "county": "Elkhart County", "rate": 4},
    {"id": 18041, "state": "Indiana", "county": "Fayette County", "rate": 5.9},
    {"id": 18043, "state": "Indiana", "county": "Floyd County", "rate": 4.4},
    {"id": 18045, "state": "Indiana", "county": "Fountain County", "rate": 5.9},
    {"id": 18047, "state": "Indiana", "county": "Franklin County", "rate": 4.2},
    {"id": 18049, "state": "Indiana", "county": "Fulton County", "rate": 4.8},
    {"id": 18051, "state": "Indiana", "county": "Gibson County", "rate": 3.8},
    {"id": 18053, "state": "Indiana", "county": "Grant County", "rate": 5.1},
    {"id": 18055, "state": "Indiana", "county": "Greene County", "rate": 6.6},
    {"id": 18057, "state": "Indiana", "county": "Hamilton County", "rate": 3.4},
    {"id": 18059, "state": "Indiana", "county": "Hancock County", "rate": 4},
    {"id": 18061, "state": "Indiana", "county": "Harrison County", "rate": 4.4},
    {
        "id": 18063,
        "state": "Indiana",
        "county": "Hendricks County",
        "rate": 3.6,
    },
    {"id": 18065, "state": "Indiana", "county": "Henry County", "rate": 4.7},
    {"id": 18067, "state": "Indiana", "county": "Howard County", "rate": 4.8},
    {
        "id": 18069,
        "state": "Indiana",
        "county": "Huntington County",
        "rate": 4.2,
    },
    {"id": 18071, "state": "Indiana", "county": "Jackson County", "rate": 4},
    {"id": 18073, "state": "Indiana", "county": "Jasper County", "rate": 5.1},
    {"id": 18075, "state": "Indiana", "county": "Jay County", "rate": 4.7},
    {"id": 18077, "state": "Indiana", "county": "Jefferson County", "rate": 5},
    {"id": 18079, "state": "Indiana", "county": "Jennings County", "rate": 4.8},
    {"id": 18081, "state": "Indiana", "county": "Johnson County", "rate": 3.8},
    {"id": 18083, "state": "Indiana", "county": "Knox County", "rate": 4.5},
    {"id": 18085, "state": "Indiana", "county": "Kosciusko County", "rate": 4},
    {"id": 18087, "state": "Indiana", "county": "LaGrange County", "rate": 3.7},
    {"id": 18089, "state": "Indiana", "county": "Lake County", "rate": 6.6},
    {"id": 18091, "state": "Indiana", "county": "LaPorte County", "rate": 6},
    {"id": 18093, "state": "Indiana", "county": "Lawrence County", "rate": 5.6},
    {"id": 18095, "state": "Indiana", "county": "Madison County", "rate": 4.9},
    {"id": 18097, "state": "Indiana", "county": "Marion County", "rate": 4.7},
    {"id": 18099, "state": "Indiana", "county": "Marshall County", "rate": 3.9},
    {"id": 18101, "state": "Indiana", "county": "Martin County", "rate": 4.1},
    {"id": 18103, "state": "Indiana", "county": "Miami County", "rate": 4.8},
    {"id": 18105, "state": "Indiana", "county": "Monroe County", "rate": 5.1},
    {
        "id": 18107,
        "state": "Indiana",
        "county": "Montgomery County",
        "rate": 4.1,
    },
    {"id": 18109, "state": "Indiana", "county": "Morgan County", "rate": 4.2},
    {"id": 18111, "state": "Indiana", "county": "Newton County", "rate": 5.3},
    {"id": 18113, "state": "Indiana", "county": "Noble County", "rate": 4.2},
    {"id": 18115, "state": "Indiana", "county": "Ohio County", "rate": 4.9},
    {"id": 18117, "state": "Indiana", "county": "Orange County", "rate": 5.2},
    {"id": 18119, "state": "Indiana", "county": "Owen County", "rate": 5.4},
    {"id": 18121, "state": "Indiana", "county": "Parke County", "rate": 5.1},
    {"id": 18123, "state": "Indiana", "county": "Perry County", "rate": 4.9},
    {"id": 18125, "state": "Indiana", "county": "Pike County", "rate": 4.6},
    {"id": 18127, "state": "Indiana", "county": "Porter County", "rate": 5.3},
    {"id": 18129, "state": "Indiana", "county": "Posey County", "rate": 4.3},
    {"id": 18131, "state": "Indiana", "county": "Pulaski County", "rate": 4.6},
    {"id": 18133, "state": "Indiana", "county": "Putnam County", "rate": 4.5},
    {"id": 18135, "state": "Indiana", "county": "Randolph County", "rate": 4.5},
    {"id": 18137, "state": "Indiana", "county": "Ripley County", "rate": 4.5},
    {"id": 18139, "state": "Indiana", "county": "Rush County", "rate": 4},
    {"id": 18141, "state": "Indiana", "county": "St. Joseph County", "rate": 5},
    {"id": 18143, "state": "Indiana", "county": "Scott County", "rate": 4.9},
    {"id": 18145, "state": "Indiana", "county": "Shelby County", "rate": 4},
    {"id": 18147, "state": "Indiana", "county": "Spencer County", "rate": 4.2},
    {"id": 18149, "state": "Indiana", "county": "Starke County", "rate": 5.2},
    {"id": 18151, "state": "Indiana", "county": "Steuben County", "rate": 3.8},
    {"id": 18153, "state": "Indiana", "county": "Sullivan County", "rate": 6.1},
    {
        "id": 18155,
        "state": "Indiana",
        "county": "Switzerland County",
        "rate": 5.3,
    },
    {
        "id": 18157,
        "state": "Indiana",
        "county": "Tippecanoe County",
        "rate": 4.3,
    },
    {"id": 18159, "state": "Indiana", "county": "Tipton County", "rate": 3.8},
    {"id": 18161, "state": "Indiana", "county": "Union County", "rate": 4.2},
    {
        "id": 18163,
        "state": "Indiana",
        "county": "Vanderburgh County",
        "rate": 4.4,
    },
    {
        "id": 18165,
        "state": "Indiana",
        "county": "Vermillion County",
        "rate": 6.5,
    },
    {"id": 18167, "state": "Indiana", "county": "Vigo County", "rate": 5.7},
    {"id": 18169, "state": "Indiana", "county": "Wabash County", "rate": 4.5},
    {"id": 18171, "state": "Indiana", "county": "Warren County", "rate": 5},
    {"id": 18173, "state": "Indiana", "county": "Warrick County", "rate": 4.4},
    {
        "id": 18175,
        "state": "Indiana",
        "county": "Washington County",
        "rate": 4.6,
    },
    {"id": 18177, "state": "Indiana", "county": "Wayne County", "rate": 4.9},
    {"id": 18179, "state": "Indiana", "county": "Wells County", "rate": 3.6},
    {"id": 18181, "state": "Indiana", "county": "White County", "rate": 3.9},
    {"id": 18183, "state": "Indiana", "county": "Whitley County", "rate": 4},
    {"id": 19001, "state": "Iowa", "county": "Adair County", "rate": 3.4},
    {"id": 19003, "state": "Iowa", "county": "Adams County", "rate": 2.9},
    {"id": 19005, "state": "Iowa", "county": "Allamakee County", "rate": 4.9},
    {"id": 19007, "state": "Iowa", "county": "Appanoose County", "rate": 5.7},
    {"id": 19009, "state": "Iowa", "county": "Audubon County", "rate": 3.7},
    {"id": 19011, "state": "Iowa", "county": "Benton County", "rate": 4},
    {"id": 19013, "state": "Iowa", "county": "Black Hawk County", "rate": 5.8},
    {"id": 19015, "state": "Iowa", "county": "Boone County", "rate": 3.4},
    {"id": 19017, "state": "Iowa", "county": "Bremer County", "rate": 4.2},
    {"id": 19019, "state": "Iowa", "county": "Buchanan County", "rate": 4.6},
    {"id": 19021, "state": "Iowa", "county": "Buena Vista County", "rate": 4.4},
    {"id": 19023, "state": "Iowa", "county": "Butler County", "rate": 4.5},
    {"id": 19025, "state": "Iowa", "county": "Calhoun County", "rate": 4.7},
    {"id": 19027, "state": "Iowa", "county": "Carroll County", "rate": 3.1},
    {"id": 19029, "state": "Iowa", "county": "Cass County", "rate": 3},
    {"id": 19031, "state": "Iowa", "county": "Cedar County", "rate": 3.8},
    {"id": 19033, "state": "Iowa", "county": "Cerro Gordo County", "rate": 4.4},
    {"id": 19035, "state": "Iowa", "county": "Cherokee County", "rate": 4.1},
    {"id": 19037, "state": "Iowa", "county": "Chickasaw County", "rate": 4.6},
    {"id": 19039, "state": "Iowa", "county": "Clarke County", "rate": 3.4},
    {"id": 19041, "state": "Iowa", "county": "Clay County", "rate": 4.1},
    {"id": 19043, "state": "Iowa", "county": "Clayton County", "rate": 4.2},
    {"id": 19045, "state": "Iowa", "county": "Clinton County", "rate": 6.1},
    {"id": 19047, "state": "Iowa", "county": "Crawford County", "rate": 5.2},
    {"id": 19049, "state": "Iowa", "county": "Dallas County", "rate": 3.2},
    {"id": 19051, "state": "Iowa", "county": "Davis County", "rate": 5.3},
    {"id": 19053, "state": "Iowa", "county": "Decatur County", "rate": 3.5},
    {"id": 19055, "state": "Iowa", "county": "Delaware County", "rate": 3.7},
    {"id": 19057, "state": "Iowa", "county": "Des Moines County", "rate": 5.9},
    {"id": 19059, "state": "Iowa", "county": "Dickinson County", "rate": 3.5},
    {"id": 19061, "state": "Iowa", "county": "Dubuque County", "rate": 4},
    {"id": 19063, "state": "Iowa", "county": "Emmet County", "rate": 4.2},
    {"id": 19065, "state": "Iowa", "county": "Fayette County", "rate": 5},
    {"id": 19067, "state": "Iowa", "county": "Floyd County", "rate": 3.9},
    {"id": 19069, "state": "Iowa", "county": "Franklin County", "rate": 3.8},
    {"id": 19071, "state": "Iowa", "county": "Fremont County", "rate": 4.1},
    {"id": 19073, "state": "Iowa", "county": "Greene County", "rate": 4.3},
    {"id": 19075, "state": "Iowa", "county": "Grundy County", "rate": 3.7},
    {"id": 19077, "state": "Iowa", "county": "Guthrie County", "rate": 3.6},
    {"id": 19079, "state": "Iowa", "county": "Hamilton County", "rate": 4},
    {"id": 19081, "state": "Iowa", "county": "Hancock County", "rate": 3},
    {"id": 19083, "state": "Iowa", "county": "Hardin County", "rate": 4.2},
    {"id": 19085, "state": "Iowa", "county": "Harrison County", "rate": 3.7},
    {"id": 19087, "state": "Iowa", "county": "Henry County", "rate": 4.4},
    {"id": 19089, "state": "Iowa", "county": "Howard County", "rate": 3.6},
    {"id": 19091, "state": "Iowa", "county": "Humboldt County", "rate": 3.2},
    {"id": 19093, "state": "Iowa", "county": "Ida County", "rate": 2.8},
    {"id": 19095, "state": "Iowa", "county": "Iowa County", "rate": 3.2},
    {"id": 19097, "state": "Iowa", "county": "Jackson County", "rate": 4.5},
    {"id": 19099, "state": "Iowa", "county": "Jasper County", "rate": 4},
    {"id": 19101, "state": "Iowa", "county": "Jefferson County", "rate": 4.6},
    {"id": 19103, "state": "Iowa", "county": "Johnson County", "rate": 3.2},
    {"id": 19105, "state": "Iowa", "county": "Jones County", "rate": 4.4},
    {"id": 19107, "state": "Iowa", "county": "Keokuk County", "rate": 5.1},
    {"id": 19109, "state": "Iowa", "county": "Kossuth County", "rate": 3},
    {"id": 19111, "state": "Iowa", "county": "Lee County", "rate": 8.1},
    {"id": 19113, "state": "Iowa", "county": "Linn County", "rate": 4.3},
    {"id": 19115, "state": "Iowa", "county": "Louisa County", "rate": 4.5},
    {"id": 19117, "state": "Iowa", "county": "Lucas County", "rate": 3.8},
    {"id": 19119, "state": "Iowa", "county": "Lyon County", "rate": 2.3},
    {"id": 19121, "state": "Iowa", "county": "Madison County", "rate": 3.7},
    {"id": 19123, "state": "Iowa", "county": "Mahaska County", "rate": 4.3},
    {"id": 19125, "state": "Iowa", "county": "Marion County", "rate": 3.4},
    {"id": 19127, "state": "Iowa", "county": "Marshall County", "rate": 4.9},
    {"id": 19129, "state": "Iowa", "county": "Mills County", "rate": 4},
    {"id": 19131, "state": "Iowa", "county": "Mitchell County", "rate": 3},
    {"id": 19133, "state": "Iowa", "county": "Monona County", "rate": 4.6},
    {"id": 19135, "state": "Iowa", "county": "Monroe County", "rate": 5.4},
    {"id": 19137, "state": "Iowa", "county": "Montgomery County", "rate": 4.2},
    {"id": 19139, "state": "Iowa", "county": "Muscatine County", "rate": 4.4},
    {"id": 19141, "state": "Iowa", "county": "O'Brien County", "rate": 3},
    {"id": 19143, "state": "Iowa", "county": "Osceola County", "rate": 3.1},
    {"id": 19145, "state": "Iowa", "county": "Page County", "rate": 6.4},
    {"id": 19147, "state": "Iowa", "county": "Palo Alto County", "rate": 3.9},
    {"id": 19149, "state": "Iowa", "county": "Plymouth County", "rate": 2.9},
    {"id": 19151, "state": "Iowa", "county": "Pocahontas County", "rate": 3.4},
    {"id": 19153, "state": "Iowa", "county": "Polk County", "rate": 4.1},
    {
        "id": 19155,
        "state": "Iowa",
        "county": "Pottawattamie County",
        "rate": 3.8,
    },
    {"id": 19157, "state": "Iowa", "county": "Poweshiek County", "rate": 4},
    {"id": 19159, "state": "Iowa", "county": "Ringgold County", "rate": 3.7},
    {"id": 19161, "state": "Iowa", "county": "Sac County", "rate": 3},
    {"id": 19163, "state": "Iowa", "county": "Scott County", "rate": 5.4},
    {"id": 19165, "state": "Iowa", "county": "Shelby County", "rate": 3.2},
    {"id": 19167, "state": "Iowa", "county": "Sioux County", "rate": 2.4},
    {"id": 19169, "state": "Iowa", "county": "Story County", "rate": 2.9},
    {"id": 19171, "state": "Iowa", "county": "Tama County", "rate": 4.2},
    {"id": 19173, "state": "Iowa", "county": "Taylor County", "rate": 3.9},
    {"id": 19175, "state": "Iowa", "county": "Union County", "rate": 4.4},
    {"id": 19177, "state": "Iowa", "county": "Van Buren County", "rate": 4.9},
    {"id": 19179, "state": "Iowa", "county": "Wapello County", "rate": 7.4},
    {"id": 19181, "state": "Iowa", "county": "Warren County", "rate": 3.8},
    {"id": 19183, "state": "Iowa", "county": "Washington County", "rate": 3.5},
    {"id": 19185, "state": "Iowa", "county": "Wayne County", "rate": 4.6},
    {"id": 19187, "state": "Iowa", "county": "Webster County", "rate": 4.7},
    {"id": 19189, "state": "Iowa", "county": "Winnebago County", "rate": 4.1},
    {"id": 19191, "state": "Iowa", "county": "Winneshiek County", "rate": 3.6},
    {"id": 19193, "state": "Iowa", "county": "Woodbury County", "rate": 4.2},
    {"id": 19195, "state": "Iowa", "county": "Worth County", "rate": 3.5},
    {"id": 19197, "state": "Iowa", "county": "Wright County", "rate": 4.5},
    {"id": 20001, "state": "Kansas", "county": "Allen County", "rate": 6.9},
    {"id": 20003, "state": "Kansas", "county": "Anderson County", "rate": 4.6},
    {"id": 20005, "state": "Kansas", "county": "Atchison County", "rate": 7.4},
    {"id": 20007, "state": "Kansas", "county": "Barber County", "rate": 4.4},
    {"id": 20009, "state": "Kansas", "county": "Barton County", "rate": 5},
    {"id": 20011, "state": "Kansas", "county": "Bourbon County", "rate": 5.7},
    {"id": 20013, "state": "Kansas", "county": "Brown County", "rate": 4.2},
    {"id": 20015, "state": "Kansas", "county": "Butler County", "rate": 4.6},
    {"id": 20017, "state": "Kansas", "county": "Chase County", "rate": 4.6},
    {
        "id": 20019,
        "state": "Kansas",
        "county": "Chautauqua County",
        "rate": 5.9,
    },
    {"id": 20021, "state": "Kansas", "county": "Cherokee County", "rate": 5.2},
    {"id": 20023, "state": "Kansas", "county": "Cheyenne County", "rate": 3.3},
    {"id": 20025, "state": "Kansas", "county": "Clark County", "rate": 3},
    {"id": 20027, "state": "Kansas", "county": "Clay County", "rate": 4.8},
    {"id": 20029, "state": "Kansas", "county": "Cloud County", "rate": 5.1},
    {"id": 20031, "state": "Kansas", "county": "Coffey County", "rate": 8.2},
    {"id": 20033, "state": "Kansas", "county": "Comanche County", "rate": 3.2},
    {"id": 20035, "state": "Kansas", "county": "Cowley County", "rate": 5.1},
    {"id": 20037, "state": "Kansas", "county": "Crawford County", "rate": 5.4},
    {"id": 20039, "state": "Kansas", "county": "Decatur County", "rate": 4.7},
    {"id": 20041, "state": "Kansas", "county": "Dickinson County", "rate": 4.7},
    {"id": 20043, "state": "Kansas", "county": "Doniphan County", "rate": 4.7},
    {"id": 20045, "state": "Kansas", "county": "Douglas County", "rate": 4.1},
    {"id": 20047, "state": "Kansas", "county": "Edwards County", "rate": 3.3},
    {"id": 20049, "state": "Kansas", "county": "Elk County", "rate": 6.8},
    {"id": 20051, "state": "Kansas", "county": "Ellis County", "rate": 3.4},
    {"id": 20053, "state": "Kansas", "county": "Ellsworth County", "rate": 4.3},
    {"id": 20055, "state": "Kansas", "county": "Finney County", "rate": 3.5},
    {"id": 20057, "state": "Kansas", "county": "Ford County", "rate": 3.6},
    {"id": 20059, "state": "Kansas", "county": "Franklin County", "rate": 4.8},
    {"id": 20061, "state": "Kansas", "county": "Geary County", "rate": 6.4},
    {"id": 20063, "state": "Kansas", "county": "Gove County", "rate": 3},
    {"id": 20065, "state": "Kansas", "county": "Graham County", "rate": 5},
    {"id": 20067, "state": "Kansas", "county": "Grant County", "rate": 3.9},
    {"id": 20069, "state": "Kansas", "county": "Gray County", "rate": 2.6},
    {"id": 20071, "state": "Kansas", "county": "Greeley County", "rate": 2.7},
    {"id": 20073, "state": "Kansas", "county": "Greenwood County", "rate": 5.7},
    {"id": 20075, "state": "Kansas", "county": "Hamilton County", "rate": 3.2},
    {"id": 20077, "state": "Kansas", "county": "Harper County", "rate": 4},
    {"id": 20079, "state": "Kansas", "county": "Harvey County", "rate": 6.3},
    {"id": 20081, "state": "Kansas", "county": "Haskell County", "rate": 2.8},
    {"id": 20083, "state": "Kansas", "county": "Hodgeman County", "rate": 2.7},
    {"id": 20085, "state": "Kansas", "county": "Jackson County", "rate": 4},
    {"id": 20087, "state": "Kansas", "county": "Jefferson County", "rate": 4.1},
    {"id": 20089, "state": "Kansas", "county": "Jewell County", "rate": 4.5},
    {"id": 20091, "state": "Kansas", "county": "Johnson County", "rate": 3.7},
    {"id": 20093, "state": "Kansas", "county": "Kearny County", "rate": 3.7},
    {"id": 20095, "state": "Kansas", "county": "Kingman County", "rate": 4.8},
    {"id": 20097, "state": "Kansas", "county": "Kiowa County", "rate": 3.5},
    {"id": 20099, "state": "Kansas", "county": "Labette County", "rate": 6.2},
    {"id": 20101, "state": "Kansas", "county": "Lane County", "rate": 3.9},
    {
        "id": 20103,
        "state": "Kansas",
        "county": "Leavenworth County",
        "rate": 4.5,
    },
    {"id": 20105, "state": "Kansas", "county": "Lincoln County", "rate": 4},
    {"id": 20107, "state": "Kansas", "county": "Linn County", "rate": 7},
    {"id": 20109, "state": "Kansas", "county": "Logan County", "rate": 3.2},
    {"id": 20111, "state": "Kansas", "county": "Lyon County", "rate": 4.3},
    {"id": 20113, "state": "Kansas", "county": "McPherson County", "rate": 3.9},
    {"id": 20115, "state": "Kansas", "county": "Marion County", "rate": 5.9},
    {"id": 20117, "state": "Kansas", "county": "Marshall County", "rate": 3.6},
    {"id": 20119, "state": "Kansas", "county": "Meade County", "rate": 2.9},
    {"id": 20121, "state": "Kansas", "county": "Miami County", "rate": 4.3},
    {"id": 20123, "state": "Kansas", "county": "Mitchell County", "rate": 3.1},
    {
        "id": 20125,
        "state": "Kansas",
        "county": "Montgomery County",
        "rate": 6.9,
    },
    {"id": 20127, "state": "Kansas", "county": "Morris County", "rate": 4.3},
    {"id": 20129, "state": "Kansas", "county": "Morton County", "rate": 4.6},
    {"id": 20131, "state": "Kansas", "county": "Nemaha County", "rate": 3},
    {"id": 20133, "state": "Kansas", "county": "Neosho County", "rate": 8.1},
    {"id": 20135, "state": "Kansas", "county": "Ness County", "rate": 4.1},
    {"id": 20137, "state": "Kansas", "county": "Norton County", "rate": 2.9},
    {"id": 20139, "state": "Kansas", "county": "Osage County", "rate": 4.6},
    {"id": 20141, "state": "Kansas", "county": "Osborne County", "rate": 4.1},
    {"id": 20143, "state": "Kansas", "county": "Ottawa County", "rate": 4.5},
    {"id": 20145, "state": "Kansas", "county": "Pawnee County", "rate": 3.6},
    {"id": 20147, "state": "Kansas", "county": "Phillips County", "rate": 3.6},
    {
        "id": 20149,
        "state": "Kansas",
        "county": "Pottawatomie County",
        "rate": 4.1,
    },
    {"id": 20151, "state": "Kansas", "county": "Pratt County", "rate": 5},
    {"id": 20153, "state": "Kansas", "county": "Rawlins County", "rate": 2.6},
    {"id": 20155, "state": "Kansas", "county": "Reno County", "rate": 5.2},
    {"id": 20157, "state": "Kansas", "county": "Republic County", "rate": 3.2},
    {"id": 20159, "state": "Kansas", "county": "Rice County", "rate": 4.8},
    {"id": 20161, "state": "Kansas", "county": "Riley County", "rate": 3.7},
    {"id": 20163, "state": "Kansas", "county": "Rooks County", "rate": 5.5},
    {"id": 20165, "state": "Kansas", "county": "Rush County", "rate": 4.7},
    {"id": 20167, "state": "Kansas", "county": "Russell County", "rate": 4.6},
    {"id": 20169, "state": "Kansas", "county": "Saline County", "rate": 4},
    {"id": 20171, "state": "Kansas", "county": "Scott County", "rate": 2.7},
    {"id": 20173, "state": "Kansas", "county": "Sedgwick County", "rate": 5.2},
    {"id": 20175, "state": "Kansas", "county": "Seward County", "rate": 4.9},
    {"id": 20177, "state": "Kansas", "county": "Shawnee County", "rate": 4.5},
    {"id": 20179, "state": "Kansas", "county": "Sheridan County", "rate": 2.9},
    {"id": 20181, "state": "Kansas", "county": "Sherman County", "rate": 3.8},
    {"id": 20183, "state": "Kansas", "county": "Smith County", "rate": 3.7},
    {"id": 20185, "state": "Kansas", "county": "Stafford County", "rate": 5},
    {"id": 20187, "state": "Kansas", "county": "Stanton County", "rate": 3.3},
    {"id": 20189, "state": "Kansas", "county": "Stevens County", "rate": 4.7},
    {"id": 20191, "state": "Kansas", "county": "Sumner County", "rate": 4.9},
    {"id": 20193, "state": "Kansas", "county": "Thomas County", "rate": 3.2},
    {"id": 20195, "state": "Kansas", "county": "Trego County", "rate": 4.2},
    {"id": 20197, "state": "Kansas", "county": "Wabaunsee County", "rate": 3.8},
    {"id": 20199, "state": "Kansas", "county": "Wallace County", "rate": 3.3},
    {
        "id": 20201,
        "state": "Kansas",
        "county": "Washington County",
        "rate": 3.9,
    },
    {"id": 20203, "state": "Kansas", "county": "Wichita County", "rate": 2.6},
    {"id": 20205, "state": "Kansas", "county": "Wilson County", "rate": 6.9},
    {"id": 20207, "state": "Kansas", "county": "Woodson County", "rate": 6.9},
    {"id": 20209, "state": "Kansas", "county": "Wyandotte County", "rate": 6.1},
    {"id": 21001, "state": "Kentucky", "county": "Adair County", "rate": 6.1},
    {"id": 21003, "state": "Kentucky", "county": "Allen County", "rate": 3.9},
    {
        "id": 21005,
        "state": "Kentucky",
        "county": "Anderson County",
        "rate": 3.6,
    },
    {"id": 21007, "state": "Kentucky", "county": "Ballard County", "rate": 9},
    {"id": 21009, "state": "Kentucky", "county": "Barren County", "rate": 4.3},
    {"id": 21011, "state": "Kentucky", "county": "Bath County", "rate": 6.5},
    {"id": 21013, "state": "Kentucky", "county": "Bell County", "rate": 8.3},
    {"id": 21015, "state": "Kentucky", "county": "Boone County", "rate": 3.5},
    {"id": 21017, "state": "Kentucky", "county": "Bourbon County", "rate": 4.6},
    {"id": 21019, "state": "Kentucky", "county": "Boyd County", "rate": 8.3},
    {"id": 21021, "state": "Kentucky", "county": "Boyle County", "rate": 4.5},
    {"id": 21023, "state": "Kentucky", "county": "Bracken County", "rate": 5.4},
    {
        "id": 21025,
        "state": "Kentucky",
        "county": "Breathitt County",
        "rate": 8.3,
    },
    {
        "id": 21027,
        "state": "Kentucky",
        "county": "Breckinridge County",
        "rate": 5.7,
    },
    {"id": 21029, "state": "Kentucky", "county": "Bullitt County", "rate": 3.8},
    {"id": 21031, "state": "Kentucky", "county": "Butler County", "rate": 4.8},
    {"id": 21033, "state": "Kentucky", "county": "Caldwell County", "rate": 5},
    {
        "id": 21035,
        "state": "Kentucky",
        "county": "Calloway County",
        "rate": 4.1,
    },
    {
        "id": 21037,
        "state": "Kentucky",
        "county": "Campbell County",
        "rate": 3.5,
    },
    {
        "id": 21039,
        "state": "Kentucky",
        "county": "Carlisle County",
        "rate": 6.8,
    },
    {"id": 21041, "state": "Kentucky", "county": "Carroll County", "rate": 5},
    {"id": 21043, "state": "Kentucky", "county": "Carter County", "rate": 10.6},
    {"id": 21045, "state": "Kentucky", "county": "Casey County", "rate": 4.4},
    {"id": 21047, "state": "Kentucky", "county": "Christian County", "rate": 6},
    {"id": 21049, "state": "Kentucky", "county": "Clark County", "rate": 4.5},
    {"id": 21051, "state": "Kentucky", "county": "Clay County", "rate": 9.7},
    {"id": 21053, "state": "Kentucky", "county": "Clinton County", "rate": 6.4},
    {
        "id": 21055,
        "state": "Kentucky",
        "county": "Crittenden County",
        "rate": 5.2,
    },
    {
        "id": 21057,
        "state": "Kentucky",
        "county": "Cumberland County",
        "rate": 4.6,
    },
    {"id": 21059, "state": "Kentucky", "county": "Daviess County", "rate": 4},
    {
        "id": 21061,
        "state": "Kentucky",
        "county": "Edmonson County",
        "rate": 5.4,
    },
    {
        "id": 21063,
        "state": "Kentucky",
        "county": "Elliott County",
        "rate": 11.7,
    },
    {"id": 21065, "state": "Kentucky", "county": "Estill County", "rate": 5.2},
    {"id": 21067, "state": "Kentucky", "county": "Fayette County", "rate": 3.3},
    {"id": 21069, "state": "Kentucky", "county": "Fleming County", "rate": 5.5},
    {"id": 21071, "state": "Kentucky", "county": "Floyd County", "rate": 10.4},
    {
        "id": 21073,
        "state": "Kentucky",
        "county": "Franklin County",
        "rate": 3.6,
    },
    {"id": 21075, "state": "Kentucky", "county": "Fulton County", "rate": 6.6},
    {
        "id": 21077,
        "state": "Kentucky",
        "county": "Gallatin County",
        "rate": 4.4,
    },
    {"id": 21079, "state": "Kentucky", "county": "Garrard County", "rate": 4.3},
    {"id": 21081, "state": "Kentucky", "county": "Grant County", "rate": 4.2},
    {"id": 21083, "state": "Kentucky", "county": "Graves County", "rate": 6.4},
    {"id": 21085, "state": "Kentucky", "county": "Grayson County", "rate": 6.2},
    {"id": 21087, "state": "Kentucky", "county": "Green County", "rate": 4},
    {"id": 21089, "state": "Kentucky", "county": "Greenup County", "rate": 8.8},
    {"id": 21091, "state": "Kentucky", "county": "Hancock County", "rate": 4.7},
    {"id": 21093, "state": "Kentucky", "county": "Hardin County", "rate": 4.1},
    {"id": 21095, "state": "Kentucky", "county": "Harlan County", "rate": 11.6},
    {
        "id": 21097,
        "state": "Kentucky",
        "county": "Harrison County",
        "rate": 4.4,
    },
    {"id": 21099, "state": "Kentucky", "county": "Hart County", "rate": 4.4},
    {
        "id": 21101,
        "state": "Kentucky",
        "county": "Henderson County",
        "rate": 4.5,
    },
    {"id": 21103, "state": "Kentucky", "county": "Henry County", "rate": 3.9},
    {"id": 21105, "state": "Kentucky", "county": "Hickman County", "rate": 5.1},
    {"id": 21107, "state": "Kentucky", "county": "Hopkins County", "rate": 5.5},
    {"id": 21109, "state": "Kentucky", "county": "Jackson County", "rate": 6.9},
    {
        "id": 21111,
        "state": "Kentucky",
        "county": "Jefferson County",
        "rate": 4.2,
    },
    {
        "id": 21113,
        "state": "Kentucky",
        "county": "Jessamine County",
        "rate": 3.6,
    },
    {"id": 21115, "state": "Kentucky", "county": "Johnson County", "rate": 9.2},
    {"id": 21117, "state": "Kentucky", "county": "Kenton County", "rate": 3.8},
    {"id": 21119, "state": "Kentucky", "county": "Knott County", "rate": 10.5},
    {"id": 21121, "state": "Kentucky", "county": "Knox County", "rate": 7.5},
    {"id": 21123, "state": "Kentucky", "county": "Larue County", "rate": 3.9},
    {"id": 21125, "state": "Kentucky", "county": "Laurel County", "rate": 5.8},
    {
        "id": 21127,
        "state": "Kentucky",
        "county": "Lawrence County",
        "rate": 10.6,
    },
    {"id": 21129, "state": "Kentucky", "county": "Lee County", "rate": 8.3},
    {"id": 21131, "state": "Kentucky", "county": "Leslie County", "rate": 13.2},
    {
        "id": 21133,
        "state": "Kentucky",
        "county": "Letcher County",
        "rate": 11.8,
    },
    {"id": 21135, "state": "Kentucky", "county": "Lewis County", "rate": 8.4},
    {"id": 21137, "state": "Kentucky", "county": "Lincoln County", "rate": 5.6},
    {
        "id": 21139,
        "state": "Kentucky",
        "county": "Livingston County",
        "rate": 6.7,
    },
    {"id": 21141, "state": "Kentucky", "county": "Logan County", "rate": 4.2},
    {"id": 21143, "state": "Kentucky", "county": "Lyon County", "rate": 5.6},
    {
        "id": 21145,
        "state": "Kentucky",
        "county": "McCracken County",
        "rate": 5.7,
    },
    {
        "id": 21147,
        "state": "Kentucky",
        "county": "McCreary County",
        "rate": 7.3,
    },
    {"id": 21149, "state": "Kentucky", "county": "McLean County", "rate": 4.5},
    {"id": 21151, "state": "Kentucky", "county": "Madison County", "rate": 3.8},
    {
        "id": 21153,
        "state": "Kentucky",
        "county": "Magoffin County",
        "rate": 16.3,
    },
    {"id": 21155, "state": "Kentucky", "county": "Marion County", "rate": 4.3},
    {
        "id": 21157,
        "state": "Kentucky",
        "county": "Marshall County",
        "rate": 5.5,
    },
    {"id": 21159, "state": "Kentucky", "county": "Martin County", "rate": 9},
    {"id": 21161, "state": "Kentucky", "county": "Mason County", "rate": 6},
    {"id": 21163, "state": "Kentucky", "county": "Meade County", "rate": 4.7},
    {"id": 21165, "state": "Kentucky", "county": "Menifee County", "rate": 7.1},
    {"id": 21167, "state": "Kentucky", "county": "Mercer County", "rate": 4.4},
    {
        "id": 21169,
        "state": "Kentucky",
        "county": "Metcalfe County",
        "rate": 3.8,
    },
    {"id": 21171, "state": "Kentucky", "county": "Monroe County", "rate": 3.6},
    {
        "id": 21173,
        "state": "Kentucky",
        "county": "Montgomery County",
        "rate": 6,
    },
    {"id": 21175, "state": "Kentucky", "county": "Morgan County", "rate": 6.8},
    {
        "id": 21177,
        "state": "Kentucky",
        "county": "Muhlenberg County",
        "rate": 6.7,
    },
    {"id": 21179, "state": "Kentucky", "county": "Nelson County", "rate": 4.3},
    {
        "id": 21181,
        "state": "Kentucky",
        "county": "Nicholas County",
        "rate": 4.9,
    },
    {"id": 21183, "state": "Kentucky", "county": "Ohio County", "rate": 6.3},
    {"id": 21185, "state": "Kentucky", "county": "Oldham County", "rate": 3.1},
    {"id": 21187, "state": "Kentucky", "county": "Owen County", "rate": 3.8},
    {"id": 21189, "state": "Kentucky", "county": "Owsley County", "rate": 8.7},
    {
        "id": 21191,
        "state": "Kentucky",
        "county": "Pendleton County",
        "rate": 4.3,
    },
    {"id": 21193, "state": "Kentucky", "county": "Perry County", "rate": 10.4},
    {"id": 21195, "state": "Kentucky", "county": "Pike County", "rate": 10.3},
    {"id": 21197, "state": "Kentucky", "county": "Powell County", "rate": 6.9},
    {"id": 21199, "state": "Kentucky", "county": "Pulaski County", "rate": 5},
    {
        "id": 21201,
        "state": "Kentucky",
        "county": "Robertson County",
        "rate": 6.1,
    },
    {
        "id": 21203,
        "state": "Kentucky",
        "county": "Rockcastle County",
        "rate": 5.3,
    },
    {"id": 21205, "state": "Kentucky", "county": "Rowan County", "rate": 5.3},
    {"id": 21207, "state": "Kentucky", "county": "Russell County", "rate": 7.6},
    {"id": 21209, "state": "Kentucky", "county": "Scott County", "rate": 3.6},
    {"id": 21211, "state": "Kentucky", "county": "Shelby County", "rate": 3.3},
    {"id": 21213, "state": "Kentucky", "county": "Simpson County", "rate": 4.2},
    {"id": 21215, "state": "Kentucky", "county": "Spencer County", "rate": 3.5},
    {"id": 21217, "state": "Kentucky", "county": "Taylor County", "rate": 4.8},
    {"id": 21219, "state": "Kentucky", "county": "Todd County", "rate": 3.9},
    {"id": 21221, "state": "Kentucky", "county": "Trigg County", "rate": 5},
    {"id": 21223, "state": "Kentucky", "county": "Trimble County", "rate": 5.4},
    {"id": 21225, "state": "Kentucky", "county": "Union County", "rate": 6.4},
    {"id": 21227, "state": "Kentucky", "county": "Warren County", "rate": 3.5},
    {
        "id": 21229,
        "state": "Kentucky",
        "county": "Washington County",
        "rate": 4,
    },
    {"id": 21231, "state": "Kentucky", "county": "Wayne County", "rate": 6.8},
    {"id": 21233, "state": "Kentucky", "county": "Webster County", "rate": 5.5},
    {"id": 21235, "state": "Kentucky", "county": "Whitley County", "rate": 6.4},
    {"id": 21237, "state": "Kentucky", "county": "Wolfe County", "rate": 9.5},
    {"id": 21239, "state": "Kentucky", "county": "Woodford County", "rate": 3},
    {"id": 22001, "state": "Louisiana", "county": "Acadia Parish", "rate": 7.9},
    {"id": 22003, "state": "Louisiana", "county": "Allen Parish", "rate": 7.5},
    {
        "id": 22005,
        "state": "Louisiana",
        "county": "Ascension Parish",
        "rate": 5.3,
    },
    {
        "id": 22007,
        "state": "Louisiana",
        "county": "Assumption Parish",
        "rate": 9,
    },
    {
        "id": 22009,
        "state": "Louisiana",
        "county": "Avoyelles Parish",
        "rate": 7.8,
    },
    {
        "id": 22011,
        "state": "Louisiana",
        "county": "Beauregard Parish",
        "rate": 6.7,
    },
    {
        "id": 22013,
        "state": "Louisiana",
        "county": "Bienville Parish",
        "rate": 8,
    },
    {
        "id": 22015,
        "state": "Louisiana",
        "county": "Bossier Parish",
        "rate": 5.9,
    },
    {"id": 22017, "state": "Louisiana", "county": "Caddo Parish", "rate": 7.4},
    {
        "id": 22019,
        "state": "Louisiana",
        "county": "Calcasieu Parish",
        "rate": 5.5,
    },
    {
        "id": 22021,
        "state": "Louisiana",
        "county": "Caldwell Parish",
        "rate": 8.4,
    },
    {
        "id": 22023,
        "state": "Louisiana",
        "county": "Cameron Parish",
        "rate": 4.8,
    },
    {
        "id": 22025,
        "state": "Louisiana",
        "county": "Catahoula Parish",
        "rate": 9.6,
    },
    {
        "id": 22027,
        "state": "Louisiana",
        "county": "Claiborne Parish",
        "rate": 7.5,
    },
    {
        "id": 22029,
        "state": "Louisiana",
        "county": "Concordia Parish",
        "rate": 9.2,
    },
    {"id": 22031, "state": "Louisiana", "county": "De Soto Parish", "rate": 8},
    {
        "id": 22033,
        "state": "Louisiana",
        "county": "East Baton Rouge Parish",
        "rate": 5.7,
    },
    {
        "id": 22035,
        "state": "Louisiana",
        "county": "East Carroll Parish",
        "rate": 11.8,
    },
    {
        "id": 22037,
        "state": "Louisiana",
        "county": "East Feliciana Parish",
        "rate": 6.2,
    },
    {
        "id": 22039,
        "state": "Louisiana",
        "county": "Evangeline Parish",
        "rate": 8.5,
    },
    {
        "id": 22041,
        "state": "Louisiana",
        "county": "Franklin Parish",
        "rate": 9.6,
    },
    {"id": 22043, "state": "Louisiana", "county": "Grant Parish", "rate": 7.8},
    {"id": 22045, "state": "Louisiana", "county": "Iberia Parish", "rate": 9.7},
    {
        "id": 22047,
        "state": "Louisiana",
        "county": "Iberville Parish",
        "rate": 7.4,
    },
    {
        "id": 22049,
        "state": "Louisiana",
        "county": "Jackson Parish",
        "rate": 6.3,
    },
    {
        "id": 22051,
        "state": "Louisiana",
        "county": "Jefferson Parish",
        "rate": 5.9,
    },
    {
        "id": 22053,
        "state": "Louisiana",
        "county": "Jefferson Davis Parish",
        "rate": 6.7,
    },
    {
        "id": 22055,
        "state": "Louisiana",
        "county": "Lafayette Parish",
        "rate": 6.7,
    },
    {
        "id": 22057,
        "state": "Louisiana",
        "county": "Lafourche Parish",
        "rate": 6.7,
    },
    {
        "id": 22059,
        "state": "Louisiana",
        "county": "La Salle Parish",
        "rate": 6.7,
    },
    {
        "id": 22061,
        "state": "Louisiana",
        "county": "Lincoln Parish",
        "rate": 7.5,
    },
    {
        "id": 22063,
        "state": "Louisiana",
        "county": "Livingston Parish",
        "rate": 5.3,
    },
    {
        "id": 22065,
        "state": "Louisiana",
        "county": "Madison Parish",
        "rate": 9.4,
    },
    {
        "id": 22067,
        "state": "Louisiana",
        "county": "Morehouse Parish",
        "rate": 10.4,
    },
    {
        "id": 22069,
        "state": "Louisiana",
        "county": "Natchitoches Parish",
        "rate": 8.1,
    },
    {
        "id": 22071,
        "state": "Louisiana",
        "county": "Orleans Parish",
        "rate": 6.7,
    },
    {
        "id": 22073,
        "state": "Louisiana",
        "county": "Ouachita Parish",
        "rate": 6.7,
    },
    {
        "id": 22075,
        "state": "Louisiana",
        "county": "Plaquemines Parish",
        "rate": 5.9,
    },
    {
        "id": 22077,
        "state": "Louisiana",
        "county": "Pointe Coupee Parish",
        "rate": 7.2,
    },
    {
        "id": 22079,
        "state": "Louisiana",
        "county": "Rapides Parish",
        "rate": 6.9,
    },
    {
        "id": 22081,
        "state": "Louisiana",
        "county": "Red River Parish",
        "rate": 6.8,
    },
    {
        "id": 22083,
        "state": "Louisiana",
        "county": "Richland Parish",
        "rate": 7.7,
    },
    {"id": 22085, "state": "Louisiana", "county": "Sabine Parish", "rate": 7.3},
    {
        "id": 22087,
        "state": "Louisiana",
        "county": "St. Bernard Parish",
        "rate": 6.9,
    },
    {
        "id": 22089,
        "state": "Louisiana",
        "county": "St. Charles Parish",
        "rate": 6,
    },
    {
        "id": 22091,
        "state": "Louisiana",
        "county": "St. Helena Parish",
        "rate": 8.5,
    },
    {
        "id": 22093,
        "state": "Louisiana",
        "county": "St. James Parish",
        "rate": 8.1,
    },
    {
        "id": 22095,
        "state": "Louisiana",
        "county": "St. John the Baptist Parish",
        "rate": 7.5,
    },
    {
        "id": 22097,
        "state": "Louisiana",
        "county": "St. Landry Parish",
        "rate": 8.9,
    },
    {
        "id": 22099,
        "state": "Louisiana",
        "county": "St. Martin Parish",
        "rate": 8.3,
    },
    {
        "id": 22101,
        "state": "Louisiana",
        "county": "St. Mary Parish",
        "rate": 10.1,
    },
    {
        "id": 22103,
        "state": "Louisiana",
        "county": "St. Tammany Parish",
        "rate": 5.7,
    },
    {
        "id": 22105,
        "state": "Louisiana",
        "county": "Tangipahoa Parish",
        "rate": 7.5,
    },
    {
        "id": 22107,
        "state": "Louisiana",
        "county": "Tensas Parish",
        "rate": 10.5,
    },
    {
        "id": 22109,
        "state": "Louisiana",
        "county": "Terrebonne Parish",
        "rate": 7.3,
    },
    {"id": 22111, "state": "Louisiana", "county": "Union Parish", "rate": 6.9},
    {
        "id": 22113,
        "state": "Louisiana",
        "county": "Vermilion Parish",
        "rate": 7.9,
    },
    {"id": 22115, "state": "Louisiana", "county": "Vernon Parish", "rate": 8.1},
    {
        "id": 22117,
        "state": "Louisiana",
        "county": "Washington Parish",
        "rate": 7.9,
    },
    {
        "id": 22119,
        "state": "Louisiana",
        "county": "Webster Parish",
        "rate": 9.4,
    },
    {
        "id": 22121,
        "state": "Louisiana",
        "county": "West Baton Rouge Parish",
        "rate": 6.1,
    },
    {
        "id": 22123,
        "state": "Louisiana",
        "county": "West Carroll Parish",
        "rate": 12,
    },
    {
        "id": 22125,
        "state": "Louisiana",
        "county": "West Feliciana Parish",
        "rate": 5.5,
    },
    {"id": 22127, "state": "Louisiana", "county": "Winn Parish", "rate": 8.5},
    {
        "id": 23001,
        "state": "Maine",
        "county": "Androscoggin County",
        "rate": 3.2,
    },
    {"id": 23003, "state": "Maine", "county": "Aroostook County", "rate": 4.8},
    {"id": 23005, "state": "Maine", "county": "Cumberland County", "rate": 2.6},
    {"id": 23007, "state": "Maine", "county": "Franklin County", "rate": 4},
    {"id": 23009, "state": "Maine", "county": "Hancock County", "rate": 3.2},
    {"id": 23011, "state": "Maine", "county": "Kennebec County", "rate": 3.2},
    {"id": 23013, "state": "Maine", "county": "Knox County", "rate": 2.7},
    {"id": 23015, "state": "Maine", "county": "Lincoln County", "rate": 2.9},
    {"id": 23017, "state": "Maine", "county": "Oxford County", "rate": 4.2},
    {"id": 23019, "state": "Maine", "county": "Penobscot County", "rate": 4},
    {
        "id": 23021,
        "state": "Maine",
        "county": "Piscataquis County",
        "rate": 4.8,
    },
    {"id": 23023, "state": "Maine", "county": "Sagadahoc County", "rate": 2.6},
    {"id": 23025, "state": "Maine", "county": "Somerset County", "rate": 4.9},
    {"id": 23027, "state": "Maine", "county": "Waldo County", "rate": 3.6},
    {"id": 23029, "state": "Maine", "county": "Washington County", "rate": 4.6},
    {"id": 23031, "state": "Maine", "county": "York County", "rate": 2.9},
    {
        "id": 24001,
        "state": "Maryland",
        "county": "Allegany County",
        "rate": 6.4,
    },
    {
        "id": 24003,
        "state": "Maryland",
        "county": "Anne Arundel County",
        "rate": 4,
    },
    {
        "id": 24005,
        "state": "Maryland",
        "county": "Baltimore County",
        "rate": 4.7,
    },
    {"id": 24009, "state": "Maryland", "county": "Calvert County", "rate": 4.1},
    {
        "id": 24011,
        "state": "Maryland",
        "county": "Caroline County",
        "rate": 4.8,
    },
    {"id": 24013, "state": "Maryland", "county": "Carroll County", "rate": 3.8},
    {"id": 24015, "state": "Maryland", "county": "Cecil County", "rate": 5.3},
    {"id": 24017, "state": "Maryland", "county": "Charles County", "rate": 4.7},
    {
        "id": 24019,
        "state": "Maryland",
        "county": "Dorchester County",
        "rate": 6.2,
    },
    {
        "id": 24021,
        "state": "Maryland",
        "county": "Frederick County",
        "rate": 3.9,
    },
    {"id": 24023, "state": "Maryland", "county": "Garrett County", "rate": 5.2},
    {"id": 24025, "state": "Maryland", "county": "Harford County", "rate": 4.3},
    {"id": 24027, "state": "Maryland", "county": "Howard County", "rate": 3.4},
    {"id": 24029, "state": "Maryland", "county": "Kent County", "rate": 4.7},
    {
        "id": 24031,
        "state": "Maryland",
        "county": "Montgomery County",
        "rate": 3.4,
    },
    {
        "id": 24033,
        "state": "Maryland",
        "county": "Prince George's County",
        "rate": 4.6,
    },
    {
        "id": 24035,
        "state": "Maryland",
        "county": "Queen Anne's County",
        "rate": 3.8,
    },
    {
        "id": 24037,
        "state": "Maryland",
        "county": "St. Mary's County",
        "rate": 4.5,
    },
    {
        "id": 24039,
        "state": "Maryland",
        "county": "Somerset County",
        "rate": 6.5,
    },
    {"id": 24041, "state": "Maryland", "county": "Talbot County", "rate": 4},
    {
        "id": 24043,
        "state": "Maryland",
        "county": "Washington County",
        "rate": 5.2,
    },
    {
        "id": 24045,
        "state": "Maryland",
        "county": "Wicomico County",
        "rate": 5.5,
    },
    {
        "id": 24047,
        "state": "Maryland",
        "county": "Worcester County",
        "rate": 5.8,
    },
    {"id": 24510, "state": "Maryland", "county": "Baltimore city", "rate": 6.9},
    {
        "id": 25001,
        "state": "Massachusetts",
        "county": "Barnstable County",
        "rate": 3.4,
    },
    {
        "id": 25003,
        "state": "Massachusetts",
        "county": "Berkshire County",
        "rate": 3.9,
    },
    {
        "id": 25005,
        "state": "Massachusetts",
        "county": "Bristol County",
        "rate": 4.7,
    },
    {
        "id": 25007,
        "state": "Massachusetts",
        "county": "Dukes County",
        "rate": 2.7,
    },
    {
        "id": 25009,
        "state": "Massachusetts",
        "county": "Essex County",
        "rate": 3.9,
    },
    {
        "id": 25011,
        "state": "Massachusetts",
        "county": "Franklin County",
        "rate": 3.3,
    },
    {
        "id": 25013,
        "state": "Massachusetts",
        "county": "Hampden County",
        "rate": 5.3,
    },
    {
        "id": 25015,
        "state": "Massachusetts",
        "county": "Hampshire County",
        "rate": 3.6,
    },
    {
        "id": 25017,
        "state": "Massachusetts",
        "county": "Middlesex County",
        "rate": 3.2,
    },
    {
        "id": 25019,
        "state": "Massachusetts",
        "county": "Nantucket County",
        "rate": 1.8,
    },
    {
        "id": 25021,
        "state": "Massachusetts",
        "county": "Norfolk County",
        "rate": 3.4,
    },
    {
        "id": 25023,
        "state": "Massachusetts",
        "county": "Plymouth County",
        "rate": 4,
    },
    {
        "id": 25025,
        "state": "Massachusetts",
        "county": "Suffolk County",
        "rate": 3.7,
    },
    {
        "id": 25027,
        "state": "Massachusetts",
        "county": "Worcester County",
        "rate": 4.1,
    },
    {"id": 26001, "state": "Michigan", "county": "Alcona County", "rate": 5.8},
    {"id": 26003, "state": "Michigan", "county": "Alger County", "rate": 6.1},
    {"id": 26005, "state": "Michigan", "county": "Allegan County", "rate": 3.3},
    {"id": 26007, "state": "Michigan", "county": "Alpena County", "rate": 5},
    {"id": 26009, "state": "Michigan", "county": "Antrim County", "rate": 5},
    {"id": 26011, "state": "Michigan", "county": "Arenac County", "rate": 6.3},
    {"id": 26013, "state": "Michigan", "county": "Baraga County", "rate": 6.8},
    {"id": 26015, "state": "Michigan", "county": "Barry County", "rate": 3.5},
    {"id": 26017, "state": "Michigan", "county": "Bay County", "rate": 4.7},
    {"id": 26019, "state": "Michigan", "county": "Benzie County", "rate": 4.6},
    {"id": 26021, "state": "Michigan", "county": "Berrien County", "rate": 4.4},
    {"id": 26023, "state": "Michigan", "county": "Branch County", "rate": 4.4},
    {"id": 26025, "state": "Michigan", "county": "Calhoun County", "rate": 4.4},
    {"id": 26027, "state": "Michigan", "county": "Cass County", "rate": 4.2},
    {
        "id": 26029,
        "state": "Michigan",
        "county": "Charlevoix County",
        "rate": 3.7,
    },
    {"id": 26031, "state": "Michigan", "county": "Cheboygan County", "rate": 4},
    {"id": 26033, "state": "Michigan", "county": "Chippewa County", "rate": 6},
    {"id": 26035, "state": "Michigan", "county": "Clare County", "rate": 6},
    {"id": 26037, "state": "Michigan", "county": "Clinton County", "rate": 3.4},
    {
        "id": 26039,
        "state": "Michigan",
        "county": "Crawford County",
        "rate": 6.2,
    },
    {"id": 26041, "state": "Michigan", "county": "Delta County", "rate": 5.5},
    {
        "id": 26043,
        "state": "Michigan",
        "county": "Dickinson County",
        "rate": 4.5,
    },
    {"id": 26045, "state": "Michigan", "county": "Eaton County", "rate": 3.6},
    {"id": 26047, "state": "Michigan", "county": "Emmet County", "rate": 4.1},
    {"id": 26049, "state": "Michigan", "county": "Genesee County", "rate": 5},
    {"id": 26051, "state": "Michigan", "county": "Gladwin County", "rate": 5.8},
    {"id": 26053, "state": "Michigan", "county": "Gogebic County", "rate": 5.5},
    {
        "id": 26055,
        "state": "Michigan",
        "county": "Grand Traverse County",
        "rate": 3.4,
    },
    {"id": 26057, "state": "Michigan", "county": "Gratiot County", "rate": 4.5},
    {
        "id": 26059,
        "state": "Michigan",
        "county": "Hillsdale County",
        "rate": 4.9,
    },
    {
        "id": 26061,
        "state": "Michigan",
        "county": "Houghton County",
        "rate": 5.2,
    },
    {"id": 26063, "state": "Michigan", "county": "Huron County", "rate": 4.3},
    {"id": 26065, "state": "Michigan", "county": "Ingham County", "rate": 4},
    {"id": 26067, "state": "Michigan", "county": "Ionia County", "rate": 3.6},
    {"id": 26069, "state": "Michigan", "county": "Iosco County", "rate": 5.8},
    {"id": 26071, "state": "Michigan", "county": "Iron County", "rate": 4.7},
    {
        "id": 26073,
        "state": "Michigan",
        "county": "Isabella County",
        "rate": 4.2,
    },
    {"id": 26075, "state": "Michigan", "county": "Jackson County", "rate": 4.6},
    {
        "id": 26077,
        "state": "Michigan",
        "county": "Kalamazoo County",
        "rate": 3.7,
    },
    {
        "id": 26079,
        "state": "Michigan",
        "county": "Kalkaska County",
        "rate": 5.4,
    },
    {"id": 26081, "state": "Michigan", "county": "Kent County", "rate": 3.2},
    {
        "id": 26083,
        "state": "Michigan",
        "county": "Keweenaw County",
        "rate": 6.4,
    },
    {"id": 26085, "state": "Michigan", "county": "Lake County", "rate": 6},
    {"id": 26087, "state": "Michigan", "county": "Lapeer County", "rate": 6.4},
    {
        "id": 26089,
        "state": "Michigan",
        "county": "Leelanau County",
        "rate": 3.5,
    },
    {"id": 26091, "state": "Michigan", "county": "Lenawee County", "rate": 4.4},
    {
        "id": 26093,
        "state": "Michigan",
        "county": "Livingston County",
        "rate": 4.6,
    },
    {"id": 26095, "state": "Michigan", "county": "Luce County", "rate": 4.4},
    {
        "id": 26097,
        "state": "Michigan",
        "county": "Mackinac County",
        "rate": 3.1,
    },
    {"id": 26099, "state": "Michigan", "county": "Macomb County", "rate": 5.9},
    {
        "id": 26101,
        "state": "Michigan",
        "county": "Manistee County",
        "rate": 4.9,
    },
    {
        "id": 26103,
        "state": "Michigan",
        "county": "Marquette County",
        "rate": 4.9,
    },
    {"id": 26105, "state": "Michigan", "county": "Mason County", "rate": 4.2},
    {"id": 26107, "state": "Michigan", "county": "Mecosta County", "rate": 4.9},
    {
        "id": 26109,
        "state": "Michigan",
        "county": "Menominee County",
        "rate": 5.1,
    },
    {"id": 26111, "state": "Michigan", "county": "Midland County", "rate": 4.1},
    {
        "id": 26113,
        "state": "Michigan",
        "county": "Missaukee County",
        "rate": 4.8,
    },
    {"id": 26115, "state": "Michigan", "county": "Monroe County", "rate": 4.3},
    {
        "id": 26117,
        "state": "Michigan",
        "county": "Montcalm County",
        "rate": 4.5,
    },
    {
        "id": 26119,
        "state": "Michigan",
        "county": "Montmorency County",
        "rate": 7.6,
    },
    {
        "id": 26121,
        "state": "Michigan",
        "county": "Muskegon County",
        "rate": 4.8,
    },
    {"id": 26123, "state": "Michigan", "county": "Newaygo County", "rate": 4.2},
    {"id": 26125, "state": "Michigan", "county": "Oakland County", "rate": 4.7},
    {"id": 26127, "state": "Michigan", "county": "Oceana County", "rate": 5},
    {"id": 26129, "state": "Michigan", "county": "Ogemaw County", "rate": 6.3},
    {
        "id": 26131,
        "state": "Michigan",
        "county": "Ontonagon County",
        "rate": 6.6,
    },
    {"id": 26133, "state": "Michigan", "county": "Osceola County", "rate": 4.9},
    {"id": 26135, "state": "Michigan", "county": "Oscoda County", "rate": 5.6},
    {"id": 26137, "state": "Michigan", "county": "Otsego County", "rate": 4.9},
    {"id": 26139, "state": "Michigan", "county": "Ottawa County", "rate": 3},
    {
        "id": 26141,
        "state": "Michigan",
        "county": "Presque Isle County",
        "rate": 6.6,
    },
    {
        "id": 26143,
        "state": "Michigan",
        "county": "Roscommon County",
        "rate": 6.8,
    },
    {"id": 26145, "state": "Michigan", "county": "Saginaw County", "rate": 4.6},
    {
        "id": 26147,
        "state": "Michigan",
        "county": "St. Clair County",
        "rate": 6.5,
    },
    {
        "id": 26149,
        "state": "Michigan",
        "county": "St. Joseph County",
        "rate": 3.7,
    },
    {"id": 26151, "state": "Michigan", "county": "Sanilac County", "rate": 5.2},
    {
        "id": 26153,
        "state": "Michigan",
        "county": "Schoolcraft County",
        "rate": 7,
    },
    {
        "id": 26155,
        "state": "Michigan",
        "county": "Shiawassee County",
        "rate": 4.6,
    },
    {"id": 26157, "state": "Michigan", "county": "Tuscola County", "rate": 5.3},
    {
        "id": 26159,
        "state": "Michigan",
        "county": "Van Buren County",
        "rate": 4.6,
    },
    {
        "id": 26161,
        "state": "Michigan",
        "county": "Washtenaw County",
        "rate": 3.5,
    },
    {"id": 26163, "state": "Michigan", "county": "Wayne County", "rate": 7.3},
    {"id": 26165, "state": "Michigan", "county": "Wexford County", "rate": 5.1},
    {"id": 27001, "state": "Minnesota", "county": "Aitkin County", "rate": 5.1},
    {"id": 27003, "state": "Minnesota", "county": "Anoka County", "rate": 3.8},
    {"id": 27005, "state": "Minnesota", "county": "Becker County", "rate": 3.8},
    {
        "id": 27007,
        "state": "Minnesota",
        "county": "Beltrami County",
        "rate": 4.8,
    },
    {"id": 27009, "state": "Minnesota", "county": "Benton County", "rate": 3.7},
    {
        "id": 27011,
        "state": "Minnesota",
        "county": "Big Stone County",
        "rate": 4,
    },
    {
        "id": 27013,
        "state": "Minnesota",
        "county": "Blue Earth County",
        "rate": 3.3,
    },
    {"id": 27015, "state": "Minnesota", "county": "Brown County", "rate": 3.9},
    {
        "id": 27017,
        "state": "Minnesota",
        "county": "Carlton County",
        "rate": 4.9,
    },
    {"id": 27019, "state": "Minnesota", "county": "Carver County", "rate": 3.4},
    {"id": 27021, "state": "Minnesota", "county": "Cass County", "rate": 5.6},
    {"id": 27023, "state": "Minnesota", "county": "Chippewa County", "rate": 4},
    {
        "id": 27025,
        "state": "Minnesota",
        "county": "Chisago County",
        "rate": 3.9,
    },
    {"id": 27027, "state": "Minnesota", "county": "Clay County", "rate": 3.2},
    {
        "id": 27029,
        "state": "Minnesota",
        "county": "Clearwater County",
        "rate": 7.6,
    },
    {"id": 27031, "state": "Minnesota", "county": "Cook County", "rate": 3.1},
    {
        "id": 27033,
        "state": "Minnesota",
        "county": "Cottonwood County",
        "rate": 7.9,
    },
    {
        "id": 27035,
        "state": "Minnesota",
        "county": "Crow Wing County",
        "rate": 4.3,
    },
    {"id": 27037, "state": "Minnesota", "county": "Dakota County", "rate": 3.5},
    {"id": 27039, "state": "Minnesota", "county": "Dodge County", "rate": 3.6},
    {"id": 27041, "state": "Minnesota", "county": "Douglas County", "rate": 3},
    {
        "id": 27043,
        "state": "Minnesota",
        "county": "Faribault County",
        "rate": 3.9,
    },
    {
        "id": 27045,
        "state": "Minnesota",
        "county": "Fillmore County",
        "rate": 3.7,
    },
    {
        "id": 27047,
        "state": "Minnesota",
        "county": "Freeborn County",
        "rate": 3.7,
    },
    {
        "id": 27049,
        "state": "Minnesota",
        "county": "Goodhue County",
        "rate": 3.7,
    },
    {"id": 27051, "state": "Minnesota", "county": "Grant County", "rate": 4.1},
    {
        "id": 27053,
        "state": "Minnesota",
        "county": "Hennepin County",
        "rate": 3.5,
    },
    {
        "id": 27055,
        "state": "Minnesota",
        "county": "Houston County",
        "rate": 3.4,
    },
    {
        "id": 27057,
        "state": "Minnesota",
        "county": "Hubbard County",
        "rate": 5.5,
    },
    {"id": 27059, "state": "Minnesota", "county": "Isanti County", "rate": 4.2},
    {"id": 27061, "state": "Minnesota", "county": "Itasca County", "rate": 7.7},
    {"id": 27063, "state": "Minnesota", "county": "Jackson County", "rate": 4},
    {
        "id": 27065,
        "state": "Minnesota",
        "county": "Kanabec County",
        "rate": 5.1,
    },
    {
        "id": 27067,
        "state": "Minnesota",
        "county": "Kandiyohi County",
        "rate": 3.5,
    },
    {
        "id": 27069,
        "state": "Minnesota",
        "county": "Kittson County",
        "rate": 4.2,
    },
    {
        "id": 27071,
        "state": "Minnesota",
        "county": "Koochiching County",
        "rate": 7.7,
    },
    {
        "id": 27073,
        "state": "Minnesota",
        "county": "Lac qui Parle County",
        "rate": 3.9,
    },
    {"id": 27075, "state": "Minnesota", "county": "Lake County", "rate": 4.3},
    {
        "id": 27077,
        "state": "Minnesota",
        "county": "Lake of the Woods County",
        "rate": 4.5,
    },
    {
        "id": 27079,
        "state": "Minnesota",
        "county": "Le Sueur County",
        "rate": 3.7,
    },
    {
        "id": 27081,
        "state": "Minnesota",
        "county": "Lincoln County",
        "rate": 3.2,
    },
    {"id": 27083, "state": "Minnesota", "county": "Lyon County", "rate": 3.4},
    {"id": 27085, "state": "Minnesota", "county": "McLeod County", "rate": 4.1},
    {
        "id": 27087,
        "state": "Minnesota",
        "county": "Mahnomen County",
        "rate": 5.2,
    },
    {
        "id": 27089,
        "state": "Minnesota",
        "county": "Marshall County",
        "rate": 5.6,
    },
    {"id": 27091, "state": "Minnesota", "county": "Martin County", "rate": 3.9},
    {"id": 27093, "state": "Minnesota", "county": "Meeker County", "rate": 4},
    {
        "id": 27095,
        "state": "Minnesota",
        "county": "Mille Lacs County",
        "rate": 4.8,
    },
    {
        "id": 27097,
        "state": "Minnesota",
        "county": "Morrison County",
        "rate": 4.5,
    },
    {"id": 27099, "state": "Minnesota", "county": "Mower County", "rate": 3},
    {"id": 27101, "state": "Minnesota", "county": "Murray County", "rate": 3.3},
    {
        "id": 27103,
        "state": "Minnesota",
        "county": "Nicollet County",
        "rate": 2.9,
    },
    {"id": 27105, "state": "Minnesota", "county": "Nobles County", "rate": 3.6},
    {"id": 27107, "state": "Minnesota", "county": "Norman County", "rate": 4.6},
    {"id": 27109, "state": "Minnesota", "county": "Olmsted County", "rate": 3},
    {
        "id": 27111,
        "state": "Minnesota",
        "county": "Otter Tail County",
        "rate": 3.6,
    },
    {
        "id": 27113,
        "state": "Minnesota",
        "county": "Pennington County",
        "rate": 4.3,
    },
    {"id": 27115, "state": "Minnesota", "county": "Pine County", "rate": 4.8},
    {
        "id": 27117,
        "state": "Minnesota",
        "county": "Pipestone County",
        "rate": 3,
    },
    {"id": 27119, "state": "Minnesota", "county": "Polk County", "rate": 4.4},
    {"id": 27121, "state": "Minnesota", "county": "Pope County", "rate": 2.8},
    {"id": 27123, "state": "Minnesota", "county": "Ramsey County", "rate": 3.8},
    {
        "id": 27125,
        "state": "Minnesota",
        "county": "Red Lake County",
        "rate": 6.3,
    },
    {"id": 27127, "state": "Minnesota", "county": "Redwood County", "rate": 4},
    {
        "id": 27129,
        "state": "Minnesota",
        "county": "Renville County",
        "rate": 5.2,
    },
    {"id": 27131, "state": "Minnesota", "county": "Rice County", "rate": 3.6},
    {"id": 27133, "state": "Minnesota", "county": "Rock County", "rate": 2.2},
    {"id": 27135, "state": "Minnesota", "county": "Roseau County", "rate": 4.3},
    {
        "id": 27137,
        "state": "Minnesota",
        "county": "St. Louis County",
        "rate": 5.7,
    },
    {"id": 27139, "state": "Minnesota", "county": "Scott County", "rate": 3.2},
    {
        "id": 27141,
        "state": "Minnesota",
        "county": "Sherburne County",
        "rate": 3.6,
    },
    {"id": 27143, "state": "Minnesota", "county": "Sibley County", "rate": 3.7},
    {
        "id": 27145,
        "state": "Minnesota",
        "county": "Stearns County",
        "rate": 3.5,
    },
    {"id": 27147, "state": "Minnesota", "county": "Steele County", "rate": 3.8},
    {
        "id": 27149,
        "state": "Minnesota",
        "county": "Stevens County",
        "rate": 2.7,
    },
    {"id": 27151, "state": "Minnesota", "county": "Swift County", "rate": 4.8},
    {"id": 27153, "state": "Minnesota", "county": "Todd County", "rate": 3.9},
    {"id": 27155, "state": "Minnesota", "county": "Traverse County", "rate": 3},
    {
        "id": 27157,
        "state": "Minnesota",
        "county": "Wabasha County",
        "rate": 3.2,
    },
    {"id": 27159, "state": "Minnesota", "county": "Wadena County", "rate": 5.1},
    {"id": 27161, "state": "Minnesota", "county": "Waseca County", "rate": 4},
    {
        "id": 27163,
        "state": "Minnesota",
        "county": "Washington County",
        "rate": 3.4,
    },
    {
        "id": 27165,
        "state": "Minnesota",
        "county": "Watonwan County",
        "rate": 5.1,
    },
    {"id": 27167, "state": "Minnesota", "county": "Wilkin County", "rate": 3.4},
    {"id": 27169, "state": "Minnesota", "county": "Winona County", "rate": 3.7},
    {"id": 27171, "state": "Minnesota", "county": "Wright County", "rate": 3.5},
    {
        "id": 27173,
        "state": "Minnesota",
        "county": "Yellow Medicine County",
        "rate": 3.7,
    },
    {
        "id": 28001,
        "state": "Mississippi",
        "county": "Adams County",
        "rate": 8.2,
    },
    {"id": 28003, "state": "Mississippi", "county": "Alcorn County", "rate": 5},
    {
        "id": 28005,
        "state": "Mississippi",
        "county": "Amite County",
        "rate": 7.1,
    },
    {
        "id": 28007,
        "state": "Mississippi",
        "county": "Attala County",
        "rate": 6.5,
    },
    {
        "id": 28009,
        "state": "Mississippi",
        "county": "Benton County",
        "rate": 6.5,
    },
    {
        "id": 28011,
        "state": "Mississippi",
        "county": "Bolivar County",
        "rate": 7.4,
    },
    {
        "id": 28013,
        "state": "Mississippi",
        "county": "Calhoun County",
        "rate": 5.2,
    },
    {
        "id": 28015,
        "state": "Mississippi",
        "county": "Carroll County",
        "rate": 7.3,
    },
    {
        "id": 28017,
        "state": "Mississippi",
        "county": "Chickasaw County",
        "rate": 6.3,
    },
    {
        "id": 28019,
        "state": "Mississippi",
        "county": "Choctaw County",
        "rate": 5.3,
    },
    {
        "id": 28021,
        "state": "Mississippi",
        "county": "Claiborne County",
        "rate": 12.3,
    },
    {
        "id": 28023,
        "state": "Mississippi",
        "county": "Clarke County",
        "rate": 6.4,
    },
    {"id": 28025, "state": "Mississippi", "county": "Clay County", "rate": 7.8},
    {
        "id": 28027,
        "state": "Mississippi",
        "county": "Coahoma County",
        "rate": 8.1,
    },
    {
        "id": 28029,
        "state": "Mississippi",
        "county": "Copiah County",
        "rate": 6.4,
    },
    {
        "id": 28031,
        "state": "Mississippi",
        "county": "Covington County",
        "rate": 5.6,
    },
    {
        "id": 28033,
        "state": "Mississippi",
        "county": "DeSoto County",
        "rate": 4.1,
    },
    {
        "id": 28035,
        "state": "Mississippi",
        "county": "Forrest County",
        "rate": 5.8,
    },
    {
        "id": 28037,
        "state": "Mississippi",
        "county": "Franklin County",
        "rate": 6.8,
    },
    {
        "id": 28039,
        "state": "Mississippi",
        "county": "George County",
        "rate": 7.7,
    },
    {"id": 28041, "state": "Mississippi", "county": "Greene County", "rate": 8},
    {
        "id": 28043,
        "state": "Mississippi",
        "county": "Grenada County",
        "rate": 4.9,
    },
    {
        "id": 28045,
        "state": "Mississippi",
        "county": "Hancock County",
        "rate": 5.9,
    },
    {
        "id": 28047,
        "state": "Mississippi",
        "county": "Harrison County",
        "rate": 5.3,
    },
    {
        "id": 28049,
        "state": "Mississippi",
        "county": "Hinds County",
        "rate": 5.2,
    },
    {
        "id": 28051,
        "state": "Mississippi",
        "county": "Holmes County",
        "rate": 10.9,
    },
    {
        "id": 28053,
        "state": "Mississippi",
        "county": "Humphreys County",
        "rate": 11.2,
    },
    {
        "id": 28055,
        "state": "Mississippi",
        "county": "Issaquena County",
        "rate": 9.9,
    },
    {
        "id": 28057,
        "state": "Mississippi",
        "county": "Itawamba County",
        "rate": 4.8,
    },
    {
        "id": 28059,
        "state": "Mississippi",
        "county": "Jackson County",
        "rate": 6.4,
    },
    {
        "id": 28061,
        "state": "Mississippi",
        "county": "Jasper County",
        "rate": 7.8,
    },
    {
        "id": 28063,
        "state": "Mississippi",
        "county": "Jefferson County",
        "rate": 15.4,
    },
    {
        "id": 28065,
        "state": "Mississippi",
        "county": "Jefferson Davis County",
        "rate": 9.3,
    },
    {
        "id": 28067,
        "state": "Mississippi",
        "county": "Jones County",
        "rate": 5.9,
    },
    {
        "id": 28069,
        "state": "Mississippi",
        "county": "Kemper County",
        "rate": 8.2,
    },
    {
        "id": 28071,
        "state": "Mississippi",
        "county": "Lafayette County",
        "rate": 4.9,
    },
    {
        "id": 28073,
        "state": "Mississippi",
        "county": "Lamar County",
        "rate": 4.5,
    },
    {
        "id": 28075,
        "state": "Mississippi",
        "county": "Lauderdale County",
        "rate": 5.8,
    },
    {
        "id": 28077,
        "state": "Mississippi",
        "county": "Lawrence County",
        "rate": 7.4,
    },
    {
        "id": 28079,
        "state": "Mississippi",
        "county": "Leake County",
        "rate": 5.6,
    },
    {"id": 28081, "state": "Mississippi", "county": "Lee County", "rate": 4.8},
    {
        "id": 28083,
        "state": "Mississippi",
        "county": "Leflore County",
        "rate": 9,
    },
    {
        "id": 28085,
        "state": "Mississippi",
        "county": "Lincoln County",
        "rate": 5.8,
    },
    {
        "id": 28087,
        "state": "Mississippi",
        "county": "Lowndes County",
        "rate": 5.9,
    },
    {
        "id": 28089,
        "state": "Mississippi",
        "county": "Madison County",
        "rate": 4.2,
    },
    {
        "id": 28091,
        "state": "Mississippi",
        "county": "Marion County",
        "rate": 6.8,
    },
    {
        "id": 28093,
        "state": "Mississippi",
        "county": "Marshall County",
        "rate": 5.7,
    },
    {
        "id": 28095,
        "state": "Mississippi",
        "county": "Monroe County",
        "rate": 6.3,
    },
    {
        "id": 28097,
        "state": "Mississippi",
        "county": "Montgomery County",
        "rate": 6.3,
    },
    {
        "id": 28099,
        "state": "Mississippi",
        "county": "Neshoba County",
        "rate": 5.4,
    },
    {
        "id": 28101,
        "state": "Mississippi",
        "county": "Newton County",
        "rate": 5.9,
    },
    {
        "id": 28103,
        "state": "Mississippi",
        "county": "Noxubee County",
        "rate": 8,
    },
    {
        "id": 28105,
        "state": "Mississippi",
        "county": "Oktibbeha County",
        "rate": 5.6,
    },
    {
        "id": 28107,
        "state": "Mississippi",
        "county": "Panola County",
        "rate": 6.7,
    },
    {
        "id": 28109,
        "state": "Mississippi",
        "county": "Pearl River County",
        "rate": 5.9,
    },
    {"id": 28111, "state": "Mississippi", "county": "Perry County", "rate": 7},
    {"id": 28113, "state": "Mississippi", "county": "Pike County", "rate": 6.7},
    {
        "id": 28115,
        "state": "Mississippi",
        "county": "Pontotoc County",
        "rate": 4.5,
    },
    {
        "id": 28117,
        "state": "Mississippi",
        "county": "Prentiss County",
        "rate": 5.3,
    },
    {
        "id": 28119,
        "state": "Mississippi",
        "county": "Quitman County",
        "rate": 8.3,
    },
    {
        "id": 28121,
        "state": "Mississippi",
        "county": "Rankin County",
        "rate": 3.8,
    },
    {
        "id": 28123,
        "state": "Mississippi",
        "county": "Scott County",
        "rate": 4.5,
    },
    {
        "id": 28125,
        "state": "Mississippi",
        "county": "Sharkey County",
        "rate": 8,
    },
    {
        "id": 28127,
        "state": "Mississippi",
        "county": "Simpson County",
        "rate": 5.5,
    },
    {
        "id": 28129,
        "state": "Mississippi",
        "county": "Smith County",
        "rate": 5.1,
    },
    {
        "id": 28131,
        "state": "Mississippi",
        "county": "Stone County",
        "rate": 6.8,
    },
    {
        "id": 28133,
        "state": "Mississippi",
        "county": "Sunflower County",
        "rate": 8.4,
    },
    {
        "id": 28135,
        "state": "Mississippi",
        "county": "Tallahatchie County",
        "rate": 5.7,
    },
    {"id": 28137, "state": "Mississippi", "county": "Tate County", "rate": 5.4},
    {
        "id": 28139,
        "state": "Mississippi",
        "county": "Tippah County",
        "rate": 5.2,
    },
    {
        "id": 28141,
        "state": "Mississippi",
        "county": "Tishomingo County",
        "rate": 5.1,
    },
    {
        "id": 28143,
        "state": "Mississippi",
        "county": "Tunica County",
        "rate": 6.1,
    },
    {
        "id": 28145,
        "state": "Mississippi",
        "county": "Union County",
        "rate": 4.2,
    },
    {
        "id": 28147,
        "state": "Mississippi",
        "county": "Walthall County",
        "rate": 7.9,
    },
    {
        "id": 28149,
        "state": "Mississippi",
        "county": "Warren County",
        "rate": 6.4,
    },
    {
        "id": 28151,
        "state": "Mississippi",
        "county": "Washington County",
        "rate": 9.2,
    },
    {
        "id": 28153,
        "state": "Mississippi",
        "county": "Wayne County",
        "rate": 7.6,
    },
    {
        "id": 28155,
        "state": "Mississippi",
        "county": "Webster County",
        "rate": 6.1,
    },
    {
        "id": 28157,
        "state": "Mississippi",
        "county": "Wilkinson County",
        "rate": 10.1,
    },
    {
        "id": 28159,
        "state": "Mississippi",
        "county": "Winston County",
        "rate": 7.4,
    },
    {
        "id": 28161,
        "state": "Mississippi",
        "county": "Yalobusha County",
        "rate": 6,
    },
    {"id": 28163, "state": "Mississippi", "county": "Yazoo County", "rate": 7},
    {"id": 29001, "state": "Missouri", "county": "Adair County", "rate": 6.7},
    {"id": 29003, "state": "Missouri", "county": "Andrew County", "rate": 4.5},
    {
        "id": 29005,
        "state": "Missouri",
        "county": "Atchison County",
        "rate": 5.9,
    },
    {"id": 29007, "state": "Missouri", "county": "Audrain County", "rate": 5.1},
    {"id": 29009, "state": "Missouri", "county": "Barry County", "rate": 5.3},
    {"id": 29011, "state": "Missouri", "county": "Barton County", "rate": 6},
    {"id": 29013, "state": "Missouri", "county": "Bates County", "rate": 6},
    {"id": 29015, "state": "Missouri", "county": "Benton County", "rate": 7.3},
    {
        "id": 29017,
        "state": "Missouri",
        "county": "Bollinger County",
        "rate": 6.2,
    },
    {"id": 29019, "state": "Missouri", "county": "Boone County", "rate": 4.1},
    {
        "id": 29021,
        "state": "Missouri",
        "county": "Buchanan County",
        "rate": 4.9,
    },
    {"id": 29023, "state": "Missouri", "county": "Butler County", "rate": 6.9},
    {
        "id": 29025,
        "state": "Missouri",
        "county": "Caldwell County",
        "rate": 5.6,
    },
    {
        "id": 29027,
        "state": "Missouri",
        "county": "Callaway County",
        "rate": 5.2,
    },
    {"id": 29029, "state": "Missouri", "county": "Camden County", "rate": 5.5},
    {
        "id": 29031,
        "state": "Missouri",
        "county": "Cape Girardeau County",
        "rate": 5.3,
    },
    {"id": 29033, "state": "Missouri", "county": "Carroll County", "rate": 5.9},
    {"id": 29035, "state": "Missouri", "county": "Carter County", "rate": 7.8},
    {"id": 29037, "state": "Missouri", "county": "Cass County", "rate": 5},
    {"id": 29039, "state": "Missouri", "county": "Cedar County", "rate": 6.4},
    {
        "id": 29041,
        "state": "Missouri",
        "county": "Chariton County",
        "rate": 5.1,
    },
    {
        "id": 29043,
        "state": "Missouri",
        "county": "Christian County",
        "rate": 4.5,
    },
    {"id": 29045, "state": "Missouri", "county": "Clark County", "rate": 8.7},
    {"id": 29047, "state": "Missouri", "county": "Clay County", "rate": 4.5},
    {"id": 29049, "state": "Missouri", "county": "Clinton County", "rate": 5},
    {"id": 29051, "state": "Missouri", "county": "Cole County", "rate": 4.6},
    {"id": 29053, "state": "Missouri", "county": "Cooper County", "rate": 5.7},
    {
        "id": 29055,
        "state": "Missouri",
        "county": "Crawford County",
        "rate": 5.9,
    },
    {"id": 29057, "state": "Missouri", "county": "Dade County", "rate": 5.6},
    {"id": 29059, "state": "Missouri", "county": "Dallas County", "rate": 6.5},
    {"id": 29061, "state": "Missouri", "county": "Daviess County", "rate": 4.6},
    {"id": 29063, "state": "Missouri", "county": "DeKalb County", "rate": 5.1},
    {"id": 29065, "state": "Missouri", "county": "Dent County", "rate": 6.6},
    {"id": 29067, "state": "Missouri", "county": "Douglas County", "rate": 7.3},
    {"id": 29069, "state": "Missouri", "county": "Dunklin County", "rate": 9.1},
    {
        "id": 29071,
        "state": "Missouri",
        "county": "Franklin County",
        "rate": 4.9,
    },
    {"id": 29073, "state": "Missouri", "county": "Gasconade County", "rate": 5},
    {"id": 29075, "state": "Missouri", "county": "Gentry County", "rate": 4.7},
    {"id": 29077, "state": "Missouri", "county": "Greene County", "rate": 4.6},
    {"id": 29079, "state": "Missouri", "county": "Grundy County", "rate": 5.6},
    {
        "id": 29081,
        "state": "Missouri",
        "county": "Harrison County",
        "rate": 4.7,
    },
    {"id": 29083, "state": "Missouri", "county": "Henry County", "rate": 5.9},
    {"id": 29085, "state": "Missouri", "county": "Hickory County", "rate": 6.6},
    {"id": 29087, "state": "Missouri", "county": "Holt County", "rate": 4},
    {"id": 29089, "state": "Missouri", "county": "Howard County", "rate": 5.4},
    {"id": 29091, "state": "Missouri", "county": "Howell County", "rate": 7.2},
    {"id": 29093, "state": "Missouri", "county": "Iron County", "rate": 8.4},
    {"id": 29095, "state": "Missouri", "county": "Jackson County", "rate": 6},
    {"id": 29097, "state": "Missouri", "county": "Jasper County", "rate": 5.2},
    {
        "id": 29099,
        "state": "Missouri",
        "county": "Jefferson County",
        "rate": 4.9,
    },
    {"id": 29101, "state": "Missouri", "county": "Johnson County", "rate": 6.1},
    {"id": 29103, "state": "Missouri", "county": "Knox County", "rate": 4.1},
    {"id": 29105, "state": "Missouri", "county": "Laclede County", "rate": 6.2},
    {
        "id": 29107,
        "state": "Missouri",
        "county": "Lafayette County",
        "rate": 4.9,
    },
    {
        "id": 29109,
        "state": "Missouri",
        "county": "Lawrence County",
        "rate": 5.4,
    },
    {"id": 29111, "state": "Missouri", "county": "Lewis County", "rate": 4.9},
    {"id": 29113, "state": "Missouri", "county": "Lincoln County", "rate": 4.8},
    {"id": 29115, "state": "Missouri", "county": "Linn County", "rate": 9},
    {
        "id": 29117,
        "state": "Missouri",
        "county": "Livingston County",
        "rate": 4.5,
    },
    {
        "id": 29119,
        "state": "Missouri",
        "county": "McDonald County",
        "rate": 4.9,
    },
    {"id": 29121, "state": "Missouri", "county": "Macon County", "rate": 6},
    {"id": 29123, "state": "Missouri", "county": "Madison County", "rate": 6.9},
    {"id": 29125, "state": "Missouri", "county": "Maries County", "rate": 6.8},
    {"id": 29127, "state": "Missouri", "county": "Marion County", "rate": 5},
    {"id": 29129, "state": "Missouri", "county": "Mercer County", "rate": 5},
    {"id": 29131, "state": "Missouri", "county": "Miller County", "rate": 5.4},
    {
        "id": 29133,
        "state": "Missouri",
        "county": "Mississippi County",
        "rate": 7.7,
    },
    {
        "id": 29135,
        "state": "Missouri",
        "county": "Moniteau County",
        "rate": 4.9,
    },
    {"id": 29137, "state": "Missouri", "county": "Monroe County", "rate": 5.5},
    {
        "id": 29139,
        "state": "Missouri",
        "county": "Montgomery County",
        "rate": 5.1,
    },
    {"id": 29141, "state": "Missouri", "county": "Morgan County", "rate": 6.4},
    {
        "id": 29143,
        "state": "Missouri",
        "county": "New Madrid County",
        "rate": 9.9,
    },
    {"id": 29145, "state": "Missouri", "county": "Newton County", "rate": 5.5},
    {"id": 29147, "state": "Missouri", "county": "Nodaway County", "rate": 6},
    {"id": 29149, "state": "Missouri", "county": "Oregon County", "rate": 7},
    {"id": 29151, "state": "Missouri", "county": "Osage County", "rate": 4.2},
    {"id": 29153, "state": "Missouri", "county": "Ozark County", "rate": 8.8},
    {
        "id": 29155,
        "state": "Missouri",
        "county": "Pemiscot County",
        "rate": 9.9,
    },
    {"id": 29157, "state": "Missouri", "county": "Perry County", "rate": 4.3},
    {"id": 29159, "state": "Missouri", "county": "Pettis County", "rate": 6.1},
    {"id": 29161, "state": "Missouri", "county": "Phelps County", "rate": 5.6},
    {"id": 29163, "state": "Missouri", "county": "Pike County", "rate": 5.4},
    {"id": 29165, "state": "Missouri", "county": "Platte County", "rate": 4.4},
    {"id": 29167, "state": "Missouri", "county": "Polk County", "rate": 6.4},
    {"id": 29169, "state": "Missouri", "county": "Pulaski County", "rate": 6.8},
    {"id": 29171, "state": "Missouri", "county": "Putnam County", "rate": 5.1},
    {"id": 29173, "state": "Missouri", "county": "Ralls County", "rate": 4.4},
    {
        "id": 29175,
        "state": "Missouri",
        "county": "Randolph County",
        "rate": 6.6,
    },
    {"id": 29177, "state": "Missouri", "county": "Ray County", "rate": 5.5},
    {
        "id": 29179,
        "state": "Missouri",
        "county": "Reynolds County",
        "rate": 8.1,
    },
    {"id": 29181, "state": "Missouri", "county": "Ripley County", "rate": 8.4},
    {
        "id": 29183,
        "state": "Missouri",
        "county": "St. Charles County",
        "rate": 4.1,
    },
    {
        "id": 29185,
        "state": "Missouri",
        "county": "St. Clair County",
        "rate": 7.3,
    },
    {
        "id": 29186,
        "state": "Missouri",
        "county": "Ste. Genevieve County",
        "rate": 5.8,
    },
    {
        "id": 29187,
        "state": "Missouri",
        "county": "St. Francois County",
        "rate": 6.7,
    },
    {
        "id": 29189,
        "state": "Missouri",
        "county": "St. Louis County",
        "rate": 5.1,
    },
    {"id": 29195, "state": "Missouri", "county": "Saline County", "rate": 5.8},
    {
        "id": 29197,
        "state": "Missouri",
        "county": "Schuyler County",
        "rate": 7.6,
    },
    {
        "id": 29199,
        "state": "Missouri",
        "county": "Scotland County",
        "rate": 4.6,
    },
    {"id": 29201, "state": "Missouri", "county": "Scott County", "rate": 6.9},
    {"id": 29203, "state": "Missouri", "county": "Shannon County", "rate": 8},
    {"id": 29205, "state": "Missouri", "county": "Shelby County", "rate": 4.8},
    {
        "id": 29207,
        "state": "Missouri",
        "county": "Stoddard County",
        "rate": 7.1,
    },
    {"id": 29209, "state": "Missouri", "county": "Stone County", "rate": 6},
    {
        "id": 29211,
        "state": "Missouri",
        "county": "Sullivan County",
        "rate": 7.4,
    },
    {"id": 29213, "state": "Missouri", "county": "Taney County", "rate": 6.1},
    {"id": 29215, "state": "Missouri", "county": "Texas County", "rate": 7.6},
    {"id": 29217, "state": "Missouri", "county": "Vernon County", "rate": 5.8},
    {"id": 29219, "state": "Missouri", "county": "Warren County", "rate": 4.6},
    {
        "id": 29221,
        "state": "Missouri",
        "county": "Washington County",
        "rate": 7.2,
    },
    {"id": 29223, "state": "Missouri", "county": "Wayne County", "rate": 6.1},
    {"id": 29225, "state": "Missouri", "county": "Webster County", "rate": 5.9},
    {"id": 29227, "state": "Missouri", "county": "Worth County", "rate": 3.6},
    {"id": 29229, "state": "Missouri", "county": "Wright County", "rate": 7.3},
    {"id": 29510, "state": "Missouri", "county": "St. Louis city", "rate": 6.6},
    {
        "id": 30001,
        "state": "Montana",
        "county": "Beaverhead County",
        "rate": 2.9,
    },
    {"id": 30003, "state": "Montana", "county": "Big Horn County", "rate": 7.7},
    {"id": 30005, "state": "Montana", "county": "Blaine County", "rate": 4.4},
    {
        "id": 30007,
        "state": "Montana",
        "county": "Broadwater County",
        "rate": 4.3,
    },
    {"id": 30009, "state": "Montana", "county": "Carbon County", "rate": 3.5},
    {"id": 30011, "state": "Montana", "county": "Carter County", "rate": 2.4},
    {"id": 30013, "state": "Montana", "county": "Cascade County", "rate": 3.8},
    {"id": 30015, "state": "Montana", "county": "Chouteau County", "rate": 3.6},
    {"id": 30017, "state": "Montana", "county": "Custer County", "rate": 3.7},
    {"id": 30019, "state": "Montana", "county": "Daniels County", "rate": 2.3},
    {"id": 30021, "state": "Montana", "county": "Dawson County", "rate": 4},
    {
        "id": 30023,
        "state": "Montana",
        "county": "Deer Lodge County",
        "rate": 4.2,
    },
    {"id": 30025, "state": "Montana", "county": "Fallon County", "rate": 3.1},
    {"id": 30027, "state": "Montana", "county": "Fergus County", "rate": 3.3},
    {"id": 30029, "state": "Montana", "county": "Flathead County", "rate": 4.8},
    {"id": 30031, "state": "Montana", "county": "Gallatin County", "rate": 2.3},
    {"id": 30033, "state": "Montana", "county": "Garfield County", "rate": 2.4},
    {"id": 30035, "state": "Montana", "county": "Glacier County", "rate": 9},
    {
        "id": 30037,
        "state": "Montana",
        "county": "Golden Valley County",
        "rate": 3.2,
    },
    {"id": 30039, "state": "Montana", "county": "Granite County", "rate": 5.5},
    {"id": 30041, "state": "Montana", "county": "Hill County", "rate": 4.9},
    {
        "id": 30043,
        "state": "Montana",
        "county": "Jefferson County",
        "rate": 4.5,
    },
    {
        "id": 30045,
        "state": "Montana",
        "county": "Judith Basin County",
        "rate": 2.7,
    },
    {"id": 30047, "state": "Montana", "county": "Lake County", "rate": 4.8},
    {
        "id": 30049,
        "state": "Montana",
        "county": "Lewis and Clark County",
        "rate": 3.3,
    },
    {"id": 30051, "state": "Montana", "county": "Liberty County", "rate": 3.1},
    {"id": 30053, "state": "Montana", "county": "Lincoln County", "rate": 7.9},
    {"id": 30055, "state": "Montana", "county": "McCone County", "rate": 2.1},
    {"id": 30057, "state": "Montana", "county": "Madison County", "rate": 3.1},
    {"id": 30059, "state": "Montana", "county": "Meagher County", "rate": 4.2},
    {"id": 30061, "state": "Montana", "county": "Mineral County", "rate": 7.1},
    {"id": 30063, "state": "Montana", "county": "Missoula County", "rate": 3.5},
    {
        "id": 30065,
        "state": "Montana",
        "county": "Musselshell County",
        "rate": 4.3,
    },
    {"id": 30067, "state": "Montana", "county": "Park County", "rate": 3.3},
    {
        "id": 30069,
        "state": "Montana",
        "county": "Petroleum County",
        "rate": 3.1,
    },
    {"id": 30071, "state": "Montana", "county": "Phillips County", "rate": 4.6},
    {"id": 30073, "state": "Montana", "county": "Pondera County", "rate": 4.7},
    {
        "id": 30075,
        "state": "Montana",
        "county": "Powder River County",
        "rate": 2.5,
    },
    {"id": 30077, "state": "Montana", "county": "Powell County", "rate": 4.5},
    {"id": 30079, "state": "Montana", "county": "Prairie County", "rate": 3},
    {"id": 30081, "state": "Montana", "county": "Ravalli County", "rate": 4.5},
    {"id": 30083, "state": "Montana", "county": "Richland County", "rate": 4.9},
    {
        "id": 30085,
        "state": "Montana",
        "county": "Roosevelt County",
        "rate": 6.2,
    },
    {"id": 30087, "state": "Montana", "county": "Rosebud County", "rate": 6.8},
    {"id": 30089, "state": "Montana", "county": "Sanders County", "rate": 7},
    {"id": 30091, "state": "Montana", "county": "Sheridan County", "rate": 3.2},
    {
        "id": 30093,
        "state": "Montana",
        "county": "Silver Bow County",
        "rate": 4.3,
    },
    {
        "id": 30095,
        "state": "Montana",
        "county": "Stillwater County",
        "rate": 3.9,
    },
    {
        "id": 30097,
        "state": "Montana",
        "county": "Sweet Grass County",
        "rate": 2.9,
    },
    {"id": 30099, "state": "Montana", "county": "Teton County", "rate": 3.5},
    {"id": 30101, "state": "Montana", "county": "Toole County", "rate": 3.3},
    {"id": 30103, "state": "Montana", "county": "Treasure County", "rate": 4.1},
    {"id": 30105, "state": "Montana", "county": "Valley County", "rate": 3},
    {
        "id": 30107,
        "state": "Montana",
        "county": "Wheatland County",
        "rate": 4.9,
    },
    {"id": 30109, "state": "Montana", "county": "Wibaux County", "rate": 4.6},
    {
        "id": 30111,
        "state": "Montana",
        "county": "Yellowstone County",
        "rate": 3.7,
    },
    {"id": 31001, "state": "Nebraska", "county": "Adams County", "rate": 3.3},
    {
        "id": 31003,
        "state": "Nebraska",
        "county": "Antelope County",
        "rate": 2.5,
    },
    {"id": 31005, "state": "Nebraska", "county": "Arthur County", "rate": 5.4},
    {"id": 31007, "state": "Nebraska", "county": "Banner County", "rate": 4.2},
    {"id": 31009, "state": "Nebraska", "county": "Blaine County", "rate": 4.1},
    {"id": 31011, "state": "Nebraska", "county": "Boone County", "rate": 2.8},
    {
        "id": 31013,
        "state": "Nebraska",
        "county": "Box Butte County",
        "rate": 3.6,
    },
    {"id": 31015, "state": "Nebraska", "county": "Boyd County", "rate": 2.8},
    {"id": 31017, "state": "Nebraska", "county": "Brown County", "rate": 3.5},
    {"id": 31019, "state": "Nebraska", "county": "Buffalo County", "rate": 2.8},
    {"id": 31021, "state": "Nebraska", "county": "Burt County", "rate": 3.6},
    {"id": 31023, "state": "Nebraska", "county": "Butler County", "rate": 3.2},
    {"id": 31025, "state": "Nebraska", "county": "Cass County", "rate": 3.5},
    {"id": 31027, "state": "Nebraska", "county": "Cedar County", "rate": 2.8},
    {"id": 31029, "state": "Nebraska", "county": "Chase County", "rate": 2.5},
    {"id": 31031, "state": "Nebraska", "county": "Cherry County", "rate": 2.5},
    {
        "id": 31033,
        "state": "Nebraska",
        "county": "Cheyenne County",
        "rate": 3.3,
    },
    {"id": 31035, "state": "Nebraska", "county": "Clay County", "rate": 3},
    {"id": 31037, "state": "Nebraska", "county": "Colfax County", "rate": 3.2},
    {"id": 31039, "state": "Nebraska", "county": "Cuming County", "rate": 3.2},
    {"id": 31041, "state": "Nebraska", "county": "Custer County", "rate": 2.6},
    {"id": 31043, "state": "Nebraska", "county": "Dakota County", "rate": 4.5},
    {"id": 31045, "state": "Nebraska", "county": "Dawes County", "rate": 3.4},
    {"id": 31047, "state": "Nebraska", "county": "Dawson County", "rate": 3},
    {"id": 31049, "state": "Nebraska", "county": "Deuel County", "rate": 3.4},
    {"id": 31051, "state": "Nebraska", "county": "Dixon County", "rate": 3.4},
    {"id": 31053, "state": "Nebraska", "county": "Dodge County", "rate": 3.3},
    {"id": 31055, "state": "Nebraska", "county": "Douglas County", "rate": 3.5},
    {"id": 31057, "state": "Nebraska", "county": "Dundy County", "rate": 2.5},
    {
        "id": 31059,
        "state": "Nebraska",
        "county": "Fillmore County",
        "rate": 2.8,
    },
    {
        "id": 31061,
        "state": "Nebraska",
        "county": "Franklin County",
        "rate": 3.3,
    },
    {
        "id": 31063,
        "state": "Nebraska",
        "county": "Frontier County",
        "rate": 2.8,
    },
    {"id": 31065, "state": "Nebraska", "county": "Furnas County", "rate": 3.3},
    {"id": 31067, "state": "Nebraska", "county": "Gage County", "rate": 3.5},
    {"id": 31069, "state": "Nebraska", "county": "Garden County", "rate": 3.2},
    {
        "id": 31071,
        "state": "Nebraska",
        "county": "Garfield County",
        "rate": 2.7,
    },
    {"id": 31073, "state": "Nebraska", "county": "Gosper County", "rate": 2.7},
    {"id": 31075, "state": "Nebraska", "county": "Grant County", "rate": 2.6},
    {"id": 31077, "state": "Nebraska", "county": "Greeley County", "rate": 3.7},
    {"id": 31079, "state": "Nebraska", "county": "Hall County", "rate": 3.4},
    {
        "id": 31081,
        "state": "Nebraska",
        "county": "Hamilton County",
        "rate": 2.9,
    },
    {"id": 31083, "state": "Nebraska", "county": "Harlan County", "rate": 2.9},
    {"id": 31085, "state": "Nebraska", "county": "Hayes County", "rate": 2.8},
    {
        "id": 31087,
        "state": "Nebraska",
        "county": "Hitchcock County",
        "rate": 4.1,
    },
    {"id": 31089, "state": "Nebraska", "county": "Holt County", "rate": 3.1},
    {"id": 31091, "state": "Nebraska", "county": "Hooker County", "rate": 2.7},
    {"id": 31093, "state": "Nebraska", "county": "Howard County", "rate": 2.9},
    {
        "id": 31095,
        "state": "Nebraska",
        "county": "Jefferson County",
        "rate": 2.7,
    },
    {"id": 31097, "state": "Nebraska", "county": "Johnson County", "rate": 3.1},
    {"id": 31099, "state": "Nebraska", "county": "Kearney County", "rate": 2.4},
    {"id": 31101, "state": "Nebraska", "county": "Keith County", "rate": 3},
    {
        "id": 31103,
        "state": "Nebraska",
        "county": "Keya Paha County",
        "rate": 2.3,
    },
    {"id": 31105, "state": "Nebraska", "county": "Kimball County", "rate": 4.3},
    {"id": 31107, "state": "Nebraska", "county": "Knox County", "rate": 2.8},
    {"id": 31109, "state": "Nebraska", "county": "Lancaster County", "rate": 3},
    {"id": 31111, "state": "Nebraska", "county": "Lincoln County", "rate": 3.2},
    {"id": 31113, "state": "Nebraska", "county": "Logan County", "rate": 3},
    {"id": 31115, "state": "Nebraska", "county": "Loup County", "rate": 3.7},
    {
        "id": 31117,
        "state": "Nebraska",
        "county": "McPherson County",
        "rate": 2.5,
    },
    {"id": 31119, "state": "Nebraska", "county": "Madison County", "rate": 3},
    {"id": 31121, "state": "Nebraska", "county": "Merrick County", "rate": 3.7},
    {"id": 31123, "state": "Nebraska", "county": "Morrill County", "rate": 3.5},
    {"id": 31125, "state": "Nebraska", "county": "Nance County", "rate": 2.9},
    {"id": 31127, "state": "Nebraska", "county": "Nemaha County", "rate": 4.3},
    {"id": 31129, "state": "Nebraska", "county": "Nuckolls County", "rate": 3},
    {"id": 31131, "state": "Nebraska", "county": "Otoe County", "rate": 3.6},
    {"id": 31133, "state": "Nebraska", "county": "Pawnee County", "rate": 2.4},
    {"id": 31135, "state": "Nebraska", "county": "Perkins County", "rate": 2.5},
    {"id": 31137, "state": "Nebraska", "county": "Phelps County", "rate": 2.8},
    {"id": 31139, "state": "Nebraska", "county": "Pierce County", "rate": 2.5},
    {"id": 31141, "state": "Nebraska", "county": "Platte County", "rate": 3.5},
    {"id": 31143, "state": "Nebraska", "county": "Polk County", "rate": 2.7},
    {
        "id": 31145,
        "state": "Nebraska",
        "county": "Red Willow County",
        "rate": 3.2,
    },
    {
        "id": 31147,
        "state": "Nebraska",
        "county": "Richardson County",
        "rate": 3.3,
    },
    {"id": 31149, "state": "Nebraska", "county": "Rock County", "rate": 2.6},
    {"id": 31151, "state": "Nebraska", "county": "Saline County", "rate": 3.1},
    {"id": 31153, "state": "Nebraska", "county": "Sarpy County", "rate": 3.1},
    {
        "id": 31155,
        "state": "Nebraska",
        "county": "Saunders County",
        "rate": 3.2,
    },
    {
        "id": 31157,
        "state": "Nebraska",
        "county": "Scotts Bluff County",
        "rate": 3.8,
    },
    {"id": 31159, "state": "Nebraska", "county": "Seward County", "rate": 3.1},
    {
        "id": 31161,
        "state": "Nebraska",
        "county": "Sheridan County",
        "rate": 3.4,
    },
    {"id": 31163, "state": "Nebraska", "county": "Sherman County", "rate": 3.3},
    {"id": 31165, "state": "Nebraska", "county": "Sioux County", "rate": 2.6},
    {"id": 31167, "state": "Nebraska", "county": "Stanton County", "rate": 2.7},
    {"id": 31169, "state": "Nebraska", "county": "Thayer County", "rate": 2.8},
    {"id": 31171, "state": "Nebraska", "county": "Thomas County", "rate": 3.2},
    {
        "id": 31173,
        "state": "Nebraska",
        "county": "Thurston County",
        "rate": 4.6,
    },
    {"id": 31175, "state": "Nebraska", "county": "Valley County", "rate": 2.9},
    {
        "id": 31177,
        "state": "Nebraska",
        "county": "Washington County",
        "rate": 3.2,
    },
    {"id": 31179, "state": "Nebraska", "county": "Wayne County", "rate": 3.4},
    {"id": 31181, "state": "Nebraska", "county": "Webster County", "rate": 3.7},
    {"id": 31183, "state": "Nebraska", "county": "Wheeler County", "rate": 2.7},
    {"id": 31185, "state": "Nebraska", "county": "York County", "rate": 2.9},
    {"id": 32001, "state": "Nevada", "county": "Churchill County", "rate": 5.8},
    {"id": 32003, "state": "Nevada", "county": "Clark County", "rate": 6},
    {"id": 32005, "state": "Nevada", "county": "Douglas County", "rate": 5.3},
    {"id": 32007, "state": "Nevada", "county": "Elko County", "rate": 4.4},
    {"id": 32009, "state": "Nevada", "county": "Esmeralda County", "rate": 4.7},
    {"id": 32011, "state": "Nevada", "county": "Eureka County", "rate": 5.1},
    {"id": 32013, "state": "Nevada", "county": "Humboldt County", "rate": 5.5},
    {"id": 32015, "state": "Nevada", "county": "Lander County", "rate": 5.9},
    {"id": 32017, "state": "Nevada", "county": "Lincoln County", "rate": 5.3},
    {"id": 32019, "state": "Nevada", "county": "Lyon County", "rate": 7.3},
    {"id": 32021, "state": "Nevada", "county": "Mineral County", "rate": 8.6},
    {"id": 32023, "state": "Nevada", "county": "Nye County", "rate": 7.5},
    {"id": 32027, "state": "Nevada", "county": "Pershing County", "rate": 6},
    {"id": 32029, "state": "Nevada", "county": "Storey County", "rate": 5.8},
    {"id": 32031, "state": "Nevada", "county": "Washoe County", "rate": 4.9},
    {
        "id": 32033,
        "state": "Nevada",
        "county": "White Pine County",
        "rate": 4.6,
    },
    {"id": 32510, "state": "Nevada", "county": "Carson City", "rate": 6},
    {
        "id": 33001,
        "state": "New Hampshire",
        "county": "Belknap County",
        "rate": 2.6,
    },
    {
        "id": 33003,
        "state": "New Hampshire",
        "county": "Carroll County",
        "rate": 2.8,
    },
    {
        "id": 33005,
        "state": "New Hampshire",
        "county": "Cheshire County",
        "rate": 2.9,
    },
    {
        "id": 33007,
        "state": "New Hampshire",
        "county": "Coos County",
        "rate": 3.5,
    },
    {
        "id": 33009,
        "state": "New Hampshire",
        "county": "Grafton County",
        "rate": 2.4,
    },
    {
        "id": 33011,
        "state": "New Hampshire",
        "county": "Hillsborough County",
        "rate": 3.2,
    },
    {
        "id": 33013,
        "state": "New Hampshire",
        "county": "Merrimack County",
        "rate": 2.6,
    },
    {
        "id": 33015,
        "state": "New Hampshire",
        "county": "Rockingham County",
        "rate": 3.1,
    },
    {
        "id": 33017,
        "state": "New Hampshire",
        "county": "Strafford County",
        "rate": 2.7,
    },
    {
        "id": 33019,
        "state": "New Hampshire",
        "county": "Sullivan County",
        "rate": 2.5,
    },
    {
        "id": 34001,
        "state": "New Jersey",
        "county": "Atlantic County",
        "rate": 7.1,
    },
    {
        "id": 34003,
        "state": "New Jersey",
        "county": "Bergen County",
        "rate": 4.7,
    },
    {
        "id": 34005,
        "state": "New Jersey",
        "county": "Burlington County",
        "rate": 5,
    },
    {"id": 34007, "state": "New Jersey", "county": "Camden County", "rate": 6},
    {
        "id": 34009,
        "state": "New Jersey",
        "county": "Cape May County",
        "rate": 5.7,
    },
    {
        "id": 34011,
        "state": "New Jersey",
        "county": "Cumberland County",
        "rate": 7.9,
    },
    {"id": 34013, "state": "New Jersey", "county": "Essex County", "rate": 6.6},
    {
        "id": 34015,
        "state": "New Jersey",
        "county": "Gloucester County",
        "rate": 5.6,
    },
    {
        "id": 34017,
        "state": "New Jersey",
        "county": "Hudson County",
        "rate": 5.2,
    },
    {
        "id": 34019,
        "state": "New Jersey",
        "county": "Hunterdon County",
        "rate": 4.1,
    },
    {
        "id": 34021,
        "state": "New Jersey",
        "county": "Mercer County",
        "rate": 4.9,
    },
    {
        "id": 34023,
        "state": "New Jersey",
        "county": "Middlesex County",
        "rate": 4.9,
    },
    {
        "id": 34025,
        "state": "New Jersey",
        "county": "Monmouth County",
        "rate": 4.8,
    },
    {
        "id": 34027,
        "state": "New Jersey",
        "county": "Morris County",
        "rate": 4.2,
    },
    {"id": 34029, "state": "New Jersey", "county": "Ocean County", "rate": 5.4},
    {
        "id": 34031,
        "state": "New Jersey",
        "county": "Passaic County",
        "rate": 6.8,
    },
    {"id": 34033, "state": "New Jersey", "county": "Salem County", "rate": 6.8},
    {
        "id": 34035,
        "state": "New Jersey",
        "county": "Somerset County",
        "rate": 4.4,
    },
    {"id": 34037, "state": "New Jersey", "county": "Sussex County", "rate": 5},
    {"id": 34039, "state": "New Jersey", "county": "Union County", "rate": 5.6},
    {
        "id": 34041,
        "state": "New Jersey",
        "county": "Warren County",
        "rate": 5.1,
    },
    {
        "id": 35001,
        "state": "New Mexico",
        "county": "Bernalillo County",
        "rate": 6.2,
    },
    {
        "id": 35003,
        "state": "New Mexico",
        "county": "Catron County",
        "rate": 6.9,
    },
    {
        "id": 35005,
        "state": "New Mexico",
        "county": "Chaves County",
        "rate": 6.9,
    },
    {
        "id": 35006,
        "state": "New Mexico",
        "county": "Cibola County",
        "rate": 9.1,
    },
    {
        "id": 35007,
        "state": "New Mexico",
        "county": "Colfax County",
        "rate": 5.6,
    },
    {"id": 35009, "state": "New Mexico", "county": "Curry County", "rate": 5.4},
    {"id": 35011, "state": "New Mexico", "county": "De Baca County", "rate": 5},
    {
        "id": 35013,
        "state": "New Mexico",
        "county": "Dona Ana County",
        "rate": 7.1,
    },
    {"id": 35015, "state": "New Mexico", "county": "Eddy County", "rate": 7.2},
    {"id": 35017, "state": "New Mexico", "county": "Grant County", "rate": 6.6},
    {
        "id": 35019,
        "state": "New Mexico",
        "county": "Guadalupe County",
        "rate": 6.4,
    },
    {
        "id": 35021,
        "state": "New Mexico",
        "county": "Harding County",
        "rate": 7.3,
    },
    {
        "id": 35023,
        "state": "New Mexico",
        "county": "Hidalgo County",
        "rate": 5.4,
    },
    {"id": 35025, "state": "New Mexico", "county": "Lea County", "rate": 9.8},
    {"id": 35027, "state": "New Mexico", "county": "Lincoln County", "rate": 6},
    {
        "id": 35028,
        "state": "New Mexico",
        "county": "Los Alamos County",
        "rate": 4.5,
    },
    {"id": 35029, "state": "New Mexico", "county": "Luna County", "rate": 10},
    {
        "id": 35031,
        "state": "New Mexico",
        "county": "McKinley County",
        "rate": 9.8,
    },
    {"id": 35033, "state": "New Mexico", "county": "Mora County", "rate": 8},
    {"id": 35035, "state": "New Mexico", "county": "Otero County", "rate": 6.4},
    {"id": 35037, "state": "New Mexico", "county": "Quay County", "rate": 7},
    {
        "id": 35039,
        "state": "New Mexico",
        "county": "Rio Arriba County",
        "rate": 7.3,
    },
    {
        "id": 35041,
        "state": "New Mexico",
        "county": "Roosevelt County",
        "rate": 6.1,
    },
    {
        "id": 35043,
        "state": "New Mexico",
        "county": "Sandoval County",
        "rate": 7.1,
    },
    {
        "id": 35045,
        "state": "New Mexico",
        "county": "San Juan County",
        "rate": 9.4,
    },
    {
        "id": 35047,
        "state": "New Mexico",
        "county": "San Miguel County",
        "rate": 8.1,
    },
    {
        "id": 35049,
        "state": "New Mexico",
        "county": "Santa Fe County",
        "rate": 5.5,
    },
    {
        "id": 35051,
        "state": "New Mexico",
        "county": "Sierra County",
        "rate": 7.8,
    },
    {"id": 35053, "state": "New Mexico", "county": "Socorro County", "rate": 8},
    {"id": 35055, "state": "New Mexico", "county": "Taos County", "rate": 8.3},
    {
        "id": 35057,
        "state": "New Mexico",
        "county": "Torrance County",
        "rate": 9.3,
    },
    {"id": 35059, "state": "New Mexico", "county": "Union County", "rate": 4.5},
    {
        "id": 35061,
        "state": "New Mexico",
        "county": "Valencia County",
        "rate": 7.6,
    },
    {"id": 36001, "state": "New York", "county": "Albany County", "rate": 4},
    {
        "id": 36003,
        "state": "New York",
        "county": "Allegany County",
        "rate": 4.9,
    },
    {"id": 36005, "state": "New York", "county": "Bronx County", "rate": 7.8},
    {"id": 36007, "state": "New York", "county": "Broome County", "rate": 5},
    {
        "id": 36009,
        "state": "New York",
        "county": "Cattaraugus County",
        "rate": 5.2,
    },
    {"id": 36011, "state": "New York", "county": "Cayuga County", "rate": 4.4},
    {
        "id": 36013,
        "state": "New York",
        "county": "Chautauqua County",
        "rate": 5.1,
    },
    {"id": 36015, "state": "New York", "county": "Chemung County", "rate": 5.2},
    {
        "id": 36017,
        "state": "New York",
        "county": "Chenango County",
        "rate": 4.4,
    },
    {"id": 36019, "state": "New York", "county": "Clinton County", "rate": 5.1},
    {
        "id": 36021,
        "state": "New York",
        "county": "Columbia County",
        "rate": 3.3,
    },
    {
        "id": 36023,
        "state": "New York",
        "county": "Cortland County",
        "rate": 4.6,
    },
    {
        "id": 36025,
        "state": "New York",
        "county": "Delaware County",
        "rate": 4.9,
    },
    {"id": 36027, "state": "New York", "county": "Dutchess County", "rate": 4},
    {"id": 36029, "state": "New York", "county": "Erie County", "rate": 4.6},
    {"id": 36031, "state": "New York", "county": "Essex County", "rate": 4.1},
    {
        "id": 36033,
        "state": "New York",
        "county": "Franklin County",
        "rate": 4.8,
    },
    {"id": 36035, "state": "New York", "county": "Fulton County", "rate": 5.2},
    {"id": 36037, "state": "New York", "county": "Genesee County", "rate": 3.8},
    {"id": 36039, "state": "New York", "county": "Greene County", "rate": 4.5},
    {
        "id": 36041,
        "state": "New York",
        "county": "Hamilton County",
        "rate": 2.9,
    },
    {
        "id": 36043,
        "state": "New York",
        "county": "Herkimer County",
        "rate": 4.4,
    },
    {
        "id": 36045,
        "state": "New York",
        "county": "Jefferson County",
        "rate": 5.1,
    },
    {"id": 36047, "state": "New York", "county": "Kings County", "rate": 5.9},
    {"id": 36049, "state": "New York", "county": "Lewis County", "rate": 5.4},
    {
        "id": 36051,
        "state": "New York",
        "county": "Livingston County",
        "rate": 4.2,
    },
    {"id": 36053, "state": "New York", "county": "Madison County", "rate": 4.6},
    {"id": 36055, "state": "New York", "county": "Monroe County", "rate": 4.6},
    {
        "id": 36057,
        "state": "New York",
        "county": "Montgomery County",
        "rate": 5.2,
    },
    {"id": 36059, "state": "New York", "county": "Nassau County", "rate": 3.8},
    {
        "id": 36061,
        "state": "New York",
        "county": "New York County",
        "rate": 4.9,
    },
    {"id": 36063, "state": "New York", "county": "Niagara County", "rate": 5.2},
    {"id": 36065, "state": "New York", "county": "Oneida County", "rate": 4.3},
    {
        "id": 36067,
        "state": "New York",
        "county": "Onondaga County",
        "rate": 4.2,
    },
    {"id": 36069, "state": "New York", "county": "Ontario County", "rate": 3.7},
    {"id": 36071, "state": "New York", "county": "Orange County", "rate": 4.2},
    {"id": 36073, "state": "New York", "county": "Orleans County", "rate": 5.2},
    {"id": 36075, "state": "New York", "county": "Oswego County", "rate": 5.7},
    {"id": 36077, "state": "New York", "county": "Otsego County", "rate": 4.3},
    {"id": 36079, "state": "New York", "county": "Putnam County", "rate": 3.9},
    {"id": 36081, "state": "New York", "county": "Queens County", "rate": 5},
    {
        "id": 36083,
        "state": "New York",
        "county": "Rensselaer County",
        "rate": 4.1,
    },
    {
        "id": 36085,
        "state": "New York",
        "county": "Richmond County",
        "rate": 5.9,
    },
    {
        "id": 36087,
        "state": "New York",
        "county": "Rockland County",
        "rate": 4.1,
    },
    {
        "id": 36089,
        "state": "New York",
        "county": "St. Lawrence County",
        "rate": 5.8,
    },
    {
        "id": 36091,
        "state": "New York",
        "county": "Saratoga County",
        "rate": 3.5,
    },
    {
        "id": 36093,
        "state": "New York",
        "county": "Schenectady County",
        "rate": 4.2,
    },
    {
        "id": 36095,
        "state": "New York",
        "county": "Schoharie County",
        "rate": 4.7,
    },
    {
        "id": 36097,
        "state": "New York",
        "county": "Schuyler County",
        "rate": 4.6,
    },
    {"id": 36099, "state": "New York", "county": "Seneca County", "rate": 3.8},
    {"id": 36101, "state": "New York", "county": "Steuben County", "rate": 5},
    {"id": 36103, "state": "New York", "county": "Suffolk County", "rate": 4.3},
    {
        "id": 36105,
        "state": "New York",
        "county": "Sullivan County",
        "rate": 4.1,
    },
    {"id": 36107, "state": "New York", "county": "Tioga County", "rate": 4.5},
    {
        "id": 36109,
        "state": "New York",
        "county": "Tompkins County",
        "rate": 3.7,
    },
    {"id": 36111, "state": "New York", "county": "Ulster County", "rate": 4.3},
    {"id": 36113, "state": "New York", "county": "Warren County", "rate": 4.1},
    {
        "id": 36115,
        "state": "New York",
        "county": "Washington County",
        "rate": 3.8,
    },
    {"id": 36117, "state": "New York", "county": "Wayne County", "rate": 4.5},
    {
        "id": 36119,
        "state": "New York",
        "county": "Westchester County",
        "rate": 4.2,
    },
    {"id": 36121, "state": "New York", "county": "Wyoming County", "rate": 4.1},
    {"id": 36123, "state": "New York", "county": "Yates County", "rate": 3.7},
    {
        "id": 37001,
        "state": "North Carolina",
        "county": "Alamance County",
        "rate": 4.8,
    },
    {
        "id": 37003,
        "state": "North Carolina",
        "county": "Alexander County",
        "rate": 4.5,
    },
    {
        "id": 37005,
        "state": "North Carolina",
        "county": "Alleghany County",
        "rate": 5.1,
    },
    {
        "id": 37007,
        "state": "North Carolina",
        "county": "Anson County",
        "rate": 5.9,
    },
    {
        "id": 37009,
        "state": "North Carolina",
        "county": "Ashe County",
        "rate": 4.5,
    },
    {
        "id": 37011,
        "state": "North Carolina",
        "county": "Avery County",
        "rate": 4.8,
    },
    {
        "id": 37013,
        "state": "North Carolina",
        "county": "Beaufort County",
        "rate": 5.6,
    },
    {
        "id": 37015,
        "state": "North Carolina",
        "county": "Bertie County",
        "rate": 6.4,
    },
    {
        "id": 37017,
        "state": "North Carolina",
        "county": "Bladen County",
        "rate": 6.7,
    },
    {
        "id": 37019,
        "state": "North Carolina",
        "county": "Brunswick County",
        "rate": 5.7,
    },
    {
        "id": 37021,
        "state": "North Carolina",
        "county": "Buncombe County",
        "rate": 3.9,
    },
    {
        "id": 37023,
        "state": "North Carolina",
        "county": "Burke County",
        "rate": 5.1,
    },
    {
        "id": 37025,
        "state": "North Carolina",
        "county": "Cabarrus County",
        "rate": 4.6,
    },
    {
        "id": 37027,
        "state": "North Carolina",
        "county": "Caldwell County",
        "rate": 5.3,
    },
    {
        "id": 37029,
        "state": "North Carolina",
        "county": "Camden County",
        "rate": 5.3,
    },
    {
        "id": 37031,
        "state": "North Carolina",
        "county": "Carteret County",
        "rate": 4.7,
    },
    {
        "id": 37033,
        "state": "North Carolina",
        "county": "Caswell County",
        "rate": 5.5,
    },
    {
        "id": 37035,
        "state": "North Carolina",
        "county": "Catawba County",
        "rate": 4.9,
    },
    {
        "id": 37037,
        "state": "North Carolina",
        "county": "Chatham County",
        "rate": 4.3,
    },
    {
        "id": 37039,
        "state": "North Carolina",
        "county": "Cherokee County",
        "rate": 5.4,
    },
    {
        "id": 37041,
        "state": "North Carolina",
        "county": "Chowan County",
        "rate": 6.5,
    },
    {
        "id": 37043,
        "state": "North Carolina",
        "county": "Clay County",
        "rate": 5.1,
    },
    {
        "id": 37045,
        "state": "North Carolina",
        "county": "Cleveland County",
        "rate": 5.6,
    },
    {
        "id": 37047,
        "state": "North Carolina",
        "county": "Columbus County",
        "rate": 6.1,
    },
    {
        "id": 37049,
        "state": "North Carolina",
        "county": "Craven County",
        "rate": 5.1,
    },
    {
        "id": 37051,
        "state": "North Carolina",
        "county": "Cumberland County",
        "rate": 6.3,
    },
    {
        "id": 37053,
        "state": "North Carolina",
        "county": "Currituck County",
        "rate": 4.6,
    },
    {
        "id": 37055,
        "state": "North Carolina",
        "county": "Dare County",
        "rate": 4.2,
    },
    {
        "id": 37057,
        "state": "North Carolina",
        "county": "Davidson County",
        "rate": 4.9,
    },
    {
        "id": 37059,
        "state": "North Carolina",
        "county": "Davie County",
        "rate": 4.6,
    },
    {
        "id": 37061,
        "state": "North Carolina",
        "county": "Duplin County",
        "rate": 5.4,
    },
    {
        "id": 37063,
        "state": "North Carolina",
        "county": "Durham County",
        "rate": 4.5,
    },
    {
        "id": 37065,
        "state": "North Carolina",
        "county": "Edgecombe County",
        "rate": 8.5,
    },
    {
        "id": 37067,
        "state": "North Carolina",
        "county": "Forsyth County",
        "rate": 4.9,
    },
    {
        "id": 37069,
        "state": "North Carolina",
        "county": "Franklin County",
        "rate": 5.1,
    },
    {
        "id": 37071,
        "state": "North Carolina",
        "county": "Gaston County",
        "rate": 5.3,
    },
    {
        "id": 37073,
        "state": "North Carolina",
        "county": "Gates County",
        "rate": 5.2,
    },
    {
        "id": 37075,
        "state": "North Carolina",
        "county": "Graham County",
        "rate": 7.1,
    },
    {
        "id": 37077,
        "state": "North Carolina",
        "county": "Granville County",
        "rate": 4.4,
    },
    {
        "id": 37079,
        "state": "North Carolina",
        "county": "Greene County",
        "rate": 5,
    },
    {
        "id": 37081,
        "state": "North Carolina",
        "county": "Guilford County",
        "rate": 5.3,
    },
    {
        "id": 37083,
        "state": "North Carolina",
        "county": "Halifax County",
        "rate": 7.8,
    },
    {
        "id": 37085,
        "state": "North Carolina",
        "county": "Harnett County",
        "rate": 5.8,
    },
    {
        "id": 37087,
        "state": "North Carolina",
        "county": "Haywood County",
        "rate": 4.4,
    },
    {
        "id": 37089,
        "state": "North Carolina",
        "county": "Henderson County",
        "rate": 4.3,
    },
    {
        "id": 37091,
        "state": "North Carolina",
        "county": "Hertford County",
        "rate": 6.5,
    },
    {
        "id": 37093,
        "state": "North Carolina",
        "county": "Hoke County",
        "rate": 6.5,
    },
    {
        "id": 37095,
        "state": "North Carolina",
        "county": "Hyde County",
        "rate": 6.4,
    },
    {
        "id": 37097,
        "state": "North Carolina",
        "county": "Iredell County",
        "rate": 4.7,
    },
    {
        "id": 37099,
        "state": "North Carolina",
        "county": "Jackson County",
        "rate": 5.1,
    },
    {
        "id": 37101,
        "state": "North Carolina",
        "county": "Johnston County",
        "rate": 4.5,
    },
    {
        "id": 37103,
        "state": "North Carolina",
        "county": "Jones County",
        "rate": 5.1,
    },
    {
        "id": 37105,
        "state": "North Carolina",
        "county": "Lee County",
        "rate": 5.7,
    },
    {
        "id": 37107,
        "state": "North Carolina",
        "county": "Lenoir County",
        "rate": 5.4,
    },
    {
        "id": 37109,
        "state": "North Carolina",
        "county": "Lincoln County",
        "rate": 4.6,
    },
    {
        "id": 37111,
        "state": "North Carolina",
        "county": "McDowell County",
        "rate": 4.6,
    },
    {
        "id": 37113,
        "state": "North Carolina",
        "county": "Macon County",
        "rate": 5.1,
    },
    {
        "id": 37115,
        "state": "North Carolina",
        "county": "Madison County",
        "rate": 4.9,
    },
    {
        "id": 37117,
        "state": "North Carolina",
        "county": "Martin County",
        "rate": 6.8,
    },
    {
        "id": 37119,
        "state": "North Carolina",
        "county": "Mecklenburg County",
        "rate": 4.7,
    },
    {
        "id": 37121,
        "state": "North Carolina",
        "county": "Mitchell County",
        "rate": 6,
    },
    {
        "id": 37123,
        "state": "North Carolina",
        "county": "Montgomery County",
        "rate": 5.2,
    },
    {
        "id": 37125,
        "state": "North Carolina",
        "county": "Moore County",
        "rate": 5,
    },
    {
        "id": 37127,
        "state": "North Carolina",
        "county": "Nash County",
        "rate": 6.8,
    },
    {
        "id": 37129,
        "state": "North Carolina",
        "county": "New Hanover County",
        "rate": 4.7,
    },
    {
        "id": 37131,
        "state": "North Carolina",
        "county": "Northampton County",
        "rate": 7.3,
    },
    {
        "id": 37133,
        "state": "North Carolina",
        "county": "Onslow County",
        "rate": 5.4,
    },
    {
        "id": 37135,
        "state": "North Carolina",
        "county": "Orange County",
        "rate": 4.5,
    },
    {
        "id": 37137,
        "state": "North Carolina",
        "county": "Pamlico County",
        "rate": 5.1,
    },
    {
        "id": 37139,
        "state": "North Carolina",
        "county": "Pasquotank County",
        "rate": 6.2,
    },
    {
        "id": 37141,
        "state": "North Carolina",
        "county": "Pender County",
        "rate": 5.2,
    },
    {
        "id": 37143,
        "state": "North Carolina",
        "county": "Perquimans County",
        "rate": 6.3,
    },
    {
        "id": 37145,
        "state": "North Carolina",
        "county": "Person County",
        "rate": 5.4,
    },
    {
        "id": 37147,
        "state": "North Carolina",
        "county": "Pitt County",
        "rate": 5.7,
    },
    {
        "id": 37149,
        "state": "North Carolina",
        "county": "Polk County",
        "rate": 4.9,
    },
    {
        "id": 37151,
        "state": "North Carolina",
        "county": "Randolph County",
        "rate": 4.8,
    },
    {
        "id": 37153,
        "state": "North Carolina",
        "county": "Richmond County",
        "rate": 6.9,
    },
    {
        "id": 37155,
        "state": "North Carolina",
        "county": "Robeson County",
        "rate": 7.3,
    },
    {
        "id": 37157,
        "state": "North Carolina",
        "county": "Rockingham County",
        "rate": 5.5,
    },
    {
        "id": 37159,
        "state": "North Carolina",
        "county": "Rowan County",
        "rate": 5.5,
    },
    {
        "id": 37161,
        "state": "North Carolina",
        "county": "Rutherford County",
        "rate": 6.6,
    },
    {
        "id": 37163,
        "state": "North Carolina",
        "county": "Sampson County",
        "rate": 5.6,
    },
    {
        "id": 37165,
        "state": "North Carolina",
        "county": "Scotland County",
        "rate": 8.9,
    },
    {
        "id": 37167,
        "state": "North Carolina",
        "county": "Stanly County",
        "rate": 4.7,
    },
    {
        "id": 37169,
        "state": "North Carolina",
        "county": "Stokes County",
        "rate": 4.9,
    },
    {
        "id": 37171,
        "state": "North Carolina",
        "county": "Surry County",
        "rate": 4.8,
    },
    {
        "id": 37173,
        "state": "North Carolina",
        "county": "Swain County",
        "rate": 4.9,
    },
    {
        "id": 37175,
        "state": "North Carolina",
        "county": "Transylvania County",
        "rate": 4.8,
    },
    {
        "id": 37177,
        "state": "North Carolina",
        "county": "Tyrrell County",
        "rate": 6.2,
    },
    {
        "id": 37179,
        "state": "North Carolina",
        "county": "Union County",
        "rate": 4.4,
    },
    {
        "id": 37181,
        "state": "North Carolina",
        "county": "Vance County",
        "rate": 7.3,
    },
    {
        "id": 37183,
        "state": "North Carolina",
        "county": "Wake County",
        "rate": 4.2,
    },
    {
        "id": 37185,
        "state": "North Carolina",
        "county": "Warren County",
        "rate": 7,
    },
    {
        "id": 37187,
        "state": "North Carolina",
        "county": "Washington County",
        "rate": 6.9,
    },
    {
        "id": 37189,
        "state": "North Carolina",
        "county": "Watauga County",
        "rate": 4.7,
    },
    {
        "id": 37191,
        "state": "North Carolina",
        "county": "Wayne County",
        "rate": 5.6,
    },
    {
        "id": 37193,
        "state": "North Carolina",
        "county": "Wilkes County",
        "rate": 4.7,
    },
    {
        "id": 37195,
        "state": "North Carolina",
        "county": "Wilson County",
        "rate": 8.1,
    },
    {
        "id": 37197,
        "state": "North Carolina",
        "county": "Yadkin County",
        "rate": 4.4,
    },
    {
        "id": 37199,
        "state": "North Carolina",
        "county": "Yancey County",
        "rate": 5.1,
    },
    {
        "id": 38001,
        "state": "North Dakota",
        "county": "Adams County",
        "rate": 2.3,
    },
    {
        "id": 38003,
        "state": "North Dakota",
        "county": "Barnes County",
        "rate": 3.2,
    },
    {
        "id": 38005,
        "state": "North Dakota",
        "county": "Benson County",
        "rate": 3.1,
    },
    {
        "id": 38007,
        "state": "North Dakota",
        "county": "Billings County",
        "rate": 2.5,
    },
    {
        "id": 38009,
        "state": "North Dakota",
        "county": "Bottineau County",
        "rate": 3.2,
    },
    {
        "id": 38011,
        "state": "North Dakota",
        "county": "Bowman County",
        "rate": 1.8,
    },
    {
        "id": 38013,
        "state": "North Dakota",
        "county": "Burke County",
        "rate": 3.8,
    },
    {
        "id": 38015,
        "state": "North Dakota",
        "county": "Burleigh County",
        "rate": 2.3,
    },
    {"id": 38017, "state": "North Dakota", "county": "Cass County", "rate": 2},
    {
        "id": 38019,
        "state": "North Dakota",
        "county": "Cavalier County",
        "rate": 1.9,
    },
    {
        "id": 38021,
        "state": "North Dakota",
        "county": "Dickey County",
        "rate": 1.8,
    },
    {
        "id": 38023,
        "state": "North Dakota",
        "county": "Divide County",
        "rate": 2,
    },
    {
        "id": 38025,
        "state": "North Dakota",
        "county": "Dunn County",
        "rate": 2.2,
    },
    {
        "id": 38027,
        "state": "North Dakota",
        "county": "Eddy County",
        "rate": 2.8,
    },
    {
        "id": 38029,
        "state": "North Dakota",
        "county": "Emmons County",
        "rate": 3.1,
    },
    {
        "id": 38031,
        "state": "North Dakota",
        "county": "Foster County",
        "rate": 2.5,
    },
    {
        "id": 38033,
        "state": "North Dakota",
        "county": "Golden Valley County",
        "rate": 2.4,
    },
    {
        "id": 38035,
        "state": "North Dakota",
        "county": "Grand Forks County",
        "rate": 2.3,
    },
    {
        "id": 38037,
        "state": "North Dakota",
        "county": "Grant County",
        "rate": 2.1,
    },
    {
        "id": 38039,
        "state": "North Dakota",
        "county": "Griggs County",
        "rate": 2.3,
    },
    {
        "id": 38041,
        "state": "North Dakota",
        "county": "Hettinger County",
        "rate": 2.9,
    },
    {
        "id": 38043,
        "state": "North Dakota",
        "county": "Kidder County",
        "rate": 2.6,
    },
    {
        "id": 38045,
        "state": "North Dakota",
        "county": "LaMoure County",
        "rate": 2,
    },
    {
        "id": 38047,
        "state": "North Dakota",
        "county": "Logan County",
        "rate": 2.9,
    },
    {
        "id": 38049,
        "state": "North Dakota",
        "county": "McHenry County",
        "rate": 4.1,
    },
    {
        "id": 38051,
        "state": "North Dakota",
        "county": "McIntosh County",
        "rate": 2.4,
    },
    {
        "id": 38053,
        "state": "North Dakota",
        "county": "McKenzie County",
        "rate": 3.1,
    },
    {
        "id": 38055,
        "state": "North Dakota",
        "county": "McLean County",
        "rate": 3.1,
    },
    {
        "id": 38057,
        "state": "North Dakota",
        "county": "Mercer County",
        "rate": 4.5,
    },
    {
        "id": 38059,
        "state": "North Dakota",
        "county": "Morton County",
        "rate": 2.9,
    },
    {
        "id": 38061,
        "state": "North Dakota",
        "county": "Mountrail County",
        "rate": 2.9,
    },
    {
        "id": 38063,
        "state": "North Dakota",
        "county": "Nelson County",
        "rate": 2.9,
    },
    {
        "id": 38065,
        "state": "North Dakota",
        "county": "Oliver County",
        "rate": 5.3,
    },
    {
        "id": 38067,
        "state": "North Dakota",
        "county": "Pembina County",
        "rate": 4.5,
    },
    {
        "id": 38069,
        "state": "North Dakota",
        "county": "Pierce County",
        "rate": 3,
    },
    {
        "id": 38071,
        "state": "North Dakota",
        "county": "Ramsey County",
        "rate": 2.5,
    },
    {
        "id": 38073,
        "state": "North Dakota",
        "county": "Ransom County",
        "rate": 1.9,
    },
    {
        "id": 38075,
        "state": "North Dakota",
        "county": "Renville County",
        "rate": 3.5,
    },
    {
        "id": 38077,
        "state": "North Dakota",
        "county": "Richland County",
        "rate": 2.7,
    },
    {
        "id": 38079,
        "state": "North Dakota",
        "county": "Rolette County",
        "rate": 14.2,
    },
    {
        "id": 38081,
        "state": "North Dakota",
        "county": "Sargent County",
        "rate": 1.6,
    },
    {
        "id": 38083,
        "state": "North Dakota",
        "county": "Sheridan County",
        "rate": 3.7,
    },
    {
        "id": 38085,
        "state": "North Dakota",
        "county": "Sioux County",
        "rate": 5.3,
    },
    {
        "id": 38087,
        "state": "North Dakota",
        "county": "Slope County",
        "rate": 2.3,
    },
    {
        "id": 38089,
        "state": "North Dakota",
        "county": "Stark County",
        "rate": 3.3,
    },
    {
        "id": 38091,
        "state": "North Dakota",
        "county": "Steele County",
        "rate": 1.7,
    },
    {
        "id": 38093,
        "state": "North Dakota",
        "county": "Stutsman County",
        "rate": 2.1,
    },
    {
        "id": 38095,
        "state": "North Dakota",
        "county": "Towner County",
        "rate": 2,
    },
    {
        "id": 38097,
        "state": "North Dakota",
        "county": "Traill County",
        "rate": 2.7,
    },
    {
        "id": 38099,
        "state": "North Dakota",
        "county": "Walsh County",
        "rate": 3.6,
    },
    {
        "id": 38101,
        "state": "North Dakota",
        "county": "Ward County",
        "rate": 3.5,
    },
    {
        "id": 38103,
        "state": "North Dakota",
        "county": "Wells County",
        "rate": 2.7,
    },
    {
        "id": 38105,
        "state": "North Dakota",
        "county": "Williams County",
        "rate": 3.7,
    },
    {"id": 39001, "state": "Ohio", "county": "Adams County", "rate": 6.8},
    {"id": 39003, "state": "Ohio", "county": "Allen County", "rate": 4.6},
    {"id": 39005, "state": "Ohio", "county": "Ashland County", "rate": 4.3},
    {"id": 39007, "state": "Ohio", "county": "Ashtabula County", "rate": 5.3},
    {"id": 39009, "state": "Ohio", "county": "Athens County", "rate": 6},
    {"id": 39011, "state": "Ohio", "county": "Auglaize County", "rate": 3.5},
    {"id": 39013, "state": "Ohio", "county": "Belmont County", "rate": 6.3},
    {"id": 39015, "state": "Ohio", "county": "Brown County", "rate": 5},
    {"id": 39017, "state": "Ohio", "county": "Butler County", "rate": 4.2},
    {"id": 39019, "state": "Ohio", "county": "Carroll County", "rate": 6},
    {"id": 39021, "state": "Ohio", "county": "Champaign County", "rate": 4.1},
    {"id": 39023, "state": "Ohio", "county": "Clark County", "rate": 4.7},
    {"id": 39025, "state": "Ohio", "county": "Clermont County", "rate": 4.1},
    {"id": 39027, "state": "Ohio", "county": "Clinton County", "rate": 5.4},
    {"id": 39029, "state": "Ohio", "county": "Columbiana County", "rate": 6},
    {"id": 39031, "state": "Ohio", "county": "Coshocton County", "rate": 6.2},
    {"id": 39033, "state": "Ohio", "county": "Crawford County", "rate": 5.5},
    {"id": 39035, "state": "Ohio", "county": "Cuyahoga County", "rate": 5.5},
    {"id": 39037, "state": "Ohio", "county": "Darke County", "rate": 3.7},
    {"id": 39039, "state": "Ohio", "county": "Defiance County", "rate": 4.3},
    {"id": 39041, "state": "Ohio", "county": "Delaware County", "rate": 3.4},
    {"id": 39043, "state": "Ohio", "county": "Erie County", "rate": 4.6},
    {"id": 39045, "state": "Ohio", "county": "Fairfield County", "rate": 3.9},
    {"id": 39047, "state": "Ohio", "county": "Fayette County", "rate": 3.9},
    {"id": 39049, "state": "Ohio", "county": "Franklin County", "rate": 3.9},
    {"id": 39051, "state": "Ohio", "county": "Fulton County", "rate": 4},
    {"id": 39053, "state": "Ohio", "county": "Gallia County", "rate": 6.4},
    {"id": 39055, "state": "Ohio", "county": "Geauga County", "rate": 4.4},
    {"id": 39057, "state": "Ohio", "county": "Greene County", "rate": 4.2},
    {"id": 39059, "state": "Ohio", "county": "Guernsey County", "rate": 5.8},
    {"id": 39061, "state": "Ohio", "county": "Hamilton County", "rate": 4.2},
    {"id": 39063, "state": "Ohio", "county": "Hancock County", "rate": 3.5},
    {"id": 39065, "state": "Ohio", "county": "Hardin County", "rate": 4.9},
    {"id": 39067, "state": "Ohio", "county": "Harrison County", "rate": 6.4},
    {"id": 39069, "state": "Ohio", "county": "Henry County", "rate": 4.2},
    {"id": 39071, "state": "Ohio", "county": "Highland County", "rate": 5.5},
    {"id": 39073, "state": "Ohio", "county": "Hocking County", "rate": 4.5},
    {"id": 39075, "state": "Ohio", "county": "Holmes County", "rate": 3.3},
    {"id": 39077, "state": "Ohio", "county": "Huron County", "rate": 5},
    {"id": 39079, "state": "Ohio", "county": "Jackson County", "rate": 7},
    {"id": 39081, "state": "Ohio", "county": "Jefferson County", "rate": 7.4},
    {"id": 39083, "state": "Ohio", "county": "Knox County", "rate": 4.1},
    {"id": 39085, "state": "Ohio", "county": "Lake County", "rate": 4.8},
    {"id": 39087, "state": "Ohio", "county": "Lawrence County", "rate": 6.1},
    {"id": 39089, "state": "Ohio", "county": "Licking County", "rate": 4.1},
    {"id": 39091, "state": "Ohio", "county": "Logan County", "rate": 3.9},
    {"id": 39093, "state": "Ohio", "county": "Lorain County", "rate": 5.9},
    {"id": 39095, "state": "Ohio", "county": "Lucas County", "rate": 4.9},
    {"id": 39097, "state": "Ohio", "county": "Madison County", "rate": 3.5},
    {"id": 39099, "state": "Ohio", "county": "Mahoning County", "rate": 5.8},
    {"id": 39101, "state": "Ohio", "county": "Marion County", "rate": 4.7},
    {"id": 39103, "state": "Ohio", "county": "Medina County", "rate": 4.3},
    {"id": 39105, "state": "Ohio", "county": "Meigs County", "rate": 7.6},
    {"id": 39107, "state": "Ohio", "county": "Mercer County", "rate": 3},
    {"id": 39109, "state": "Ohio", "county": "Miami County", "rate": 4},
    {"id": 39111, "state": "Ohio", "county": "Monroe County", "rate": 9.1},
    {"id": 39113, "state": "Ohio", "county": "Montgomery County", "rate": 4.6},
    {"id": 39115, "state": "Ohio", "county": "Morgan County", "rate": 6.8},
    {"id": 39117, "state": "Ohio", "county": "Morrow County", "rate": 4.4},
    {"id": 39119, "state": "Ohio", "county": "Muskingum County", "rate": 5.3},
    {"id": 39121, "state": "Ohio", "county": "Noble County", "rate": 7.3},
    {"id": 39123, "state": "Ohio", "county": "Ottawa County", "rate": 4.6},
    {"id": 39125, "state": "Ohio", "county": "Paulding County", "rate": 4.2},
    {"id": 39127, "state": "Ohio", "county": "Perry County", "rate": 5.5},
    {"id": 39129, "state": "Ohio", "county": "Pickaway County", "rate": 4.2},
    {"id": 39131, "state": "Ohio", "county": "Pike County", "rate": 6.8},
    {"id": 39133, "state": "Ohio", "county": "Portage County", "rate": 4.5},
    {"id": 39135, "state": "Ohio", "county": "Preble County", "rate": 4.3},
    {"id": 39137, "state": "Ohio", "county": "Putnam County", "rate": 3.2},
    {"id": 39139, "state": "Ohio", "county": "Richland County", "rate": 4.9},
    {"id": 39141, "state": "Ohio", "county": "Ross County", "rate": 4.8},
    {"id": 39143, "state": "Ohio", "county": "Sandusky County", "rate": 4.1},
    {"id": 39145, "state": "Ohio", "county": "Scioto County", "rate": 7},
    {"id": 39147, "state": "Ohio", "county": "Seneca County", "rate": 4.4},
    {"id": 39149, "state": "Ohio", "county": "Shelby County", "rate": 3.7},
    {"id": 39151, "state": "Ohio", "county": "Stark County", "rate": 4.9},
    {"id": 39153, "state": "Ohio", "county": "Summit County", "rate": 4.7},
    {"id": 39155, "state": "Ohio", "county": "Trumbull County", "rate": 6},
    {"id": 39157, "state": "Ohio", "county": "Tuscarawas County", "rate": 5},
    {"id": 39159, "state": "Ohio", "county": "Union County", "rate": 3.5},
    {"id": 39161, "state": "Ohio", "county": "Van Wert County", "rate": 3.7},
    {"id": 39163, "state": "Ohio", "county": "Vinton County", "rate": 5.9},
    {"id": 39165, "state": "Ohio", "county": "Warren County", "rate": 3.9},
    {"id": 39167, "state": "Ohio", "county": "Washington County", "rate": 6.4},
    {"id": 39169, "state": "Ohio", "county": "Wayne County", "rate": 3.6},
    {"id": 39171, "state": "Ohio", "county": "Williams County", "rate": 4.1},
    {"id": 39173, "state": "Ohio", "county": "Wood County", "rate": 4},
    {"id": 39175, "state": "Ohio", "county": "Wyandot County", "rate": 3.4},
    {"id": 40001, "state": "Oklahoma", "county": "Adair County", "rate": 5.8},
    {"id": 40003, "state": "Oklahoma", "county": "Alfalfa County", "rate": 3.6},
    {"id": 40005, "state": "Oklahoma", "county": "Atoka County", "rate": 7.1},
    {"id": 40007, "state": "Oklahoma", "county": "Beaver County", "rate": 3.2},
    {"id": 40009, "state": "Oklahoma", "county": "Beckham County", "rate": 7.7},
    {"id": 40011, "state": "Oklahoma", "county": "Blaine County", "rate": 3.7},
    {"id": 40013, "state": "Oklahoma", "county": "Bryan County", "rate": 4.3},
    {"id": 40015, "state": "Oklahoma", "county": "Caddo County", "rate": 5.6},
    {
        "id": 40017,
        "state": "Oklahoma",
        "county": "Canadian County",
        "rate": 4.3,
    },
    {"id": 40019, "state": "Oklahoma", "county": "Carter County", "rate": 5.4},
    {
        "id": 40021,
        "state": "Oklahoma",
        "county": "Cherokee County",
        "rate": 5.6,
    },
    {"id": 40023, "state": "Oklahoma", "county": "Choctaw County", "rate": 8.3},
    {
        "id": 40025,
        "state": "Oklahoma",
        "county": "Cimarron County",
        "rate": 2.7,
    },
    {"id": 40027, "state": "Oklahoma", "county": "Cleveland County", "rate": 4},
    {"id": 40029, "state": "Oklahoma", "county": "Coal County", "rate": 7.5},
    {
        "id": 40031,
        "state": "Oklahoma",
        "county": "Comanche County",
        "rate": 4.8,
    },
    {"id": 40033, "state": "Oklahoma", "county": "Cotton County", "rate": 5.2},
    {"id": 40035, "state": "Oklahoma", "county": "Craig County", "rate": 4.9},
    {"id": 40037, "state": "Oklahoma", "county": "Creek County", "rate": 6.3},
    {"id": 40039, "state": "Oklahoma", "county": "Custer County", "rate": 4.9},
    {
        "id": 40041,
        "state": "Oklahoma",
        "county": "Delaware County",
        "rate": 5.2,
    },
    {"id": 40043, "state": "Oklahoma", "county": "Dewey County", "rate": 4.1},
    {"id": 40045, "state": "Oklahoma", "county": "Ellis County", "rate": 3.7},
    {
        "id": 40047,
        "state": "Oklahoma",
        "county": "Garfield County",
        "rate": 4.5,
    },
    {"id": 40049, "state": "Oklahoma", "county": "Garvin County", "rate": 5.7},
    {"id": 40051, "state": "Oklahoma", "county": "Grady County", "rate": 5.2},
    {"id": 40053, "state": "Oklahoma", "county": "Grant County", "rate": 3.1},
    {"id": 40055, "state": "Oklahoma", "county": "Greer County", "rate": 7.9},
    {"id": 40057, "state": "Oklahoma", "county": "Harmon County", "rate": 3.6},
    {"id": 40059, "state": "Oklahoma", "county": "Harper County", "rate": 4.3},
    {"id": 40061, "state": "Oklahoma", "county": "Haskell County", "rate": 8.8},
    {"id": 40063, "state": "Oklahoma", "county": "Hughes County", "rate": 7.5},
    {"id": 40065, "state": "Oklahoma", "county": "Jackson County", "rate": 4.9},
    {"id": 40067, "state": "Oklahoma", "county": "Jefferson County", "rate": 8},
    {
        "id": 40069,
        "state": "Oklahoma",
        "county": "Johnston County",
        "rate": 6.5,
    },
    {"id": 40071, "state": "Oklahoma", "county": "Kay County", "rate": 6.7},
    {
        "id": 40073,
        "state": "Oklahoma",
        "county": "Kingfisher County",
        "rate": 3.6,
    },
    {"id": 40075, "state": "Oklahoma", "county": "Kiowa County", "rate": 5.6},
    {"id": 40077, "state": "Oklahoma", "county": "Latimer County", "rate": 9.7},
    {
        "id": 40079,
        "state": "Oklahoma",
        "county": "Le Flore County",
        "rate": 7.4,
    },
    {"id": 40081, "state": "Oklahoma", "county": "Lincoln County", "rate": 5.5},
    {"id": 40083, "state": "Oklahoma", "county": "Logan County", "rate": 4.3},
    {"id": 40085, "state": "Oklahoma", "county": "Love County", "rate": 3.3},
    {"id": 40087, "state": "Oklahoma", "county": "McClain County", "rate": 4.3},
    {
        "id": 40089,
        "state": "Oklahoma",
        "county": "McCurtain County",
        "rate": 7.7,
    },
    {
        "id": 40091,
        "state": "Oklahoma",
        "county": "McIntosh County",
        "rate": 9.5,
    },
    {"id": 40093, "state": "Oklahoma", "county": "Major County", "rate": 4.3},
    {
        "id": 40095,
        "state": "Oklahoma",
        "county": "Marshall County",
        "rate": 5.1,
    },
    {"id": 40097, "state": "Oklahoma", "county": "Mayes County", "rate": 5.5},
    {"id": 40099, "state": "Oklahoma", "county": "Murray County", "rate": 4.3},
    {"id": 40101, "state": "Oklahoma", "county": "Muskogee County", "rate": 6},
    {"id": 40103, "state": "Oklahoma", "county": "Noble County", "rate": 4},
    {"id": 40105, "state": "Oklahoma", "county": "Nowata County", "rate": 6.5},
    {
        "id": 40107,
        "state": "Oklahoma",
        "county": "Okfuskee County",
        "rate": 6.1,
    },
    {
        "id": 40109,
        "state": "Oklahoma",
        "county": "Oklahoma County",
        "rate": 4.5,
    },
    {
        "id": 40111,
        "state": "Oklahoma",
        "county": "Okmulgee County",
        "rate": 7.4,
    },
    {"id": 40113, "state": "Oklahoma", "county": "Osage County", "rate": 6.1},
    {"id": 40115, "state": "Oklahoma", "county": "Ottawa County", "rate": 5.3},
    {"id": 40117, "state": "Oklahoma", "county": "Pawnee County", "rate": 7.1},
    {"id": 40119, "state": "Oklahoma", "county": "Payne County", "rate": 3.9},
    {
        "id": 40121,
        "state": "Oklahoma",
        "county": "Pittsburg County",
        "rate": 6.5,
    },
    {
        "id": 40123,
        "state": "Oklahoma",
        "county": "Pontotoc County",
        "rate": 4.2,
    },
    {
        "id": 40125,
        "state": "Oklahoma",
        "county": "Pottawatomie County",
        "rate": 5,
    },
    {
        "id": 40127,
        "state": "Oklahoma",
        "county": "Pushmataha County",
        "rate": 7.7,
    },
    {
        "id": 40129,
        "state": "Oklahoma",
        "county": "Roger Mills County",
        "rate": 5.4,
    },
    {"id": 40131, "state": "Oklahoma", "county": "Rogers County", "rate": 5.6},
    {
        "id": 40133,
        "state": "Oklahoma",
        "county": "Seminole County",
        "rate": 7.3,
    },
    {
        "id": 40135,
        "state": "Oklahoma",
        "county": "Sequoyah County",
        "rate": 6.5,
    },
    {
        "id": 40137,
        "state": "Oklahoma",
        "county": "Stephens County",
        "rate": 10.4,
    },
    {"id": 40139, "state": "Oklahoma", "county": "Texas County", "rate": 3.5},
    {"id": 40141, "state": "Oklahoma", "county": "Tillman County", "rate": 4.5},
    {"id": 40143, "state": "Oklahoma", "county": "Tulsa County", "rate": 5.2},
    {"id": 40145, "state": "Oklahoma", "county": "Wagoner County", "rate": 5.2},
    {
        "id": 40147,
        "state": "Oklahoma",
        "county": "Washington County",
        "rate": 5,
    },
    {"id": 40149, "state": "Oklahoma", "county": "Washita County", "rate": 7.2},
    {"id": 40151, "state": "Oklahoma", "county": "Woods County", "rate": 3.3},
    {
        "id": 40153,
        "state": "Oklahoma",
        "county": "Woodward County",
        "rate": 6.4,
    },
    {"id": 41001, "state": "Oregon", "county": "Baker County", "rate": 6.7},
    {"id": 41003, "state": "Oregon", "county": "Benton County", "rate": 5},
    {"id": 41005, "state": "Oregon", "county": "Clackamas County", "rate": 5.2},
    {"id": 41007, "state": "Oregon", "county": "Clatsop County", "rate": 5.1},
    {"id": 41009, "state": "Oregon", "county": "Columbia County", "rate": 7.1},
    {"id": 41011, "state": "Oregon", "county": "Coos County", "rate": 7.3},
    {"id": 41013, "state": "Oregon", "county": "Crook County", "rate": 6.9},
    {"id": 41015, "state": "Oregon", "county": "Curry County", "rate": 7},
    {"id": 41017, "state": "Oregon", "county": "Deschutes County", "rate": 5.1},
    {"id": 41019, "state": "Oregon", "county": "Douglas County", "rate": 7.3},
    {"id": 41021, "state": "Oregon", "county": "Gilliam County", "rate": 7.1},
    {"id": 41023, "state": "Oregon", "county": "Grant County", "rate": 6.1},
    {"id": 41025, "state": "Oregon", "county": "Harney County", "rate": 5.9},
    {"id": 41027, "state": "Oregon", "county": "Hood River County", "rate": 4},
    {"id": 41029, "state": "Oregon", "county": "Jackson County", "rate": 6.8},
    {"id": 41031, "state": "Oregon", "county": "Jefferson County", "rate": 6.6},
    {"id": 41033, "state": "Oregon", "county": "Josephine County", "rate": 7.4},
    {"id": 41035, "state": "Oregon", "county": "Klamath County", "rate": 7.2},
    {"id": 41037, "state": "Oregon", "county": "Lake County", "rate": 5.7},
    {"id": 41039, "state": "Oregon", "county": "Lane County", "rate": 6.2},
    {"id": 41041, "state": "Oregon", "county": "Lincoln County", "rate": 5.9},
    {"id": 41043, "state": "Oregon", "county": "Linn County", "rate": 6.7},
    {"id": 41045, "state": "Oregon", "county": "Malheur County", "rate": 6.1},
    {"id": 41047, "state": "Oregon", "county": "Marion County", "rate": 5.9},
    {"id": 41049, "state": "Oregon", "county": "Morrow County", "rate": 5.8},
    {"id": 41051, "state": "Oregon", "county": "Multnomah County", "rate": 5},
    {"id": 41053, "state": "Oregon", "county": "Polk County", "rate": 6},
    {"id": 41055, "state": "Oregon", "county": "Sherman County", "rate": 4.3},
    {"id": 41057, "state": "Oregon", "county": "Tillamook County", "rate": 5.2},
    {"id": 41059, "state": "Oregon", "county": "Umatilla County", "rate": 5.9},
    {"id": 41061, "state": "Oregon", "county": "Union County", "rate": 6.3},
    {"id": 41063, "state": "Oregon", "county": "Wallowa County", "rate": 5},
    {"id": 41065, "state": "Oregon", "county": "Wasco County", "rate": 5.9},
    {"id": 41067, "state": "Oregon", "county": "Washington County", "rate": 5},
    {"id": 41069, "state": "Oregon", "county": "Wheeler County", "rate": 4.9},
    {"id": 41071, "state": "Oregon", "county": "Yamhill County", "rate": 5.4},
    {
        "id": 42001,
        "state": "Pennsylvania",
        "county": "Adams County",
        "rate": 4.5,
    },
    {
        "id": 42003,
        "state": "Pennsylvania",
        "county": "Allegheny County",
        "rate": 5.9,
    },
    {
        "id": 42005,
        "state": "Pennsylvania",
        "county": "Armstrong County",
        "rate": 8,
    },
    {
        "id": 42007,
        "state": "Pennsylvania",
        "county": "Beaver County",
        "rate": 6.8,
    },
    {
        "id": 42009,
        "state": "Pennsylvania",
        "county": "Bedford County",
        "rate": 5.8,
    },
    {
        "id": 42011,
        "state": "Pennsylvania",
        "county": "Berks County",
        "rate": 5.6,
    },
    {
        "id": 42013,
        "state": "Pennsylvania",
        "county": "Blair County",
        "rate": 5.7,
    },
    {
        "id": 42015,
        "state": "Pennsylvania",
        "county": "Bradford County",
        "rate": 6.5,
    },
    {
        "id": 42017,
        "state": "Pennsylvania",
        "county": "Bucks County",
        "rate": 5.2,
    },
    {
        "id": 42019,
        "state": "Pennsylvania",
        "county": "Butler County",
        "rate": 5.7,
    },
    {
        "id": 42021,
        "state": "Pennsylvania",
        "county": "Cambria County",
        "rate": 7.7,
    },
    {
        "id": 42023,
        "state": "Pennsylvania",
        "county": "Cameron County",
        "rate": 8.2,
    },
    {
        "id": 42025,
        "state": "Pennsylvania",
        "county": "Carbon County",
        "rate": 6.7,
    },
    {
        "id": 42027,
        "state": "Pennsylvania",
        "county": "Centre County",
        "rate": 4.8,
    },
    {
        "id": 42029,
        "state": "Pennsylvania",
        "county": "Chester County",
        "rate": 4.6,
    },
    {
        "id": 42031,
        "state": "Pennsylvania",
        "county": "Clarion County",
        "rate": 7.2,
    },
    {
        "id": 42033,
        "state": "Pennsylvania",
        "county": "Clearfield County",
        "rate": 7.5,
    },
    {
        "id": 42035,
        "state": "Pennsylvania",
        "county": "Clinton County",
        "rate": 7.3,
    },
    {
        "id": 42037,
        "state": "Pennsylvania",
        "county": "Columbia County",
        "rate": 6.2,
    },
    {
        "id": 42039,
        "state": "Pennsylvania",
        "county": "Crawford County",
        "rate": 6.7,
    },
    {
        "id": 42041,
        "state": "Pennsylvania",
        "county": "Cumberland County",
        "rate": 4.7,
    },
    {
        "id": 42043,
        "state": "Pennsylvania",
        "county": "Dauphin County",
        "rate": 5.3,
    },
    {
        "id": 42045,
        "state": "Pennsylvania",
        "county": "Delaware County",
        "rate": 5.6,
    },
    {"id": 42047, "state": "Pennsylvania", "county": "Elk County", "rate": 6.7},
    {
        "id": 42049,
        "state": "Pennsylvania",
        "county": "Erie County",
        "rate": 7.1,
    },
    {
        "id": 42051,
        "state": "Pennsylvania",
        "county": "Fayette County",
        "rate": 8.5,
    },
    {
        "id": 42053,
        "state": "Pennsylvania",
        "county": "Forest County",
        "rate": 8.5,
    },
    {
        "id": 42055,
        "state": "Pennsylvania",
        "county": "Franklin County",
        "rate": 5.7,
    },
    {
        "id": 42057,
        "state": "Pennsylvania",
        "county": "Fulton County",
        "rate": 6.3,
    },
    {
        "id": 42059,
        "state": "Pennsylvania",
        "county": "Greene County",
        "rate": 8.1,
    },
    {
        "id": 42061,
        "state": "Pennsylvania",
        "county": "Huntingdon County",
        "rate": 7.1,
    },
    {
        "id": 42063,
        "state": "Pennsylvania",
        "county": "Indiana County",
        "rate": 8.3,
    },
    {
        "id": 42065,
        "state": "Pennsylvania",
        "county": "Jefferson County",
        "rate": 7.3,
    },
    {
        "id": 42067,
        "state": "Pennsylvania",
        "county": "Juniata County",
        "rate": 5.8,
    },
    {
        "id": 42069,
        "state": "Pennsylvania",
        "county": "Lackawanna County",
        "rate": 6.4,
    },
    {
        "id": 42071,
        "state": "Pennsylvania",
        "county": "Lancaster County",
        "rate": 4.8,
    },
    {
        "id": 42073,
        "state": "Pennsylvania",
        "county": "Lawrence County",
        "rate": 7,
    },
    {
        "id": 42075,
        "state": "Pennsylvania",
        "county": "Lebanon County",
        "rate": 5.1,
    },
    {
        "id": 42077,
        "state": "Pennsylvania",
        "county": "Lehigh County",
        "rate": 6.3,
    },
    {
        "id": 42079,
        "state": "Pennsylvania",
        "county": "Luzerne County",
        "rate": 6.9,
    },
    {
        "id": 42081,
        "state": "Pennsylvania",
        "county": "Lycoming County",
        "rate": 7,
    },
    {
        "id": 42083,
        "state": "Pennsylvania",
        "county": "McKean County",
        "rate": 7.2,
    },
    {
        "id": 42085,
        "state": "Pennsylvania",
        "county": "Mercer County",
        "rate": 6.8,
    },
    {
        "id": 42087,
        "state": "Pennsylvania",
        "county": "Mifflin County",
        "rate": 6,
    },
    {
        "id": 42089,
        "state": "Pennsylvania",
        "county": "Monroe County",
        "rate": 6.9,
    },
    {
        "id": 42091,
        "state": "Pennsylvania",
        "county": "Montgomery County",
        "rate": 4.7,
    },
    {
        "id": 42093,
        "state": "Pennsylvania",
        "county": "Montour County",
        "rate": 4.9,
    },
    {
        "id": 42095,
        "state": "Pennsylvania",
        "county": "Northampton County",
        "rate": 5.9,
    },
    {
        "id": 42097,
        "state": "Pennsylvania",
        "county": "Northumberland County",
        "rate": 6.6,
    },
    {
        "id": 42099,
        "state": "Pennsylvania",
        "county": "Perry County",
        "rate": 5.1,
    },
    {
        "id": 42101,
        "state": "Pennsylvania",
        "county": "Philadelphia County",
        "rate": 7.7,
    },
    {
        "id": 42103,
        "state": "Pennsylvania",
        "county": "Pike County",
        "rate": 6.8,
    },
    {
        "id": 42105,
        "state": "Pennsylvania",
        "county": "Potter County",
        "rate": 7.8,
    },
    {
        "id": 42107,
        "state": "Pennsylvania",
        "county": "Schuylkill County",
        "rate": 6.7,
    },
    {
        "id": 42109,
        "state": "Pennsylvania",
        "county": "Snyder County",
        "rate": 5.1,
    },
    {
        "id": 42111,
        "state": "Pennsylvania",
        "county": "Somerset County",
        "rate": 7.5,
    },
    {
        "id": 42113,
        "state": "Pennsylvania",
        "county": "Sullivan County",
        "rate": 7,
    },
    {
        "id": 42115,
        "state": "Pennsylvania",
        "county": "Susquehanna County",
        "rate": 5.4,
    },
    {
        "id": 42117,
        "state": "Pennsylvania",
        "county": "Tioga County",
        "rate": 7.4,
    },
    {
        "id": 42119,
        "state": "Pennsylvania",
        "county": "Union County",
        "rate": 5.4,
    },
    {
        "id": 42121,
        "state": "Pennsylvania",
        "county": "Venango County",
        "rate": 8,
    },
    {
        "id": 42123,
        "state": "Pennsylvania",
        "county": "Warren County",
        "rate": 5.9,
    },
    {
        "id": 42125,
        "state": "Pennsylvania",
        "county": "Washington County",
        "rate": 6.6,
    },
    {
        "id": 42127,
        "state": "Pennsylvania",
        "county": "Wayne County",
        "rate": 5.6,
    },
    {
        "id": 42129,
        "state": "Pennsylvania",
        "county": "Westmoreland County",
        "rate": 6.6,
    },
    {
        "id": 42131,
        "state": "Pennsylvania",
        "county": "Wyoming County",
        "rate": 6.2,
    },
    {
        "id": 42133,
        "state": "Pennsylvania",
        "county": "York County",
        "rate": 5.2,
    },
    {
        "id": 44001,
        "state": "Rhode Island",
        "county": "Bristol County",
        "rate": 5,
    },
    {
        "id": 44003,
        "state": "Rhode Island",
        "county": "Kent County",
        "rate": 5.3,
    },
    {
        "id": 44005,
        "state": "Rhode Island",
        "county": "Newport County",
        "rate": 4.8,
    },
    {
        "id": 44007,
        "state": "Rhode Island",
        "county": "Providence County",
        "rate": 6.3,
    },
    {
        "id": 44009,
        "state": "Rhode Island",
        "county": "Washington County",
        "rate": 5,
    },
    {
        "id": 45001,
        "state": "South Carolina",
        "county": "Abbeville County",
        "rate": 6.1,
    },
    {
        "id": 45003,
        "state": "South Carolina",
        "county": "Aiken County",
        "rate": 5.6,
    },
    {
        "id": 45005,
        "state": "South Carolina",
        "county": "Allendale County",
        "rate": 9.1,
    },
    {
        "id": 45007,
        "state": "South Carolina",
        "county": "Anderson County",
        "rate": 5.2,
    },
    {
        "id": 45009,
        "state": "South Carolina",
        "county": "Bamberg County",
        "rate": 11.4,
    },
    {
        "id": 45011,
        "state": "South Carolina",
        "county": "Barnwell County",
        "rate": 8,
    },
    {
        "id": 45013,
        "state": "South Carolina",
        "county": "Beaufort County",
        "rate": 5.1,
    },
    {
        "id": 45015,
        "state": "South Carolina",
        "county": "Berkeley County",
        "rate": 5,
    },
    {
        "id": 45017,
        "state": "South Carolina",
        "county": "Calhoun County",
        "rate": 7.4,
    },
    {
        "id": 45019,
        "state": "South Carolina",
        "county": "Charleston County",
        "rate": 4.5,
    },
    {
        "id": 45021,
        "state": "South Carolina",
        "county": "Cherokee County",
        "rate": 6.6,
    },
    {
        "id": 45023,
        "state": "South Carolina",
        "county": "Chester County",
        "rate": 8.4,
    },
    {
        "id": 45025,
        "state": "South Carolina",
        "county": "Chesterfield County",
        "rate": 5.7,
    },
    {
        "id": 45027,
        "state": "South Carolina",
        "county": "Clarendon County",
        "rate": 7.1,
    },
    {
        "id": 45029,
        "state": "South Carolina",
        "county": "Colleton County",
        "rate": 5.8,
    },
    {
        "id": 45031,
        "state": "South Carolina",
        "county": "Darlington County",
        "rate": 6.8,
    },
    {
        "id": 45033,
        "state": "South Carolina",
        "county": "Dillon County",
        "rate": 7.1,
    },
    {
        "id": 45035,
        "state": "South Carolina",
        "county": "Dorchester County",
        "rate": 4.9,
    },
    {
        "id": 45037,
        "state": "South Carolina",
        "county": "Edgefield County",
        "rate": 6.1,
    },
    {
        "id": 45039,
        "state": "South Carolina",
        "county": "Fairfield County",
        "rate": 7.7,
    },
    {
        "id": 45041,
        "state": "South Carolina",
        "county": "Florence County",
        "rate": 5.8,
    },
    {
        "id": 45043,
        "state": "South Carolina",
        "county": "Georgetown County",
        "rate": 6.6,
    },
    {
        "id": 45045,
        "state": "South Carolina",
        "county": "Greenville County",
        "rate": 4.7,
    },
    {
        "id": 45047,
        "state": "South Carolina",
        "county": "Greenwood County",
        "rate": 5.8,
    },
    {
        "id": 45049,
        "state": "South Carolina",
        "county": "Hampton County",
        "rate": 6.2,
    },
    {
        "id": 45051,
        "state": "South Carolina",
        "county": "Horry County",
        "rate": 5.4,
    },
    {
        "id": 45053,
        "state": "South Carolina",
        "county": "Jasper County",
        "rate": 4.7,
    },
    {
        "id": 45055,
        "state": "South Carolina",
        "county": "Kershaw County",
        "rate": 5.8,
    },
    {
        "id": 45057,
        "state": "South Carolina",
        "county": "Lancaster County",
        "rate": 6,
    },
    {
        "id": 45059,
        "state": "South Carolina",
        "county": "Laurens County",
        "rate": 5.6,
    },
    {
        "id": 45061,
        "state": "South Carolina",
        "county": "Lee County",
        "rate": 8.2,
    },
    {
        "id": 45063,
        "state": "South Carolina",
        "county": "Lexington County",
        "rate": 4.5,
    },
    {
        "id": 45065,
        "state": "South Carolina",
        "county": "McCormick County",
        "rate": 5.8,
    },
    {
        "id": 45067,
        "state": "South Carolina",
        "county": "Marion County",
        "rate": 8.6,
    },
    {
        "id": 45069,
        "state": "South Carolina",
        "county": "Marlboro County",
        "rate": 9.1,
    },
    {
        "id": 45071,
        "state": "South Carolina",
        "county": "Newberry County",
        "rate": 4.8,
    },
    {
        "id": 45073,
        "state": "South Carolina",
        "county": "Oconee County",
        "rate": 5.5,
    },
    {
        "id": 45075,
        "state": "South Carolina",
        "county": "Orangeburg County",
        "rate": 11.3,
    },
    {
        "id": 45077,
        "state": "South Carolina",
        "county": "Pickens County",
        "rate": 5.6,
    },
    {
        "id": 45079,
        "state": "South Carolina",
        "county": "Richland County",
        "rate": 5.4,
    },
    {
        "id": 45081,
        "state": "South Carolina",
        "county": "Saluda County",
        "rate": 4.8,
    },
    {
        "id": 45083,
        "state": "South Carolina",
        "county": "Spartanburg County",
        "rate": 5.3,
    },
    {
        "id": 45085,
        "state": "South Carolina",
        "county": "Sumter County",
        "rate": 6.5,
    },
    {
        "id": 45087,
        "state": "South Carolina",
        "county": "Union County",
        "rate": 6.5,
    },
    {
        "id": 45089,
        "state": "South Carolina",
        "county": "Williamsburg County",
        "rate": 7.9,
    },
    {
        "id": 45091,
        "state": "South Carolina",
        "county": "York County",
        "rate": 5.2,
    },
    {
        "id": 46003,
        "state": "South Dakota",
        "county": "Aurora County",
        "rate": 1.9,
    },
    {
        "id": 46005,
        "state": "South Dakota",
        "county": "Beadle County",
        "rate": 2.3,
    },
    {
        "id": 46007,
        "state": "South Dakota",
        "county": "Bennett County",
        "rate": 4,
    },
    {
        "id": 46009,
        "state": "South Dakota",
        "county": "Bon Homme County",
        "rate": 2.4,
    },
    {
        "id": 46011,
        "state": "South Dakota",
        "county": "Brookings County",
        "rate": 2.5,
    },
    {
        "id": 46013,
        "state": "South Dakota",
        "county": "Brown County",
        "rate": 2.4,
    },
    {
        "id": 46015,
        "state": "South Dakota",
        "county": "Brule County",
        "rate": 2.5,
    },
    {
        "id": 46017,
        "state": "South Dakota",
        "county": "Buffalo County",
        "rate": 9.8,
    },
    {
        "id": 46019,
        "state": "South Dakota",
        "county": "Butte County",
        "rate": 2.9,
    },
    {
        "id": 46021,
        "state": "South Dakota",
        "county": "Campbell County",
        "rate": 2.5,
    },
    {
        "id": 46023,
        "state": "South Dakota",
        "county": "Charles Mix County",
        "rate": 3,
    },
    {
        "id": 46025,
        "state": "South Dakota",
        "county": "Clark County",
        "rate": 2.9,
    },
    {
        "id": 46027,
        "state": "South Dakota",
        "county": "Clay County",
        "rate": 2.7,
    },
    {
        "id": 46029,
        "state": "South Dakota",
        "county": "Codington County",
        "rate": 2.6,
    },
    {
        "id": 46031,
        "state": "South Dakota",
        "county": "Corson County",
        "rate": 5.5,
    },
    {
        "id": 46033,
        "state": "South Dakota",
        "county": "Custer County",
        "rate": 3.1,
    },
    {
        "id": 46035,
        "state": "South Dakota",
        "county": "Davison County",
        "rate": 1.9,
    },
    {"id": 46037, "state": "South Dakota", "county": "Day County", "rate": 3.6},
    {
        "id": 46039,
        "state": "South Dakota",
        "county": "Deuel County",
        "rate": 3.2,
    },
    {
        "id": 46041,
        "state": "South Dakota",
        "county": "Dewey County",
        "rate": 14.1,
    },
    {
        "id": 46043,
        "state": "South Dakota",
        "county": "Douglas County",
        "rate": 2.2,
    },
    {
        "id": 46045,
        "state": "South Dakota",
        "county": "Edmunds County",
        "rate": 2.1,
    },
    {
        "id": 46047,
        "state": "South Dakota",
        "county": "Fall River County",
        "rate": 3.8,
    },
    {
        "id": 46049,
        "state": "South Dakota",
        "county": "Faulk County",
        "rate": 2.5,
    },
    {
        "id": 46051,
        "state": "South Dakota",
        "county": "Grant County",
        "rate": 2.5,
    },
    {
        "id": 46053,
        "state": "South Dakota",
        "county": "Gregory County",
        "rate": 2.6,
    },
    {
        "id": 46055,
        "state": "South Dakota",
        "county": "Haakon County",
        "rate": 2.1,
    },
    {
        "id": 46057,
        "state": "South Dakota",
        "county": "Hamlin County",
        "rate": 2.4,
    },
    {
        "id": 46059,
        "state": "South Dakota",
        "county": "Hand County",
        "rate": 1.9,
    },
    {
        "id": 46061,
        "state": "South Dakota",
        "county": "Hanson County",
        "rate": 3.2,
    },
    {
        "id": 46063,
        "state": "South Dakota",
        "county": "Harding County",
        "rate": 2.1,
    },
    {
        "id": 46065,
        "state": "South Dakota",
        "county": "Hughes County",
        "rate": 2,
    },
    {
        "id": 46067,
        "state": "South Dakota",
        "county": "Hutchinson County",
        "rate": 2,
    },
    {
        "id": 46069,
        "state": "South Dakota",
        "county": "Hyde County",
        "rate": 2.8,
    },
    {
        "id": 46071,
        "state": "South Dakota",
        "county": "Jackson County",
        "rate": 2.8,
    },
    {
        "id": 46073,
        "state": "South Dakota",
        "county": "Jerauld County",
        "rate": 2.4,
    },
    {
        "id": 46075,
        "state": "South Dakota",
        "county": "Jones County",
        "rate": 2.4,
    },
    {
        "id": 46077,
        "state": "South Dakota",
        "county": "Kingsbury County",
        "rate": 2.1,
    },
    {"id": 46079, "state": "South Dakota", "county": "Lake County", "rate": 3},
    {
        "id": 46081,
        "state": "South Dakota",
        "county": "Lawrence County",
        "rate": 2.9,
    },
    {
        "id": 46083,
        "state": "South Dakota",
        "county": "Lincoln County",
        "rate": 1.8,
    },
    {"id": 46085, "state": "South Dakota", "county": "Lyman County", "rate": 5},
    {
        "id": 46087,
        "state": "South Dakota",
        "county": "McCook County",
        "rate": 2.1,
    },
    {
        "id": 46089,
        "state": "South Dakota",
        "county": "McPherson County",
        "rate": 3.6,
    },
    {
        "id": 46091,
        "state": "South Dakota",
        "county": "Marshall County",
        "rate": 2.8,
    },
    {
        "id": 46093,
        "state": "South Dakota",
        "county": "Meade County",
        "rate": 2.7,
    },
    {
        "id": 46095,
        "state": "South Dakota",
        "county": "Mellette County",
        "rate": 3.8,
    },
    {
        "id": 46097,
        "state": "South Dakota",
        "county": "Miner County",
        "rate": 2.4,
    },
    {
        "id": 46099,
        "state": "South Dakota",
        "county": "Minnehaha County",
        "rate": 2.2,
    },
    {
        "id": 46101,
        "state": "South Dakota",
        "county": "Moody County",
        "rate": 3.6,
    },
    {
        "id": 46102,
        "state": "South Dakota",
        "county": "Oglala Lakota County",
        "rate": 13.2,
    },
    {
        "id": 46103,
        "state": "South Dakota",
        "county": "Pennington County",
        "rate": 2.6,
    },
    {
        "id": 46105,
        "state": "South Dakota",
        "county": "Perkins County",
        "rate": 2.8,
    },
    {
        "id": 46107,
        "state": "South Dakota",
        "county": "Potter County",
        "rate": 2.3,
    },
    {
        "id": 46109,
        "state": "South Dakota",
        "county": "Roberts County",
        "rate": 3.3,
    },
    {
        "id": 46111,
        "state": "South Dakota",
        "county": "Sanborn County",
        "rate": 2.5,
    },
    {
        "id": 46115,
        "state": "South Dakota",
        "county": "Spink County",
        "rate": 2.6,
    },
    {
        "id": 46117,
        "state": "South Dakota",
        "county": "Stanley County",
        "rate": 2,
    },
    {
        "id": 46119,
        "state": "South Dakota",
        "county": "Sully County",
        "rate": 1.9,
    },
    {
        "id": 46121,
        "state": "South Dakota",
        "county": "Todd County",
        "rate": 6.8,
    },
    {
        "id": 46123,
        "state": "South Dakota",
        "county": "Tripp County",
        "rate": 1.9,
    },
    {
        "id": 46125,
        "state": "South Dakota",
        "county": "Turner County",
        "rate": 2.4,
    },
    {
        "id": 46127,
        "state": "South Dakota",
        "county": "Union County",
        "rate": 2.9,
    },
    {
        "id": 46129,
        "state": "South Dakota",
        "county": "Walworth County",
        "rate": 4.3,
    },
    {
        "id": 46135,
        "state": "South Dakota",
        "county": "Yankton County",
        "rate": 2.3,
    },
    {
        "id": 46137,
        "state": "South Dakota",
        "county": "Ziebach County",
        "rate": 5.6,
    },
    {
        "id": 47001,
        "state": "Tennessee",
        "county": "Anderson County",
        "rate": 5.1,
    },
    {
        "id": 47003,
        "state": "Tennessee",
        "county": "Bedford County",
        "rate": 5.3,
    },
    {"id": 47005, "state": "Tennessee", "county": "Benton County", "rate": 7.4},
    {
        "id": 47007,
        "state": "Tennessee",
        "county": "Bledsoe County",
        "rate": 6.6,
    },
    {"id": 47009, "state": "Tennessee", "county": "Blount County", "rate": 4.5},
    {
        "id": 47011,
        "state": "Tennessee",
        "county": "Bradley County",
        "rate": 4.7,
    },
    {
        "id": 47013,
        "state": "Tennessee",
        "county": "Campbell County",
        "rate": 6.7,
    },
    {"id": 47015, "state": "Tennessee", "county": "Cannon County", "rate": 4.9},
    {
        "id": 47017,
        "state": "Tennessee",
        "county": "Carroll County",
        "rate": 7.1,
    },
    {"id": 47019, "state": "Tennessee", "county": "Carter County", "rate": 5.9},
    {"id": 47021, "state": "Tennessee", "county": "Cheatham County", "rate": 4},
    {
        "id": 47023,
        "state": "Tennessee",
        "county": "Chester County",
        "rate": 5.7,
    },
    {
        "id": 47025,
        "state": "Tennessee",
        "county": "Claiborne County",
        "rate": 6.4,
    },
    {"id": 47027, "state": "Tennessee", "county": "Clay County", "rate": 6.4},
    {"id": 47029, "state": "Tennessee", "county": "Cocke County", "rate": 6},
    {"id": 47031, "state": "Tennessee", "county": "Coffee County", "rate": 4.9},
    {
        "id": 47033,
        "state": "Tennessee",
        "county": "Crockett County",
        "rate": 5.2,
    },
    {
        "id": 47035,
        "state": "Tennessee",
        "county": "Cumberland County",
        "rate": 5.9,
    },
    {
        "id": 47037,
        "state": "Tennessee",
        "county": "Davidson County",
        "rate": 3.8,
    },
    {
        "id": 47039,
        "state": "Tennessee",
        "county": "Decatur County",
        "rate": 7.3,
    },
    {"id": 47041, "state": "Tennessee", "county": "DeKalb County", "rate": 5.8},
    {
        "id": 47043,
        "state": "Tennessee",
        "county": "Dickson County",
        "rate": 4.4,
    },
    {"id": 47045, "state": "Tennessee", "county": "Dyer County", "rate": 6.1},
    {
        "id": 47047,
        "state": "Tennessee",
        "county": "Fayette County",
        "rate": 5.3,
    },
    {"id": 47049, "state": "Tennessee", "county": "Fentress County", "rate": 6},
    {
        "id": 47051,
        "state": "Tennessee",
        "county": "Franklin County",
        "rate": 5.7,
    },
    {"id": 47053, "state": "Tennessee", "county": "Gibson County", "rate": 6.2},
    {"id": 47055, "state": "Tennessee", "county": "Giles County", "rate": 4.3},
    {
        "id": 47057,
        "state": "Tennessee",
        "county": "Grainger County",
        "rate": 5.4,
    },
    {"id": 47059, "state": "Tennessee", "county": "Greene County", "rate": 5.8},
    {"id": 47061, "state": "Tennessee", "county": "Grundy County", "rate": 6.6},
    {
        "id": 47063,
        "state": "Tennessee",
        "county": "Hamblen County",
        "rate": 5.3,
    },
    {"id": 47065, "state": "Tennessee", "county": "Hamilton County", "rate": 5},
    {"id": 47067, "state": "Tennessee", "county": "Hancock County", "rate": 8},
    {
        "id": 47069,
        "state": "Tennessee",
        "county": "Hardeman County",
        "rate": 6.2,
    },
    {"id": 47071, "state": "Tennessee", "county": "Hardin County", "rate": 6.2},
    {
        "id": 47073,
        "state": "Tennessee",
        "county": "Hawkins County",
        "rate": 5.5,
    },
    {
        "id": 47075,
        "state": "Tennessee",
        "county": "Haywood County",
        "rate": 6.5,
    },
    {
        "id": 47077,
        "state": "Tennessee",
        "county": "Henderson County",
        "rate": 7,
    },
    {"id": 47079, "state": "Tennessee", "county": "Henry County", "rate": 6},
    {
        "id": 47081,
        "state": "Tennessee",
        "county": "Hickman County",
        "rate": 4.6,
    },
    {
        "id": 47083,
        "state": "Tennessee",
        "county": "Houston County",
        "rate": 7.7,
    },
    {
        "id": 47085,
        "state": "Tennessee",
        "county": "Humphreys County",
        "rate": 6.2,
    },
    {
        "id": 47087,
        "state": "Tennessee",
        "county": "Jackson County",
        "rate": 7.3,
    },
    {
        "id": 47089,
        "state": "Tennessee",
        "county": "Jefferson County",
        "rate": 5.4,
    },
    {
        "id": 47091,
        "state": "Tennessee",
        "county": "Johnson County",
        "rate": 4.9,
    },
    {"id": 47093, "state": "Tennessee", "county": "Knox County", "rate": 4.3},
    {"id": 47095, "state": "Tennessee", "county": "Lake County", "rate": 7.3},
    {
        "id": 47097,
        "state": "Tennessee",
        "county": "Lauderdale County",
        "rate": 7.8,
    },
    {
        "id": 47099,
        "state": "Tennessee",
        "county": "Lawrence County",
        "rate": 5.9,
    },
    {"id": 47101, "state": "Tennessee", "county": "Lewis County", "rate": 5.8},
    {
        "id": 47103,
        "state": "Tennessee",
        "county": "Lincoln County",
        "rate": 4.4,
    },
    {"id": 47105, "state": "Tennessee", "county": "Loudon County", "rate": 4.8},
    {"id": 47107, "state": "Tennessee", "county": "McMinn County", "rate": 5.8},
    {
        "id": 47109,
        "state": "Tennessee",
        "county": "McNairy County",
        "rate": 7.4,
    },
    {"id": 47111, "state": "Tennessee", "county": "Macon County", "rate": 4.5},
    {
        "id": 47113,
        "state": "Tennessee",
        "county": "Madison County",
        "rate": 5.3,
    },
    {"id": 47115, "state": "Tennessee", "county": "Marion County", "rate": 6.7},
    {
        "id": 47117,
        "state": "Tennessee",
        "county": "Marshall County",
        "rate": 4.8,
    },
    {"id": 47119, "state": "Tennessee", "county": "Maury County", "rate": 4.2},
    {"id": 47121, "state": "Tennessee", "county": "Meigs County", "rate": 7.2},
    {"id": 47123, "state": "Tennessee", "county": "Monroe County", "rate": 5.5},
    {
        "id": 47125,
        "state": "Tennessee",
        "county": "Montgomery County",
        "rate": 5.5,
    },
    {"id": 47127, "state": "Tennessee", "county": "Moore County", "rate": 4.2},
    {"id": 47129, "state": "Tennessee", "county": "Morgan County", "rate": 6.6},
    {"id": 47131, "state": "Tennessee", "county": "Obion County", "rate": 7.5},
    {
        "id": 47133,
        "state": "Tennessee",
        "county": "Overton County",
        "rate": 5.6,
    },
    {"id": 47135, "state": "Tennessee", "county": "Perry County", "rate": 6.2},
    {
        "id": 47137,
        "state": "Tennessee",
        "county": "Pickett County",
        "rate": 5.7,
    },
    {"id": 47139, "state": "Tennessee", "county": "Polk County", "rate": 5.8},
    {"id": 47141, "state": "Tennessee", "county": "Putnam County", "rate": 5.2},
    {"id": 47143, "state": "Tennessee", "county": "Rhea County", "rate": 7.5},
    {"id": 47145, "state": "Tennessee", "county": "Roane County", "rate": 5.8},
    {
        "id": 47147,
        "state": "Tennessee",
        "county": "Robertson County",
        "rate": 4.4,
    },
    {
        "id": 47149,
        "state": "Tennessee",
        "county": "Rutherford County",
        "rate": 4,
    },
    {"id": 47151, "state": "Tennessee", "county": "Scott County", "rate": 7.4},
    {
        "id": 47153,
        "state": "Tennessee",
        "county": "Sequatchie County",
        "rate": 6.1,
    },
    {"id": 47155, "state": "Tennessee", "county": "Sevier County", "rate": 4.2},
    {"id": 47157, "state": "Tennessee", "county": "Shelby County", "rate": 5.8},
    {"id": 47159, "state": "Tennessee", "county": "Smith County", "rate": 4.6},
    {
        "id": 47161,
        "state": "Tennessee",
        "county": "Stewart County",
        "rate": 6.8,
    },
    {
        "id": 47163,
        "state": "Tennessee",
        "county": "Sullivan County",
        "rate": 5.5,
    },
    {"id": 47165, "state": "Tennessee", "county": "Sumner County", "rate": 4.1},
    {"id": 47167, "state": "Tennessee", "county": "Tipton County", "rate": 5.9},
    {
        "id": 47169,
        "state": "Tennessee",
        "county": "Trousdale County",
        "rate": 4.9,
    },
    {"id": 47171, "state": "Tennessee", "county": "Unicoi County", "rate": 6.9},
    {"id": 47173, "state": "Tennessee", "county": "Union County", "rate": 5.9},
    {
        "id": 47175,
        "state": "Tennessee",
        "county": "Van Buren County",
        "rate": 5.8,
    },
    {"id": 47177, "state": "Tennessee", "county": "Warren County", "rate": 4.8},
    {
        "id": 47179,
        "state": "Tennessee",
        "county": "Washington County",
        "rate": 5.1,
    },
    {"id": 47181, "state": "Tennessee", "county": "Wayne County", "rate": 6.2},
    {
        "id": 47183,
        "state": "Tennessee",
        "county": "Weakley County",
        "rate": 7.3,
    },
    {"id": 47185, "state": "Tennessee", "county": "White County", "rate": 5.4},
    {
        "id": 47187,
        "state": "Tennessee",
        "county": "Williamson County",
        "rate": 3.7,
    },
    {"id": 47189, "state": "Tennessee", "county": "Wilson County", "rate": 4},
    {"id": 48001, "state": "Texas", "county": "Anderson County", "rate": 4.8},
    {"id": 48003, "state": "Texas", "county": "Andrews County", "rate": 4.9},
    {"id": 48005, "state": "Texas", "county": "Angelina County", "rate": 6.6},
    {"id": 48007, "state": "Texas", "county": "Aransas County", "rate": 5.9},
    {"id": 48009, "state": "Texas", "county": "Archer County", "rate": 4.5},
    {"id": 48011, "state": "Texas", "county": "Armstrong County", "rate": 3.5},
    {"id": 48013, "state": "Texas", "county": "Atascosa County", "rate": 5.7},
    {"id": 48015, "state": "Texas", "county": "Austin County", "rate": 5.9},
    {"id": 48017, "state": "Texas", "county": "Bailey County", "rate": 4.6},
    {"id": 48019, "state": "Texas", "county": "Bandera County", "rate": 4.3},
    {"id": 48021, "state": "Texas", "county": "Bastrop County", "rate": 4.1},
    {"id": 48023, "state": "Texas", "county": "Baylor County", "rate": 3.5},
    {"id": 48025, "state": "Texas", "county": "Bee County", "rate": 8.5},
    {"id": 48027, "state": "Texas", "county": "Bell County", "rate": 4.8},
    {"id": 48029, "state": "Texas", "county": "Bexar County", "rate": 4.1},
    {"id": 48031, "state": "Texas", "county": "Blanco County", "rate": 3.3},
    {"id": 48033, "state": "Texas", "county": "Borden County", "rate": 3.4},
    {"id": 48035, "state": "Texas", "county": "Bosque County", "rate": 4.7},
    {"id": 48037, "state": "Texas", "county": "Bowie County", "rate": 5.4},
    {"id": 48039, "state": "Texas", "county": "Brazoria County", "rate": 5.7},
    {"id": 48041, "state": "Texas", "county": "Brazos County", "rate": 3.9},
    {"id": 48043, "state": "Texas", "county": "Brewster County", "rate": 4.3},
    {"id": 48045, "state": "Texas", "county": "Briscoe County", "rate": 4.8},
    {"id": 48047, "state": "Texas", "county": "Brooks County", "rate": 12.5},
    {"id": 48049, "state": "Texas", "county": "Brown County", "rate": 4.8},
    {"id": 48051, "state": "Texas", "county": "Burleson County", "rate": 5.2},
    {"id": 48053, "state": "Texas", "county": "Burnet County", "rate": 3.8},
    {"id": 48055, "state": "Texas", "county": "Caldwell County", "rate": 4.5},
    {"id": 48057, "state": "Texas", "county": "Calhoun County", "rate": 5.5},
    {"id": 48059, "state": "Texas", "county": "Callahan County", "rate": 4.6},
    {"id": 48061, "state": "Texas", "county": "Cameron County", "rate": 7.6},
    {"id": 48063, "state": "Texas", "county": "Camp County", "rate": 7.8},
    {"id": 48065, "state": "Texas", "county": "Carson County", "rate": 3.5},
    {"id": 48067, "state": "Texas", "county": "Cass County", "rate": 8.2},
    {"id": 48069, "state": "Texas", "county": "Castro County", "rate": 3.5},
    {"id": 48071, "state": "Texas", "county": "Chambers County", "rate": 6.8},
    {"id": 48073, "state": "Texas", "county": "Cherokee County", "rate": 5.5},
    {"id": 48075, "state": "Texas", "county": "Childress County", "rate": 3.5},
    {"id": 48077, "state": "Texas", "county": "Clay County", "rate": 4.5},
    {"id": 48079, "state": "Texas", "county": "Cochran County", "rate": 6},
    {"id": 48081, "state": "Texas", "county": "Coke County", "rate": 3.9},
    {"id": 48083, "state": "Texas", "county": "Coleman County", "rate": 6.4},
    {"id": 48085, "state": "Texas", "county": "Collin County", "rate": 3.8},
    {
        "id": 48087,
        "state": "Texas",
        "county": "Collingsworth County",
        "rate": 3.6,
    },
    {"id": 48089, "state": "Texas", "county": "Colorado County", "rate": 5.3},
    {"id": 48091, "state": "Texas", "county": "Comal County", "rate": 4.2},
    {"id": 48093, "state": "Texas", "county": "Comanche County", "rate": 4.6},
    {"id": 48095, "state": "Texas", "county": "Concho County", "rate": 3.2},
    {"id": 48097, "state": "Texas", "county": "Cooke County", "rate": 4.2},
    {"id": 48099, "state": "Texas", "county": "Coryell County", "rate": 4.9},
    {"id": 48101, "state": "Texas", "county": "Cottle County", "rate": 5},
    {"id": 48103, "state": "Texas", "county": "Crane County", "rate": 9.1},
    {"id": 48105, "state": "Texas", "county": "Crockett County", "rate": 6.7},
    {"id": 48107, "state": "Texas", "county": "Crosby County", "rate": 5.1},
    {"id": 48109, "state": "Texas", "county": "Culberson County", "rate": 4.1},
    {"id": 48111, "state": "Texas", "county": "Dallam County", "rate": 2.5},
    {"id": 48113, "state": "Texas", "county": "Dallas County", "rate": 4.2},
    {"id": 48115, "state": "Texas", "county": "Dawson County", "rate": 6},
    {"id": 48117, "state": "Texas", "county": "Deaf Smith County", "rate": 3.5},
    {"id": 48119, "state": "Texas", "county": "Delta County", "rate": 4.3},
    {"id": 48121, "state": "Texas", "county": "Denton County", "rate": 3.7},
    {"id": 48123, "state": "Texas", "county": "DeWitt County", "rate": 6},
    {"id": 48125, "state": "Texas", "county": "Dickens County", "rate": 5.7},
    {"id": 48127, "state": "Texas", "county": "Dimmit County", "rate": 7.4},
    {"id": 48129, "state": "Texas", "county": "Donley County", "rate": 5.3},
    {"id": 48131, "state": "Texas", "county": "Duval County", "rate": 12.1},
    {"id": 48133, "state": "Texas", "county": "Eastland County", "rate": 5.7},
    {"id": 48135, "state": "Texas", "county": "Ector County", "rate": 6.8},
    {"id": 48137, "state": "Texas", "county": "Edwards County", "rate": 5.2},
    {"id": 48139, "state": "Texas", "county": "Ellis County", "rate": 4.1},
    {"id": 48141, "state": "Texas", "county": "El Paso County", "rate": 5.4},
    {"id": 48143, "state": "Texas", "county": "Erath County", "rate": 4.7},
    {"id": 48145, "state": "Texas", "county": "Falls County", "rate": 4.6},
    {"id": 48147, "state": "Texas", "county": "Fannin County", "rate": 4.1},
    {"id": 48149, "state": "Texas", "county": "Fayette County", "rate": 4.1},
    {"id": 48151, "state": "Texas", "county": "Fisher County", "rate": 4.7},
    {"id": 48153, "state": "Texas", "county": "Floyd County", "rate": 5.5},
    {"id": 48155, "state": "Texas", "county": "Foard County", "rate": 4.6},
    {"id": 48157, "state": "Texas", "county": "Fort Bend County", "rate": 5.7},
    {"id": 48159, "state": "Texas", "county": "Franklin County", "rate": 5.5},
    {"id": 48161, "state": "Texas", "county": "Freestone County", "rate": 7.2},
    {"id": 48163, "state": "Texas", "county": "Frio County", "rate": 5.4},
    {"id": 48165, "state": "Texas", "county": "Gaines County", "rate": 3.7},
    {"id": 48167, "state": "Texas", "county": "Galveston County", "rate": 5.9},
    {"id": 48169, "state": "Texas", "county": "Garza County", "rate": 3.8},
    {"id": 48171, "state": "Texas", "county": "Gillespie County", "rate": 3.1},
    {"id": 48173, "state": "Texas", "county": "Glasscock County", "rate": 4.3},
    {"id": 48175, "state": "Texas", "county": "Goliad County", "rate": 5.7},
    {"id": 48177, "state": "Texas", "county": "Gonzales County", "rate": 4.6},
    {"id": 48179, "state": "Texas", "county": "Gray County", "rate": 7.3},
    {"id": 48181, "state": "Texas", "county": "Grayson County", "rate": 4},
    {"id": 48183, "state": "Texas", "county": "Gregg County", "rate": 6.8},
    {"id": 48185, "state": "Texas", "county": "Grimes County", "rate": 7.6},
    {"id": 48187, "state": "Texas", "county": "Guadalupe County", "rate": 4},
    {"id": 48189, "state": "Texas", "county": "Hale County", "rate": 6.4},
    {"id": 48191, "state": "Texas", "county": "Hall County", "rate": 8.2},
    {"id": 48193, "state": "Texas", "county": "Hamilton County", "rate": 5.3},
    {"id": 48195, "state": "Texas", "county": "Hansford County", "rate": 3.2},
    {"id": 48197, "state": "Texas", "county": "Hardeman County", "rate": 4.2},
    {"id": 48199, "state": "Texas", "county": "Hardin County", "rate": 6.8},
    {"id": 48201, "state": "Texas", "county": "Harris County", "rate": 5.8},
    {"id": 48203, "state": "Texas", "county": "Harrison County", "rate": 6.7},
    {"id": 48205, "state": "Texas", "county": "Hartley County", "rate": 2.1},
    {"id": 48207, "state": "Texas", "county": "Haskell County", "rate": 5.1},
    {"id": 48209, "state": "Texas", "county": "Hays County", "rate": 3.7},
    {"id": 48211, "state": "Texas", "county": "Hemphill County", "rate": 3.8},
    {"id": 48213, "state": "Texas", "county": "Henderson County", "rate": 5.3},
    {"id": 48215, "state": "Texas", "county": "Hidalgo County", "rate": 8.4},
    {"id": 48217, "state": "Texas", "county": "Hill County", "rate": 4.8},
    {"id": 48219, "state": "Texas", "county": "Hockley County", "rate": 5.1},
    {"id": 48221, "state": "Texas", "county": "Hood County", "rate": 5},
    {"id": 48223, "state": "Texas", "county": "Hopkins County", "rate": 4.5},
    {"id": 48225, "state": "Texas", "county": "Houston County", "rate": 5},
    {"id": 48227, "state": "Texas", "county": "Howard County", "rate": 6.1},
    {"id": 48229, "state": "Texas", "county": "Hudspeth County", "rate": 7},
    {"id": 48231, "state": "Texas", "county": "Hunt County", "rate": 4.5},
    {"id": 48233, "state": "Texas", "county": "Hutchinson County", "rate": 6.7},
    {"id": 48235, "state": "Texas", "county": "Irion County", "rate": 3.3},
    {"id": 48237, "state": "Texas", "county": "Jack County", "rate": 5.2},
    {"id": 48239, "state": "Texas", "county": "Jackson County", "rate": 5.3},
    {"id": 48241, "state": "Texas", "county": "Jasper County", "rate": 8.8},
    {"id": 48243, "state": "Texas", "county": "Jeff Davis County", "rate": 2.9},
    {"id": 48245, "state": "Texas", "county": "Jefferson County", "rate": 7.6},
    {"id": 48247, "state": "Texas", "county": "Jim Hogg County", "rate": 9.9},
    {"id": 48249, "state": "Texas", "county": "Jim Wells County", "rate": 11.3},
    {"id": 48251, "state": "Texas", "county": "Johnson County", "rate": 4.7},
    {"id": 48253, "state": "Texas", "county": "Jones County", "rate": 6.2},
    {"id": 48255, "state": "Texas", "county": "Karnes County", "rate": 5.3},
    {"id": 48257, "state": "Texas", "county": "Kaufman County", "rate": 4},
    {"id": 48259, "state": "Texas", "county": "Kendall County", "rate": 3.6},
    {"id": 48261, "state": "Texas", "county": "Kenedy County", "rate": 5.4},
    {"id": 48263, "state": "Texas", "county": "Kent County", "rate": 2.9},
    {"id": 48265, "state": "Texas", "county": "Kerr County", "rate": 3.9},
    {"id": 48267, "state": "Texas", "county": "Kimble County", "rate": 3.9},
    {"id": 48269, "state": "Texas", "county": "King County", "rate": 3.6},
    {"id": 48271, "state": "Texas", "county": "Kinney County", "rate": 6.6},
    {"id": 48273, "state": "Texas", "county": "Kleberg County", "rate": 7.9},
    {"id": 48275, "state": "Texas", "county": "Knox County", "rate": 5.3},
    {"id": 48277, "state": "Texas", "county": "Lamar County", "rate": 5.4},
    {"id": 48279, "state": "Texas", "county": "Lamb County", "rate": 7.1},
    {"id": 48281, "state": "Texas", "county": "Lampasas County", "rate": 4.6},
    {"id": 48283, "state": "Texas", "county": "La Salle County", "rate": 5.7},
    {"id": 48285, "state": "Texas", "county": "Lavaca County", "rate": 4.7},
    {"id": 48287, "state": "Texas", "county": "Lee County", "rate": 4.2},
    {"id": 48289, "state": "Texas", "county": "Leon County", "rate": 7.3},
    {"id": 48291, "state": "Texas", "county": "Liberty County", "rate": 8.5},
    {"id": 48293, "state": "Texas", "county": "Limestone County", "rate": 6.2},
    {"id": 48295, "state": "Texas", "county": "Lipscomb County", "rate": 5.1},
    {"id": 48297, "state": "Texas", "county": "Live Oak County", "rate": 5.5},
    {"id": 48299, "state": "Texas", "county": "Llano County", "rate": 4.3},
    {"id": 48301, "state": "Texas", "county": "Loving County", "rate": 4.4},
    {"id": 48303, "state": "Texas", "county": "Lubbock County", "rate": 3.7},
    {"id": 48305, "state": "Texas", "county": "Lynn County", "rate": 4.5},
    {"id": 48307, "state": "Texas", "county": "McCulloch County", "rate": 5.1},
    {"id": 48309, "state": "Texas", "county": "McLennan County", "rate": 4.4},
    {"id": 48311, "state": "Texas", "county": "McMullen County", "rate": 2.2},
    {"id": 48313, "state": "Texas", "county": "Madison County", "rate": 4.8},
    {"id": 48315, "state": "Texas", "county": "Marion County", "rate": 7.7},
    {"id": 48317, "state": "Texas", "county": "Martin County", "rate": 4.8},
    {"id": 48319, "state": "Texas", "county": "Mason County", "rate": 3.7},
    {"id": 48321, "state": "Texas", "county": "Matagorda County", "rate": 7.6},
    {"id": 48323, "state": "Texas", "county": "Maverick County", "rate": 11.4},
    {"id": 48325, "state": "Texas", "county": "Medina County", "rate": 4.8},
    {"id": 48327, "state": "Texas", "county": "Menard County", "rate": 5},
    {"id": 48329, "state": "Texas", "county": "Midland County", "rate": 4.6},
    {"id": 48331, "state": "Texas", "county": "Milam County", "rate": 5.6},
    {"id": 48333, "state": "Texas", "county": "Mills County", "rate": 4.6},
    {"id": 48335, "state": "Texas", "county": "Mitchell County", "rate": 7.3},
    {"id": 48337, "state": "Texas", "county": "Montague County", "rate": 5.1},
    {"id": 48339, "state": "Texas", "county": "Montgomery County", "rate": 5.5},
    {"id": 48341, "state": "Texas", "county": "Moore County", "rate": 3.2},
    {"id": 48343, "state": "Texas", "county": "Morris County", "rate": 13.1},
    {"id": 48345, "state": "Texas", "county": "Motley County", "rate": 3.9},
    {
        "id": 48347,
        "state": "Texas",
        "county": "Nacogdoches County",
        "rate": 5.5,
    },
    {"id": 48349, "state": "Texas", "county": "Navarro County", "rate": 4.6},
    {"id": 48351, "state": "Texas", "county": "Newton County", "rate": 7.9},
    {"id": 48353, "state": "Texas", "county": "Nolan County", "rate": 5.1},
    {"id": 48355, "state": "Texas", "county": "Nueces County", "rate": 5.9},
    {"id": 48357, "state": "Texas", "county": "Ochiltree County", "rate": 5.4},
    {"id": 48359, "state": "Texas", "county": "Oldham County", "rate": 3.4},
    {"id": 48361, "state": "Texas", "county": "Orange County", "rate": 7.5},
    {"id": 48363, "state": "Texas", "county": "Palo Pinto County", "rate": 5.9},
    {"id": 48365, "state": "Texas", "county": "Panola County", "rate": 7.5},
    {"id": 48367, "state": "Texas", "county": "Parker County", "rate": 4.3},
    {"id": 48369, "state": "Texas", "county": "Parmer County", "rate": 2.8},
    {"id": 48371, "state": "Texas", "county": "Pecos County", "rate": 5.9},
    {"id": 48373, "state": "Texas", "county": "Polk County", "rate": 7.1},
    {"id": 48375, "state": "Texas", "county": "Potter County", "rate": 3.6},
    {"id": 48377, "state": "Texas", "county": "Presidio County", "rate": 11.9},
    {"id": 48379, "state": "Texas", "county": "Rains County", "rate": 4.5},
    {"id": 48381, "state": "Texas", "county": "Randall County", "rate": 3.2},
    {"id": 48383, "state": "Texas", "county": "Reagan County", "rate": 7.7},
    {"id": 48385, "state": "Texas", "county": "Real County", "rate": 5.3},
    {"id": 48387, "state": "Texas", "county": "Red River County", "rate": 6.6},
    {"id": 48389, "state": "Texas", "county": "Reeves County", "rate": 6.1},
    {"id": 48391, "state": "Texas", "county": "Refugio County", "rate": 7.4},
    {"id": 48393, "state": "Texas", "county": "Roberts County", "rate": 4.9},
    {"id": 48395, "state": "Texas", "county": "Robertson County", "rate": 5.5},
    {"id": 48397, "state": "Texas", "county": "Rockwall County", "rate": 3.9},
    {"id": 48399, "state": "Texas", "county": "Runnels County", "rate": 4.8},
    {"id": 48401, "state": "Texas", "county": "Rusk County", "rate": 6.4},
    {"id": 48403, "state": "Texas", "county": "Sabine County", "rate": 9.8},
    {
        "id": 48405,
        "state": "Texas",
        "county": "San Augustine County",
        "rate": 9.6,
    },
    {
        "id": 48407,
        "state": "Texas",
        "county": "San Jacinto County",
        "rate": 6.9,
    },
    {
        "id": 48409,
        "state": "Texas",
        "county": "San Patricio County",
        "rate": 7.8,
    },
    {"id": 48411, "state": "Texas", "county": "San Saba County", "rate": 3.9},
    {"id": 48413, "state": "Texas", "county": "Schleicher County", "rate": 5.9},
    {"id": 48415, "state": "Texas", "county": "Scurry County", "rate": 6.3},
    {"id": 48417, "state": "Texas", "county": "Shackelford County", "rate": 4},
    {"id": 48419, "state": "Texas", "county": "Shelby County", "rate": 6.5},
    {"id": 48421, "state": "Texas", "county": "Sherman County", "rate": 3.4},
    {"id": 48423, "state": "Texas", "county": "Smith County", "rate": 5.1},
    {"id": 48425, "state": "Texas", "county": "Somervell County", "rate": 5.1},
    {"id": 48427, "state": "Texas", "county": "Starr County", "rate": 14.1},
    {"id": 48429, "state": "Texas", "county": "Stephens County", "rate": 5.9},
    {"id": 48431, "state": "Texas", "county": "Sterling County", "rate": 3.8},
    {"id": 48433, "state": "Texas", "county": "Stonewall County", "rate": 4.9},
    {"id": 48435, "state": "Texas", "county": "Sutton County", "rate": 8.5},
    {"id": 48437, "state": "Texas", "county": "Swisher County", "rate": 4.7},
    {"id": 48439, "state": "Texas", "county": "Tarrant County", "rate": 4.2},
    {"id": 48441, "state": "Texas", "county": "Taylor County", "rate": 4},
    {"id": 48443, "state": "Texas", "county": "Terrell County", "rate": 4.1},
    {"id": 48445, "state": "Texas", "county": "Terry County", "rate": 5.5},
    {"id": 48447, "state": "Texas", "county": "Throckmorton County", "rate": 4},
    {"id": 48449, "state": "Texas", "county": "Titus County", "rate": 7.2},
    {"id": 48451, "state": "Texas", "county": "Tom Green County", "rate": 4.7},
    {"id": 48453, "state": "Texas", "county": "Travis County", "rate": 3.4},
    {"id": 48455, "state": "Texas", "county": "Trinity County", "rate": 6.7},
    {"id": 48457, "state": "Texas", "county": "Tyler County", "rate": 8.2},
    {"id": 48459, "state": "Texas", "county": "Upshur County", "rate": 7.3},
    {"id": 48461, "state": "Texas", "county": "Upton County", "rate": 5},
    {"id": 48463, "state": "Texas", "county": "Uvalde County", "rate": 5.6},
    {"id": 48465, "state": "Texas", "county": "Val Verde County", "rate": 6.7},
    {"id": 48467, "state": "Texas", "county": "Van Zandt County", "rate": 4.6},
    {"id": 48469, "state": "Texas", "county": "Victoria County", "rate": 5.7},
    {"id": 48471, "state": "Texas", "county": "Walker County", "rate": 6.2},
    {"id": 48473, "state": "Texas", "county": "Waller County", "rate": 6.4},
    {"id": 48475, "state": "Texas", "county": "Ward County", "rate": 5.8},
    {"id": 48477, "state": "Texas", "county": "Washington County", "rate": 5.6},
    {"id": 48479, "state": "Texas", "county": "Webb County", "rate": 5.2},
    {"id": 48481, "state": "Texas", "county": "Wharton County", "rate": 5.4},
    {"id": 48483, "state": "Texas", "county": "Wheeler County", "rate": 4.4},
    {"id": 48485, "state": "Texas", "county": "Wichita County", "rate": 4.6},
    {"id": 48487, "state": "Texas", "county": "Wilbarger County", "rate": 5.4},
    {"id": 48489, "state": "Texas", "county": "Willacy County", "rate": 13.3},
    {"id": 48491, "state": "Texas", "county": "Williamson County", "rate": 3.6},
    {"id": 48493, "state": "Texas", "county": "Wilson County", "rate": 4.2},
    {"id": 48495, "state": "Texas", "county": "Winkler County", "rate": 8.5},
    {"id": 48497, "state": "Texas", "county": "Wise County", "rate": 5.1},
    {"id": 48499, "state": "Texas", "county": "Wood County", "rate": 5.8},
    {"id": 48501, "state": "Texas", "county": "Yoakum County", "rate": 5},
    {"id": 48503, "state": "Texas", "county": "Young County", "rate": 4.8},
    {"id": 48505, "state": "Texas", "county": "Zapata County", "rate": 11.2},
    {"id": 48507, "state": "Texas", "county": "Zavala County", "rate": 17.3},
    {"id": 49001, "state": "Utah", "county": "Beaver County", "rate": 6.1},
    {"id": 49003, "state": "Utah", "county": "Box Elder County", "rate": 3.3},
    {"id": 49005, "state": "Utah", "county": "Cache County", "rate": 3},
    {"id": 49007, "state": "Utah", "county": "Carbon County", "rate": 6.1},
    {"id": 49009, "state": "Utah", "county": "Daggett County", "rate": 3.4},
    {"id": 49011, "state": "Utah", "county": "Davis County", "rate": 3.1},
    {"id": 49013, "state": "Utah", "county": "Duchesne County", "rate": 8.6},
    {"id": 49015, "state": "Utah", "county": "Emery County", "rate": 5.9},
    {"id": 49017, "state": "Utah", "county": "Garfield County", "rate": 5.3},
    {"id": 49019, "state": "Utah", "county": "Grand County", "rate": 3.5},
    {"id": 49021, "state": "Utah", "county": "Iron County", "rate": 4.6},
    {"id": 49023, "state": "Utah", "county": "Juab County", "rate": 3.4},
    {"id": 49025, "state": "Utah", "county": "Kane County", "rate": 2.8},
    {"id": 49027, "state": "Utah", "county": "Millard County", "rate": 3.2},
    {"id": 49029, "state": "Utah", "county": "Morgan County", "rate": 2.8},
    {"id": 49031, "state": "Utah", "county": "Piute County", "rate": 4.8},
    {"id": 49033, "state": "Utah", "county": "Rich County", "rate": 2.2},
    {"id": 49035, "state": "Utah", "county": "Salt Lake County", "rate": 3.2},
    {"id": 49037, "state": "Utah", "county": "San Juan County", "rate": 7.3},
    {"id": 49039, "state": "Utah", "county": "Sanpete County", "rate": 4},
    {"id": 49041, "state": "Utah", "county": "Sevier County", "rate": 4.2},
    {"id": 49043, "state": "Utah", "county": "Summit County", "rate": 2.9},
    {"id": 49045, "state": "Utah", "county": "Tooele County", "rate": 3.9},
    {"id": 49047, "state": "Utah", "county": "Uintah County", "rate": 9.8},
    {"id": 49049, "state": "Utah", "county": "Utah County", "rate": 3.1},
    {"id": 49051, "state": "Utah", "county": "Wasatch County", "rate": 3},
    {"id": 49053, "state": "Utah", "county": "Washington County", "rate": 3.5},
    {"id": 49055, "state": "Utah", "county": "Wayne County", "rate": 5.3},
    {"id": 49057, "state": "Utah", "county": "Weber County", "rate": 3.7},
    {"id": 50001, "state": "Vermont", "county": "Addison County", "rate": 3.1},
    {"id": 50003, "state": "Vermont", "county": "Bennington County", "rate": 4},
    {
        "id": 50005,
        "state": "Vermont",
        "county": "Caledonia County",
        "rate": 4.2,
    },
    {
        "id": 50007,
        "state": "Vermont",
        "county": "Chittenden County",
        "rate": 2.7,
    },
    {"id": 50009, "state": "Vermont", "county": "Essex County", "rate": 5.2},
    {"id": 50011, "state": "Vermont", "county": "Franklin County", "rate": 3.2},
    {
        "id": 50013,
        "state": "Vermont",
        "county": "Grand Isle County",
        "rate": 3.7,
    },
    {"id": 50015, "state": "Vermont", "county": "Lamoille County", "rate": 3.7},
    {"id": 50017, "state": "Vermont", "county": "Orange County", "rate": 2.9},
    {"id": 50019, "state": "Vermont", "county": "Orleans County", "rate": 4.5},
    {"id": 50021, "state": "Vermont", "county": "Rutland County", "rate": 4},
    {
        "id": 50023,
        "state": "Vermont",
        "county": "Washington County",
        "rate": 3.1,
    },
    {"id": 50025, "state": "Vermont", "county": "Windham County", "rate": 3.3},
    {"id": 50027, "state": "Vermont", "county": "Windsor County", "rate": 3.1},
    {
        "id": 51001,
        "state": "Virginia",
        "county": "Accomack County",
        "rate": 4.3,
    },
    {
        "id": 51003,
        "state": "Virginia",
        "county": "Albemarle County",
        "rate": 3.6,
    },
    {
        "id": 51005,
        "state": "Virginia",
        "county": "Alleghany County",
        "rate": 4.9,
    },
    {"id": 51007, "state": "Virginia", "county": "Amelia County", "rate": 4.3},
    {"id": 51009, "state": "Virginia", "county": "Amherst County", "rate": 4.6},
    {
        "id": 51011,
        "state": "Virginia",
        "county": "Appomattox County",
        "rate": 4.7,
    },
    {
        "id": 51013,
        "state": "Virginia",
        "county": "Arlington County",
        "rate": 2.6,
    },
    {"id": 51015, "state": "Virginia", "county": "Augusta County", "rate": 3.6},
    {"id": 51017, "state": "Virginia", "county": "Bath County", "rate": 3.6},
    {"id": 51019, "state": "Virginia", "county": "Bedford County", "rate": 4.1},
    {"id": 51021, "state": "Virginia", "county": "Bland County", "rate": 6.4},
    {
        "id": 51023,
        "state": "Virginia",
        "county": "Botetourt County",
        "rate": 3.5,
    },
    {
        "id": 51025,
        "state": "Virginia",
        "county": "Brunswick County",
        "rate": 5.9,
    },
    {
        "id": 51027,
        "state": "Virginia",
        "county": "Buchanan County",
        "rate": 10.9,
    },
    {
        "id": 51029,
        "state": "Virginia",
        "county": "Buckingham County",
        "rate": 5.1,
    },
    {
        "id": 51031,
        "state": "Virginia",
        "county": "Campbell County",
        "rate": 4.5,
    },
    {
        "id": 51033,
        "state": "Virginia",
        "county": "Caroline County",
        "rate": 4.3,
    },
    {"id": 51035, "state": "Virginia", "county": "Carroll County", "rate": 5.9},
    {
        "id": 51036,
        "state": "Virginia",
        "county": "Charles City County",
        "rate": 4.4,
    },
    {
        "id": 51037,
        "state": "Virginia",
        "county": "Charlotte County",
        "rate": 5.2,
    },
    {
        "id": 51041,
        "state": "Virginia",
        "county": "Chesterfield County",
        "rate": 3.8,
    },
    {"id": 51043, "state": "Virginia", "county": "Clarke County", "rate": 3.3},
    {"id": 51045, "state": "Virginia", "county": "Craig County", "rate": 4.4},
    {
        "id": 51047,
        "state": "Virginia",
        "county": "Culpeper County",
        "rate": 3.8,
    },
    {
        "id": 51049,
        "state": "Virginia",
        "county": "Cumberland County",
        "rate": 4.6,
    },
    {
        "id": 51051,
        "state": "Virginia",
        "county": "Dickenson County",
        "rate": 10,
    },
    {
        "id": 51053,
        "state": "Virginia",
        "county": "Dinwiddie County",
        "rate": 4.8,
    },
    {"id": 51057, "state": "Virginia", "county": "Essex County", "rate": 4.7},
    {"id": 51059, "state": "Virginia", "county": "Fairfax County", "rate": 3.2},
    {
        "id": 51061,
        "state": "Virginia",
        "county": "Fauquier County",
        "rate": 3.4,
    },
    {"id": 51063, "state": "Virginia", "county": "Floyd County", "rate": 4.4},
    {
        "id": 51065,
        "state": "Virginia",
        "county": "Fluvanna County",
        "rate": 3.4,
    },
    {
        "id": 51067,
        "state": "Virginia",
        "county": "Franklin County",
        "rate": 4.3,
    },
    {
        "id": 51069,
        "state": "Virginia",
        "county": "Frederick County",
        "rate": 3.4,
    },
    {"id": 51071, "state": "Virginia", "county": "Giles County", "rate": 6.9},
    {
        "id": 51073,
        "state": "Virginia",
        "county": "Gloucester County",
        "rate": 3.6,
    },
    {
        "id": 51075,
        "state": "Virginia",
        "county": "Goochland County",
        "rate": 3.6,
    },
    {"id": 51077, "state": "Virginia", "county": "Grayson County", "rate": 5.2},
    {"id": 51079, "state": "Virginia", "county": "Greene County", "rate": 3.2},
    {
        "id": 51081,
        "state": "Virginia",
        "county": "Greensville County",
        "rate": 5.1,
    },
    {"id": 51083, "state": "Virginia", "county": "Halifax County", "rate": 6.2},
    {"id": 51085, "state": "Virginia", "county": "Hanover County", "rate": 3.5},
    {"id": 51087, "state": "Virginia", "county": "Henrico County", "rate": 3.9},
    {"id": 51089, "state": "Virginia", "county": "Henry County", "rate": 5.4},
    {"id": 51091, "state": "Virginia", "county": "Highland County", "rate": 3},
    {
        "id": 51093,
        "state": "Virginia",
        "county": "Isle of Wight County",
        "rate": 4.4,
    },
    {
        "id": 51095,
        "state": "Virginia",
        "county": "James City County",
        "rate": 3.8,
    },
    {
        "id": 51097,
        "state": "Virginia",
        "county": "King and Queen County",
        "rate": 4.2,
    },
    {
        "id": 51099,
        "state": "Virginia",
        "county": "King George County",
        "rate": 4.1,
    },
    {
        "id": 51101,
        "state": "Virginia",
        "county": "King William County",
        "rate": 3.7,
    },
    {
        "id": 51103,
        "state": "Virginia",
        "county": "Lancaster County",
        "rate": 4.4,
    },
    {"id": 51105, "state": "Virginia", "county": "Lee County", "rate": 6.9},
    {"id": 51107, "state": "Virginia", "county": "Loudoun County", "rate": 3.2},
    {"id": 51109, "state": "Virginia", "county": "Louisa County", "rate": 3.7},
    {
        "id": 51111,
        "state": "Virginia",
        "county": "Lunenburg County",
        "rate": 4.6,
    },
    {"id": 51113, "state": "Virginia", "county": "Madison County", "rate": 3.1},
    {"id": 51115, "state": "Virginia", "county": "Mathews County", "rate": 3.8},
    {
        "id": 51117,
        "state": "Virginia",
        "county": "Mecklenburg County",
        "rate": 5.6,
    },
    {
        "id": 51119,
        "state": "Virginia",
        "county": "Middlesex County",
        "rate": 3.7,
    },
    {
        "id": 51121,
        "state": "Virginia",
        "county": "Montgomery County",
        "rate": 4.5,
    },
    {"id": 51125, "state": "Virginia", "county": "Nelson County", "rate": 3.5},
    {
        "id": 51127,
        "state": "Virginia",
        "county": "New Kent County",
        "rate": 3.3,
    },
    {
        "id": 51131,
        "state": "Virginia",
        "county": "Northampton County",
        "rate": 5,
    },
    {
        "id": 51133,
        "state": "Virginia",
        "county": "Northumberland County",
        "rate": 4.4,
    },
    {"id": 51135, "state": "Virginia", "county": "Nottoway County", "rate": 4},
    {"id": 51137, "state": "Virginia", "county": "Orange County", "rate": 3.9},
    {"id": 51139, "state": "Virginia", "county": "Page County", "rate": 4.5},
    {"id": 51141, "state": "Virginia", "county": "Patrick County", "rate": 4.5},
    {
        "id": 51143,
        "state": "Virginia",
        "county": "Pittsylvania County",
        "rate": 4.9,
    },
    {
        "id": 51145,
        "state": "Virginia",
        "county": "Powhatan County",
        "rate": 3.6,
    },
    {
        "id": 51147,
        "state": "Virginia",
        "county": "Prince Edward County",
        "rate": 5.6,
    },
    {
        "id": 51149,
        "state": "Virginia",
        "county": "Prince George County",
        "rate": 4.9,
    },
    {
        "id": 51153,
        "state": "Virginia",
        "county": "Prince William County",
        "rate": 3.6,
    },
    {"id": 51155, "state": "Virginia", "county": "Pulaski County", "rate": 10},
    {
        "id": 51157,
        "state": "Virginia",
        "county": "Rappahannock County",
        "rate": 3.5,
    },
    {
        "id": 51159,
        "state": "Virginia",
        "county": "Richmond County",
        "rate": 3.3,
    },
    {"id": 51161, "state": "Virginia", "county": "Roanoke County", "rate": 3.6},
    {
        "id": 51163,
        "state": "Virginia",
        "county": "Rockbridge County",
        "rate": 4.3,
    },
    {
        "id": 51165,
        "state": "Virginia",
        "county": "Rockingham County",
        "rate": 3.6,
    },
    {"id": 51167, "state": "Virginia", "county": "Russell County", "rate": 6.5},
    {"id": 51169, "state": "Virginia", "county": "Scott County", "rate": 4.6},
    {
        "id": 51171,
        "state": "Virginia",
        "county": "Shenandoah County",
        "rate": 3.6,
    },
    {"id": 51173, "state": "Virginia", "county": "Smyth County", "rate": 5.6},
    {
        "id": 51175,
        "state": "Virginia",
        "county": "Southampton County",
        "rate": 3.8,
    },
    {
        "id": 51177,
        "state": "Virginia",
        "county": "Spotsylvania County",
        "rate": 4.1,
    },
    {"id": 51179, "state": "Virginia", "county": "Stafford County", "rate": 4},
    {"id": 51181, "state": "Virginia", "county": "Surry County", "rate": 5.6},
    {"id": 51183, "state": "Virginia", "county": "Sussex County", "rate": 5.9},
    {
        "id": 51185,
        "state": "Virginia",
        "county": "Tazewell County",
        "rate": 8.1,
    },
    {"id": 51187, "state": "Virginia", "county": "Warren County", "rate": 3.9},
    {
        "id": 51191,
        "state": "Virginia",
        "county": "Washington County",
        "rate": 4.6,
    },
    {
        "id": 51193,
        "state": "Virginia",
        "county": "Westmoreland County",
        "rate": 4.2,
    },
    {"id": 51195, "state": "Virginia", "county": "Wise County", "rate": 8.7},
    {"id": 51197, "state": "Virginia", "county": "Wythe County", "rate": 7},
    {"id": 51199, "state": "Virginia", "county": "York County", "rate": 4},
    {
        "id": 51510,
        "state": "Virginia",
        "county": "Alexandria city",
        "rate": 2.9,
    },
    {"id": 51520, "state": "Virginia", "county": "Bristol city", "rate": 5.2},
    {
        "id": 51530,
        "state": "Virginia",
        "county": "Buena Vista city",
        "rate": 5.1,
    },
    {
        "id": 51540,
        "state": "Virginia",
        "county": "Charlottesville city",
        "rate": 3.5,
    },
    {
        "id": 51550,
        "state": "Virginia",
        "county": "Chesapeake city",
        "rate": 4.4,
    },
    {
        "id": 51570,
        "state": "Virginia",
        "county": "Colonial Heights city",
        "rate": 4.5,
    },
    {"id": 51580, "state": "Virginia", "county": "Covington city", "rate": 5.8},
    {"id": 51590, "state": "Virginia", "county": "Danville city", "rate": 6.6},
    {"id": 51595, "state": "Virginia", "county": "Emporia city", "rate": 6.4},
    {"id": 51600, "state": "Virginia", "county": "Fairfax city", "rate": 3.1},
    {
        "id": 51610,
        "state": "Virginia",
        "county": "Falls Church city",
        "rate": 2.7,
    },
    {"id": 51620, "state": "Virginia", "county": "Franklin city", "rate": 5.9},
    {
        "id": 51630,
        "state": "Virginia",
        "county": "Fredericksburg city",
        "rate": 4.7,
    },
    {"id": 51640, "state": "Virginia", "county": "Galax city", "rate": 5.1},
    {"id": 51650, "state": "Virginia", "county": "Hampton city", "rate": 5.7},
    {
        "id": 51660,
        "state": "Virginia",
        "county": "Harrisonburg city",
        "rate": 5.2,
    },
    {"id": 51670, "state": "Virginia", "county": "Hopewell city", "rate": 6.5},
    {"id": 51678, "state": "Virginia", "county": "Lexington city", "rate": 7.4},
    {"id": 51680, "state": "Virginia", "county": "Lynchburg city", "rate": 5.5},
    {"id": 51683, "state": "Virginia", "county": "Manassas city", "rate": 3.5},
    {
        "id": 51685,
        "state": "Virginia",
        "county": "Manassas Park city",
        "rate": 3.5,
    },
    {
        "id": 51690,
        "state": "Virginia",
        "county": "Martinsville city",
        "rate": 6.8,
    },
    {
        "id": 51700,
        "state": "Virginia",
        "county": "Newport News city",
        "rate": 5.1,
    },
    {"id": 51710, "state": "Virginia", "county": "Norfolk city", "rate": 5.4},
    {"id": 51720, "state": "Virginia", "county": "Norton city", "rate": 7.2},
    {
        "id": 51730,
        "state": "Virginia",
        "county": "Petersburg city",
        "rate": 7.4,
    },
    {"id": 51735, "state": "Virginia", "county": "Poquoson city", "rate": 3.6},
    {
        "id": 51740,
        "state": "Virginia",
        "county": "Portsmouth city",
        "rate": 6.2,
    },
    {"id": 51750, "state": "Virginia", "county": "Radford city", "rate": 6.7},
    {"id": 51760, "state": "Virginia", "county": "Richmond city", "rate": 4.7},
    {"id": 51770, "state": "Virginia", "county": "Roanoke city", "rate": 4.4},
    {"id": 51775, "state": "Virginia", "county": "Salem city", "rate": 4},
    {"id": 51790, "state": "Virginia", "county": "Staunton city", "rate": 3.9},
    {"id": 51800, "state": "Virginia", "county": "Suffolk city", "rate": 4.7},
    {
        "id": 51810,
        "state": "Virginia",
        "county": "Virginia Beach city",
        "rate": 4,
    },
    {
        "id": 51820,
        "state": "Virginia",
        "county": "Waynesboro city",
        "rate": 4.1,
    },
    {
        "id": 51830,
        "state": "Virginia",
        "county": "Williamsburg city",
        "rate": 5.9,
    },
    {
        "id": 51840,
        "state": "Virginia",
        "county": "Winchester city",
        "rate": 3.9,
    },
    {"id": 53001, "state": "Washington", "county": "Adams County", "rate": 4.8},
    {
        "id": 53003,
        "state": "Washington",
        "county": "Asotin County",
        "rate": 4.7,
    },
    {
        "id": 53005,
        "state": "Washington",
        "county": "Benton County",
        "rate": 6.5,
    },
    {
        "id": 53007,
        "state": "Washington",
        "county": "Chelan County",
        "rate": 5.1,
    },
    {
        "id": 53009,
        "state": "Washington",
        "county": "Clallam County",
        "rate": 7.6,
    },
    {"id": 53011, "state": "Washington", "county": "Clark County", "rate": 6.6},
    {
        "id": 53013,
        "state": "Washington",
        "county": "Columbia County",
        "rate": 5.9,
    },
    {
        "id": 53015,
        "state": "Washington",
        "county": "Cowlitz County",
        "rate": 7.6,
    },
    {
        "id": 53017,
        "state": "Washington",
        "county": "Douglas County",
        "rate": 6.6,
    },
    {"id": 53019, "state": "Washington", "county": "Ferry County", "rate": 9.8},
    {
        "id": 53021,
        "state": "Washington",
        "county": "Franklin County",
        "rate": 6.5,
    },
    {
        "id": 53023,
        "state": "Washington",
        "county": "Garfield County",
        "rate": 5.3,
    },
    {"id": 53025, "state": "Washington", "county": "Grant County", "rate": 6.6},
    {
        "id": 53027,
        "state": "Washington",
        "county": "Grays Harbor County",
        "rate": 8.7,
    },
    {"id": 53029, "state": "Washington", "county": "Island County", "rate": 6},
    {
        "id": 53031,
        "state": "Washington",
        "county": "Jefferson County",
        "rate": 7.2,
    },
    {"id": 53033, "state": "Washington", "county": "King County", "rate": 3.9},
    {
        "id": 53035,
        "state": "Washington",
        "county": "Kitsap County",
        "rate": 5.9,
    },
    {
        "id": 53037,
        "state": "Washington",
        "county": "Kittitas County",
        "rate": 5.8,
    },
    {
        "id": 53039,
        "state": "Washington",
        "county": "Klickitat County",
        "rate": 6.5,
    },
    {"id": 53041, "state": "Washington", "county": "Lewis County", "rate": 8.1},
    {
        "id": 53043,
        "state": "Washington",
        "county": "Lincoln County",
        "rate": 4.1,
    },
    {"id": 53045, "state": "Washington", "county": "Mason County", "rate": 8.3},
    {
        "id": 53047,
        "state": "Washington",
        "county": "Okanogan County",
        "rate": 6.1,
    },
    {
        "id": 53049,
        "state": "Washington",
        "county": "Pacific County",
        "rate": 7.7,
    },
    {
        "id": 53051,
        "state": "Washington",
        "county": "Pend Oreille County",
        "rate": 8.9,
    },
    {
        "id": 53053,
        "state": "Washington",
        "county": "Pierce County",
        "rate": 6.5,
    },
    {
        "id": 53055,
        "state": "Washington",
        "county": "San Juan County",
        "rate": 3.9,
    },
    {
        "id": 53057,
        "state": "Washington",
        "county": "Skagit County",
        "rate": 6.7,
    },
    {
        "id": 53059,
        "state": "Washington",
        "county": "Skamania County",
        "rate": 6.7,
    },
    {
        "id": 53061,
        "state": "Washington",
        "county": "Snohomish County",
        "rate": 4.3,
    },
    {
        "id": 53063,
        "state": "Washington",
        "county": "Spokane County",
        "rate": 6.3,
    },
    {
        "id": 53065,
        "state": "Washington",
        "county": "Stevens County",
        "rate": 7.9,
    },
    {
        "id": 53067,
        "state": "Washington",
        "county": "Thurston County",
        "rate": 5.9,
    },
    {
        "id": 53069,
        "state": "Washington",
        "county": "Wahkiakum County",
        "rate": 8.4,
    },
    {
        "id": 53071,
        "state": "Washington",
        "county": "Walla Walla County",
        "rate": 5.6,
    },
    {
        "id": 53073,
        "state": "Washington",
        "county": "Whatcom County",
        "rate": 6.3,
    },
    {
        "id": 53075,
        "state": "Washington",
        "county": "Whitman County",
        "rate": 5.5,
    },
    {
        "id": 53077,
        "state": "Washington",
        "county": "Yakima County",
        "rate": 7.3,
    },
    {
        "id": 54001,
        "state": "West Virginia",
        "county": "Barbour County",
        "rate": 5.8,
    },
    {
        "id": 54003,
        "state": "West Virginia",
        "county": "Berkeley County",
        "rate": 4,
    },
    {
        "id": 54005,
        "state": "West Virginia",
        "county": "Boone County",
        "rate": 8.5,
    },
    {
        "id": 54007,
        "state": "West Virginia",
        "county": "Braxton County",
        "rate": 6.9,
    },
    {
        "id": 54009,
        "state": "West Virginia",
        "county": "Brooke County",
        "rate": 6.5,
    },
    {
        "id": 54011,
        "state": "West Virginia",
        "county": "Cabell County",
        "rate": 4.8,
    },
    {
        "id": 54013,
        "state": "West Virginia",
        "county": "Calhoun County",
        "rate": 8.5,
    },
    {
        "id": 54015,
        "state": "West Virginia",
        "county": "Clay County",
        "rate": 9.1,
    },
    {
        "id": 54017,
        "state": "West Virginia",
        "county": "Doddridge County",
        "rate": 4.9,
    },
    {
        "id": 54019,
        "state": "West Virginia",
        "county": "Fayette County",
        "rate": 6.8,
    },
    {
        "id": 54021,
        "state": "West Virginia",
        "county": "Gilmer County",
        "rate": 6.8,
    },
    {
        "id": 54023,
        "state": "West Virginia",
        "county": "Grant County",
        "rate": 6.1,
    },
    {
        "id": 54025,
        "state": "West Virginia",
        "county": "Greenbrier County",
        "rate": 4.5,
    },
    {
        "id": 54027,
        "state": "West Virginia",
        "county": "Hampshire County",
        "rate": 4.1,
    },
    {
        "id": 54029,
        "state": "West Virginia",
        "county": "Hancock County",
        "rate": 6.5,
    },
    {
        "id": 54031,
        "state": "West Virginia",
        "county": "Hardy County",
        "rate": 5.3,
    },
    {
        "id": 54033,
        "state": "West Virginia",
        "county": "Harrison County",
        "rate": 5.4,
    },
    {
        "id": 54035,
        "state": "West Virginia",
        "county": "Jackson County",
        "rate": 5.9,
    },
    {
        "id": 54037,
        "state": "West Virginia",
        "county": "Jefferson County",
        "rate": 3.4,
    },
    {
        "id": 54039,
        "state": "West Virginia",
        "county": "Kanawha County",
        "rate": 5.2,
    },
    {
        "id": 54041,
        "state": "West Virginia",
        "county": "Lewis County",
        "rate": 7.1,
    },
    {
        "id": 54043,
        "state": "West Virginia",
        "county": "Lincoln County",
        "rate": 7.4,
    },
    {
        "id": 54045,
        "state": "West Virginia",
        "county": "Logan County",
        "rate": 10,
    },
    {
        "id": 54047,
        "state": "West Virginia",
        "county": "McDowell County",
        "rate": 13.1,
    },
    {
        "id": 54049,
        "state": "West Virginia",
        "county": "Marion County",
        "rate": 6,
    },
    {
        "id": 54051,
        "state": "West Virginia",
        "county": "Marshall County",
        "rate": 7,
    },
    {
        "id": 54053,
        "state": "West Virginia",
        "county": "Mason County",
        "rate": 7,
    },
    {
        "id": 54055,
        "state": "West Virginia",
        "county": "Mercer County",
        "rate": 6.7,
    },
    {
        "id": 54057,
        "state": "West Virginia",
        "county": "Mineral County",
        "rate": 6,
    },
    {
        "id": 54059,
        "state": "West Virginia",
        "county": "Mingo County",
        "rate": 11.7,
    },
    {
        "id": 54061,
        "state": "West Virginia",
        "county": "Monongalia County",
        "rate": 4.5,
    },
    {
        "id": 54063,
        "state": "West Virginia",
        "county": "Monroe County",
        "rate": 4.5,
    },
    {
        "id": 54065,
        "state": "West Virginia",
        "county": "Morgan County",
        "rate": 4.5,
    },
    {
        "id": 54067,
        "state": "West Virginia",
        "county": "Nicholas County",
        "rate": 8.5,
    },
    {
        "id": 54069,
        "state": "West Virginia",
        "county": "Ohio County",
        "rate": 5.3,
    },
    {
        "id": 54071,
        "state": "West Virginia",
        "county": "Pendleton County",
        "rate": 3.5,
    },
    {
        "id": 54073,
        "state": "West Virginia",
        "county": "Pleasants County",
        "rate": 7.5,
    },
    {
        "id": 54075,
        "state": "West Virginia",
        "county": "Pocahontas County",
        "rate": 5.1,
    },
    {
        "id": 54077,
        "state": "West Virginia",
        "county": "Preston County",
        "rate": 5.2,
    },
    {
        "id": 54079,
        "state": "West Virginia",
        "county": "Putnam County",
        "rate": 4.6,
    },
    {
        "id": 54081,
        "state": "West Virginia",
        "county": "Raleigh County",
        "rate": 6.3,
    },
    {
        "id": 54083,
        "state": "West Virginia",
        "county": "Randolph County",
        "rate": 5.3,
    },
    {
        "id": 54085,
        "state": "West Virginia",
        "county": "Ritchie County",
        "rate": 5.9,
    },
    {
        "id": 54087,
        "state": "West Virginia",
        "county": "Roane County",
        "rate": 8.5,
    },
    {
        "id": 54089,
        "state": "West Virginia",
        "county": "Summers County",
        "rate": 5.5,
    },
    {
        "id": 54091,
        "state": "West Virginia",
        "county": "Taylor County",
        "rate": 5.2,
    },
    {
        "id": 54093,
        "state": "West Virginia",
        "county": "Tucker County",
        "rate": 4.8,
    },
    {
        "id": 54095,
        "state": "West Virginia",
        "county": "Tyler County",
        "rate": 8.1,
    },
    {
        "id": 54097,
        "state": "West Virginia",
        "county": "Upshur County",
        "rate": 7,
    },
    {
        "id": 54099,
        "state": "West Virginia",
        "county": "Wayne County",
        "rate": 6.3,
    },
    {
        "id": 54101,
        "state": "West Virginia",
        "county": "Webster County",
        "rate": 8.2,
    },
    {
        "id": 54103,
        "state": "West Virginia",
        "county": "Wetzel County",
        "rate": 8.1,
    },
    {
        "id": 54105,
        "state": "West Virginia",
        "county": "Wirt County",
        "rate": 8.1,
    },
    {
        "id": 54107,
        "state": "West Virginia",
        "county": "Wood County",
        "rate": 5.7,
    },
    {
        "id": 54109,
        "state": "West Virginia",
        "county": "Wyoming County",
        "rate": 9.4,
    },
    {"id": 55001, "state": "Wisconsin", "county": "Adams County", "rate": 5.6},
    {
        "id": 55003,
        "state": "Wisconsin",
        "county": "Ashland County",
        "rate": 5.1,
    },
    {"id": 55005, "state": "Wisconsin", "county": "Barron County", "rate": 3.9},
    {
        "id": 55007,
        "state": "Wisconsin",
        "county": "Bayfield County",
        "rate": 5.8,
    },
    {"id": 55009, "state": "Wisconsin", "county": "Brown County", "rate": 3.6},
    {
        "id": 55011,
        "state": "Wisconsin",
        "county": "Buffalo County",
        "rate": 4.2,
    },
    {
        "id": 55013,
        "state": "Wisconsin",
        "county": "Burnett County",
        "rate": 4.9,
    },
    {
        "id": 55015,
        "state": "Wisconsin",
        "county": "Calumet County",
        "rate": 3.4,
    },
    {
        "id": 55017,
        "state": "Wisconsin",
        "county": "Chippewa County",
        "rate": 3.6,
    },
    {"id": 55019, "state": "Wisconsin", "county": "Clark County", "rate": 3.2},
    {
        "id": 55021,
        "state": "Wisconsin",
        "county": "Columbia County",
        "rate": 3.3,
    },
    {
        "id": 55023,
        "state": "Wisconsin",
        "county": "Crawford County",
        "rate": 4.3,
    },
    {"id": 55025, "state": "Wisconsin", "county": "Dane County", "rate": 2.8},
    {"id": 55027, "state": "Wisconsin", "county": "Dodge County", "rate": 3.7},
    {"id": 55029, "state": "Wisconsin", "county": "Door County", "rate": 3.4},
    {
        "id": 55031,
        "state": "Wisconsin",
        "county": "Douglas County",
        "rate": 5.2,
    },
    {"id": 55033, "state": "Wisconsin", "county": "Dunn County", "rate": 3.8},
    {
        "id": 55035,
        "state": "Wisconsin",
        "county": "Eau Claire County",
        "rate": 3.4,
    },
    {
        "id": 55037,
        "state": "Wisconsin",
        "county": "Florence County",
        "rate": 5.5,
    },
    {
        "id": 55039,
        "state": "Wisconsin",
        "county": "Fond du Lac County",
        "rate": 3.6,
    },
    {"id": 55041, "state": "Wisconsin", "county": "Forest County", "rate": 6.2},
    {"id": 55043, "state": "Wisconsin", "county": "Grant County", "rate": 3.7},
    {"id": 55045, "state": "Wisconsin", "county": "Green County", "rate": 3.1},
    {
        "id": 55047,
        "state": "Wisconsin",
        "county": "Green Lake County",
        "rate": 3.9,
    },
    {"id": 55049, "state": "Wisconsin", "county": "Iowa County", "rate": 3},
    {"id": 55051, "state": "Wisconsin", "county": "Iron County", "rate": 7},
    {
        "id": 55053,
        "state": "Wisconsin",
        "county": "Jackson County",
        "rate": 3.7,
    },
    {
        "id": 55055,
        "state": "Wisconsin",
        "county": "Jefferson County",
        "rate": 3.9,
    },
    {"id": 55057, "state": "Wisconsin", "county": "Juneau County", "rate": 4},
    {
        "id": 55059,
        "state": "Wisconsin",
        "county": "Kenosha County",
        "rate": 4.8,
    },
    {
        "id": 55061,
        "state": "Wisconsin",
        "county": "Kewaunee County",
        "rate": 3.1,
    },
    {
        "id": 55063,
        "state": "Wisconsin",
        "county": "La Crosse County",
        "rate": 3.7,
    },
    {
        "id": 55065,
        "state": "Wisconsin",
        "county": "Lafayette County",
        "rate": 2.9,
    },
    {
        "id": 55067,
        "state": "Wisconsin",
        "county": "Langlade County",
        "rate": 5.4,
    },
    {
        "id": 55069,
        "state": "Wisconsin",
        "county": "Lincoln County",
        "rate": 4.3,
    },
    {
        "id": 55071,
        "state": "Wisconsin",
        "county": "Manitowoc County",
        "rate": 4.3,
    },
    {
        "id": 55073,
        "state": "Wisconsin",
        "county": "Marathon County",
        "rate": 3.4,
    },
    {
        "id": 55075,
        "state": "Wisconsin",
        "county": "Marinette County",
        "rate": 5.2,
    },
    {
        "id": 55077,
        "state": "Wisconsin",
        "county": "Marquette County",
        "rate": 4.7,
    },
    {
        "id": 55078,
        "state": "Wisconsin",
        "county": "Menominee County",
        "rate": 8.6,
    },
    {
        "id": 55079,
        "state": "Wisconsin",
        "county": "Milwaukee County",
        "rate": 5.5,
    },
    {"id": 55081, "state": "Wisconsin", "county": "Monroe County", "rate": 3.5},
    {"id": 55083, "state": "Wisconsin", "county": "Oconto County", "rate": 3.9},
    {"id": 55085, "state": "Wisconsin", "county": "Oneida County", "rate": 4.3},
    {
        "id": 55087,
        "state": "Wisconsin",
        "county": "Outagamie County",
        "rate": 3.5,
    },
    {
        "id": 55089,
        "state": "Wisconsin",
        "county": "Ozaukee County",
        "rate": 3.4,
    },
    {"id": 55091, "state": "Wisconsin", "county": "Pepin County", "rate": 3.2},
    {"id": 55093, "state": "Wisconsin", "county": "Pierce County", "rate": 3.8},
    {"id": 55095, "state": "Wisconsin", "county": "Polk County", "rate": 3.8},
    {
        "id": 55097,
        "state": "Wisconsin",
        "county": "Portage County",
        "rate": 3.5,
    },
    {"id": 55099, "state": "Wisconsin", "county": "Price County", "rate": 3.8},
    {"id": 55101, "state": "Wisconsin", "county": "Racine County", "rate": 5.3},
    {
        "id": 55103,
        "state": "Wisconsin",
        "county": "Richland County",
        "rate": 3.4,
    },
    {"id": 55105, "state": "Wisconsin", "county": "Rock County", "rate": 4.3},
    {"id": 55107, "state": "Wisconsin", "county": "Rusk County", "rate": 4.5},
    {
        "id": 55109,
        "state": "Wisconsin",
        "county": "St. Croix County",
        "rate": 3.5,
    },
    {"id": 55111, "state": "Wisconsin", "county": "Sauk County", "rate": 3.1},
    {"id": 55113, "state": "Wisconsin", "county": "Sawyer County", "rate": 5.6},
    {
        "id": 55115,
        "state": "Wisconsin",
        "county": "Shawano County",
        "rate": 3.9,
    },
    {
        "id": 55117,
        "state": "Wisconsin",
        "county": "Sheboygan County",
        "rate": 3.3,
    },
    {"id": 55119, "state": "Wisconsin", "county": "Taylor County", "rate": 3.5},
    {
        "id": 55121,
        "state": "Wisconsin",
        "county": "Trempealeau County",
        "rate": 3.4,
    },
    {"id": 55123, "state": "Wisconsin", "county": "Vernon County", "rate": 3.1},
    {"id": 55125, "state": "Wisconsin", "county": "Vilas County", "rate": 4.5},
    {"id": 55127, "state": "Wisconsin", "county": "Walworth County", "rate": 4},
    {
        "id": 55129,
        "state": "Wisconsin",
        "county": "Washburn County",
        "rate": 4.6,
    },
    {
        "id": 55131,
        "state": "Wisconsin",
        "county": "Washington County",
        "rate": 3.4,
    },
    {
        "id": 55133,
        "state": "Wisconsin",
        "county": "Waukesha County",
        "rate": 3.6,
    },
    {
        "id": 55135,
        "state": "Wisconsin",
        "county": "Waupaca County",
        "rate": 3.7,
    },
    {
        "id": 55137,
        "state": "Wisconsin",
        "county": "Waushara County",
        "rate": 4.6,
    },
    {
        "id": 55139,
        "state": "Wisconsin",
        "county": "Winnebago County",
        "rate": 3.7,
    },
    {"id": 55141, "state": "Wisconsin", "county": "Wood County", "rate": 4.7},
    {"id": 56001, "state": "Wyoming", "county": "Albany County", "rate": 3.3},
    {"id": 56003, "state": "Wyoming", "county": "Big Horn County", "rate": 4.4},
    {"id": 56005, "state": "Wyoming", "county": "Campbell County", "rate": 6.8},
    {"id": 56007, "state": "Wyoming", "county": "Carbon County", "rate": 4.1},
    {"id": 56009, "state": "Wyoming", "county": "Converse County", "rate": 6},
    {"id": 56011, "state": "Wyoming", "county": "Crook County", "rate": 4},
    {"id": 56013, "state": "Wyoming", "county": "Fremont County", "rate": 6.6},
    {"id": 56015, "state": "Wyoming", "county": "Goshen County", "rate": 3.3},
    {
        "id": 56017,
        "state": "Wyoming",
        "county": "Hot Springs County",
        "rate": 4.3,
    },
    {"id": 56019, "state": "Wyoming", "county": "Johnson County", "rate": 4.4},
    {"id": 56021, "state": "Wyoming", "county": "Laramie County", "rate": 4},
    {"id": 56023, "state": "Wyoming", "county": "Lincoln County", "rate": 3.8},
    {"id": 56025, "state": "Wyoming", "county": "Natrona County", "rate": 6.8},
    {"id": 56027, "state": "Wyoming", "county": "Niobrara County", "rate": 3.3},
    {"id": 56029, "state": "Wyoming", "county": "Park County", "rate": 3.7},
    {"id": 56031, "state": "Wyoming", "county": "Platte County", "rate": 4.6},
    {"id": 56033, "state": "Wyoming", "county": "Sheridan County", "rate": 4},
    {"id": 56035, "state": "Wyoming", "county": "Sublette County", "rate": 5.4},
    {
        "id": 56037,
        "state": "Wyoming",
        "county": "Sweetwater County",
        "rate": 5.7,
    },
    {"id": 56039, "state": "Wyoming", "county": "Teton County", "rate": 2},
    {"id": 56041, "state": "Wyoming", "county": "Uinta County", "rate": 5.3},
    {"id": 56043, "state": "Wyoming", "county": "Washakie County", "rate": 4.5},
    {"id": 56045, "state": "Wyoming", "county": "Weston County", "rate": 4.9},
    {
        "id": 72001,
        "state": "Puerto Rico",
        "county": "Adjuntas Municipio",
        "rate": 15.7,
    },
    {
        "id": 72003,
        "state": "Puerto Rico",
        "county": "Aguada Municipio",
        "rate": 14.5,
    },
    {
        "id": 72005,
        "state": "Puerto Rico",
        "county": "Aguadilla Municipio",
        "rate": 14.9,
    },
    {
        "id": 72007,
        "state": "Puerto Rico",
        "county": "Aguas Buenas Municipio",
        "rate": 14.5,
    },
    {
        "id": 72009,
        "state": "Puerto Rico",
        "county": "Aibonito Municipio",
        "rate": 15.1,
    },
    {
        "id": 72011,
        "state": "Puerto Rico",
        "county": "Anasco Municipio",
        "rate": 13,
    },
    {
        "id": 72013,
        "state": "Puerto Rico",
        "county": "Arecibo Municipio",
        "rate": 13.7,
    },
    {
        "id": 72015,
        "state": "Puerto Rico",
        "county": "Arroyo Municipio",
        "rate": 18.7,
    },
    {
        "id": 72017,
        "state": "Puerto Rico",
        "county": "Barceloneta Municipio",
        "rate": 15,
    },
    {
        "id": 72019,
        "state": "Puerto Rico",
        "county": "Barranquitas Municipio",
        "rate": 14.1,
    },
    {
        "id": 72021,
        "state": "Puerto Rico",
        "county": "Bayamon Municipio",
        "rate": 9.2,
    },
    {
        "id": 72023,
        "state": "Puerto Rico",
        "county": "Cabo Rojo Municipio",
        "rate": 12.9,
    },
    {
        "id": 72025,
        "state": "Puerto Rico",
        "county": "Caguas Municipio",
        "rate": 10.7,
    },
    {
        "id": 72027,
        "state": "Puerto Rico",
        "county": "Camuy Municipio",
        "rate": 12.8,
    },
    {
        "id": 72029,
        "state": "Puerto Rico",
        "county": "Canovanas Municipio",
        "rate": 12.4,
    },
    {
        "id": 72031,
        "state": "Puerto Rico",
        "county": "Carolina Municipio",
        "rate": 8.8,
    },
    {
        "id": 72033,
        "state": "Puerto Rico",
        "county": "Catano Municipio",
        "rate": 9.9,
    },
    {
        "id": 72035,
        "state": "Puerto Rico",
        "county": "Cayey Municipio",
        "rate": 11.2,
    },
    {
        "id": 72037,
        "state": "Puerto Rico",
        "county": "Ceiba Municipio",
        "rate": 13.4,
    },
    {
        "id": 72039,
        "state": "Puerto Rico",
        "county": "Ciales Municipio",
        "rate": 16.9,
    },
    {
        "id": 72041,
        "state": "Puerto Rico",
        "county": "Cidra Municipio",
        "rate": 10.6,
    },
    {
        "id": 72043,
        "state": "Puerto Rico",
        "county": "Coamo Municipio",
        "rate": 19.5,
    },
    {
        "id": 72045,
        "state": "Puerto Rico",
        "county": "Comerio Municipio",
        "rate": 13.7,
    },
    {
        "id": 72047,
        "state": "Puerto Rico",
        "county": "Corozal Municipio",
        "rate": 12.6,
    },
    {
        "id": 72049,
        "state": "Puerto Rico",
        "county": "Culebra Municipio",
        "rate": 3.8,
    },
    {
        "id": 72051,
        "state": "Puerto Rico",
        "county": "Dorado Municipio",
        "rate": 8.5,
    },
    {
        "id": 72053,
        "state": "Puerto Rico",
        "county": "Fajardo Municipio",
        "rate": 13.8,
    },
    {
        "id": 72054,
        "state": "Puerto Rico",
        "county": "Florida Municipio",
        "rate": 15.4,
    },
    {
        "id": 72055,
        "state": "Puerto Rico",
        "county": "Guanica Municipio",
        "rate": 17.9,
    },
    {
        "id": 72057,
        "state": "Puerto Rico",
        "county": "Guayama Municipio",
        "rate": 17.3,
    },
    {
        "id": 72059,
        "state": "Puerto Rico",
        "county": "Guayanilla Municipio",
        "rate": 18.1,
    },
    {
        "id": 72061,
        "state": "Puerto Rico",
        "county": "Guaynabo Municipio",
        "rate": 6.3,
    },
    {
        "id": 72063,
        "state": "Puerto Rico",
        "county": "Gurabo Municipio",
        "rate": 9.2,
    },
    {
        "id": 72065,
        "state": "Puerto Rico",
        "county": "Hatillo Municipio",
        "rate": 14.1,
    },
    {
        "id": 72067,
        "state": "Puerto Rico",
        "county": "Hormigueros Municipio",
        "rate": 12.7,
    },
    {
        "id": 72069,
        "state": "Puerto Rico",
        "county": "Humacao Municipio",
        "rate": 14.2,
    },
    {
        "id": 72071,
        "state": "Puerto Rico",
        "county": "Isabela Municipio",
        "rate": 14.3,
    },
    {
        "id": 72073,
        "state": "Puerto Rico",
        "county": "Jayuya Municipio",
        "rate": 12.5,
    },
    {
        "id": 72075,
        "state": "Puerto Rico",
        "county": "Juana Diaz Municipio",
        "rate": 15.8,
    },
    {
        "id": 72077,
        "state": "Puerto Rico",
        "county": "Juncos Municipio",
        "rate": 13.4,
    },
    {
        "id": 72079,
        "state": "Puerto Rico",
        "county": "Lajas Municipio",
        "rate": 18.8,
    },
    {
        "id": 72081,
        "state": "Puerto Rico",
        "county": "Lares Municipio",
        "rate": 17.8,
    },
    {
        "id": 72083,
        "state": "Puerto Rico",
        "county": "Las Marias Municipio",
        "rate": 13.7,
    },
    {
        "id": 72085,
        "state": "Puerto Rico",
        "county": "Las Piedras Municipio",
        "rate": 14.7,
    },
    {
        "id": 72087,
        "state": "Puerto Rico",
        "county": "Loiza Municipio",
        "rate": 12.7,
    },
    {
        "id": 72089,
        "state": "Puerto Rico",
        "county": "Luquillo Municipio",
        "rate": 15,
    },
    {
        "id": 72091,
        "state": "Puerto Rico",
        "county": "Manati Municipio",
        "rate": 12.2,
    },
    {
        "id": 72093,
        "state": "Puerto Rico",
        "county": "Maricao Municipio",
        "rate": 13.7,
    },
    {
        "id": 72095,
        "state": "Puerto Rico",
        "county": "Maunabo Municipio",
        "rate": 17.7,
    },
    {
        "id": 72097,
        "state": "Puerto Rico",
        "county": "Mayaguez Municipio",
        "rate": 14.5,
    },
    {
        "id": 72099,
        "state": "Puerto Rico",
        "county": "Moca Municipio",
        "rate": 15.6,
    },
    {
        "id": 72101,
        "state": "Puerto Rico",
        "county": "Morovis Municipio",
        "rate": 14.3,
    },
    {
        "id": 72103,
        "state": "Puerto Rico",
        "county": "Naguabo Municipio",
        "rate": 12.8,
    },
    {
        "id": 72105,
        "state": "Puerto Rico",
        "county": "Naranjito Municipio",
        "rate": 13.6,
    },
    {
        "id": 72107,
        "state": "Puerto Rico",
        "county": "Orocovis Municipio",
        "rate": 15.4,
    },
    {
        "id": 72109,
        "state": "Puerto Rico",
        "county": "Patillas Municipio",
        "rate": 20.6,
    },
    {
        "id": 72111,
        "state": "Puerto Rico",
        "county": "Penuelas Municipio",
        "rate": 16.9,
    },
    {
        "id": 72113,
        "state": "Puerto Rico",
        "county": "Ponce Municipio",
        "rate": 13.8,
    },
    {
        "id": 72115,
        "state": "Puerto Rico",
        "county": "Quebradillas Municipio",
        "rate": 16.1,
    },
    {
        "id": 72117,
        "state": "Puerto Rico",
        "county": "Rincon Municipio",
        "rate": 13.3,
    },
    {
        "id": 72119,
        "state": "Puerto Rico",
        "county": "Rio Grande Municipio",
        "rate": 11.5,
    },
    {
        "id": 72121,
        "state": "Puerto Rico",
        "county": "Sabana Grande Municipio",
        "rate": 16.5,
    },
    {
        "id": 72123,
        "state": "Puerto Rico",
        "county": "Salinas Municipio",
        "rate": 23.4,
    },
    {
        "id": 72125,
        "state": "Puerto Rico",
        "county": "San German Municipio",
        "rate": 15.4,
    },
    {
        "id": 72127,
        "state": "Puerto Rico",
        "county": "San Juan Municipio",
        "rate": 8.2,
    },
    {
        "id": 72129,
        "state": "Puerto Rico",
        "county": "San Lorenzo Municipio",
        "rate": 13.2,
    },
    {
        "id": 72131,
        "state": "Puerto Rico",
        "county": "San Sebastian Municipio",
        "rate": 18.1,
    },
    {
        "id": 72133,
        "state": "Puerto Rico",
        "county": "Santa Isabel Municipio",
        "rate": 20.6,
    },
    {
        "id": 72135,
        "state": "Puerto Rico",
        "county": "Toa Alta Municipio",
        "rate": 8.9,
    },
    {
        "id": 72137,
        "state": "Puerto Rico",
        "county": "Toa Baja Municipio",
        "rate": 9.1,
    },
    {
        "id": 72139,
        "state": "Puerto Rico",
        "county": "Trujillo Alto Municipio",
        "rate": 7.6,
    },
    {
        "id": 72141,
        "state": "Puerto Rico",
        "county": "Utuado Municipio",
        "rate": 15.3,
    },
    {
        "id": 72143,
        "state": "Puerto Rico",
        "county": "Vega Alta Municipio",
        "rate": 12,
    },
    {
        "id": 72145,
        "state": "Puerto Rico",
        "county": "Vega Baja Municipio",
        "rate": 14.3,
    },
    {
        "id": 72147,
        "state": "Puerto Rico",
        "county": "Vieques Municipio",
        "rate": 11.3,
    },
    {
        "id": 72149,
        "state": "Puerto Rico",
        "county": "Villalba Municipio",
        "rate": 19.6,
    },
    {
        "id": 72151,
        "state": "Puerto Rico",
        "county": "Yabucoa Municipio",
        "rate": 16.6,
    },
    {
        "id": 72153,
        "state": "Puerto Rico",
        "county": "Yauco Municipio",
        "rate": 18,
    },
]


class TestGeoPathChart(unittest.TestCase):

    @chart_base_test(chart_type=ChartType.GEOPATH)
    def test_geo_path_base(self):
        c = (
            GeoPath()
            .add_js_dependencies(*["topojson-client"])
            .add_js_funcs(
                JsCode(
                    """
                G2.register('data.feature', ({ name }) => {
                    return (data) => topojson.feature(data, data.objects[name]).features;
                });
                """
                ).js_code
            )
            .set_data(
                data=opts.FetchDataOpts(
                    value="https://assets.antv.antgroup.com/g2/us-10m.json",
                    transform=[
                        opts.FeatureDataTransformOpts(
                            name="counties",
                        ),
                        opts.JoinDataTransformOpts(
                            join=TEST_GEO_PATH_JOIN_DATA,
                            on_=["id", "id"],
                            select_=["rate"],
                        ),
                    ],
                )
            )
            .set_encode(color_field="rate")
            .set_scale(color_scale_opts={"palette": "ylGnBu", "unknown": "#fff"})
            .set_global_options(
                coordinate_opts={"type": "albersUsa"},
                legend_opts={"color": {"layout": {"justifyContent": "center"}}},
            )
        )

        return c
